@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: doxygen.cpp,v 1.87 2001/03/19 19:27:40 root Exp $
+ * $Id: basehandler.h,v 1.87 2004/05/07 12:55:57 dimitri Exp $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -75,11 +75,11 @@ template<class T> class ElementMapper
         Handler m_handler;
     };
 
+  public:
     typedef StartElementHandler StartElementHandlerT;
     typedef EndElementHandler   EndElementHandlerT;
-  
-  public:
-    ElementMapper() : m_startHandlers(67), m_endHandlers(67)
+	  
+	ElementMapper() : m_startHandlers(67), m_endHandlers(67)
     {
       m_startHandlers.setAutoDelete(TRUE);
       m_endHandlers.setAutoDelete(TRUE);
@@ -129,6 +129,9 @@ template<class T> class BaseHandler : public QXmlDefaultHandler,
                                       public IBaseHandler
 {
   public:
+    typedef typename ElementMapper<T>::StartElementHandlerT StartElementHandlerT;
+    typedef typename ElementMapper<T>::EndElementHandlerT EndElementHandlerT;
+
     BaseHandler() : m_delegateHandler(0), m_fallBackHandler(0)
     {
     }
@@ -285,12 +288,16 @@ template<class T> class BaseFallBackHandler : public ElementMapper<T>,
                                               public IFallBackHandler
 {
   public:
+    typedef typename ElementMapper<T>::StartElementHandlerT StartElementHandlerT;
+    typedef typename ElementMapper<T>::EndElementHandlerT EndElementHandlerT;
+
     BaseFallBackHandler() 
     {
     }
     virtual ~BaseFallBackHandler()
     {
     }
+    
     bool handleStartElement(const QString & name, 
                                     const QXmlAttributes & attrib)
     {
