@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * $Id: dochandler.cpp,v 1.87 2004/05/07 12:55:58 dimitri Exp $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -13,9 +13,12 @@
  *
  */
 
+#include <qmap.h>
+
 #include "dochandler.h"
 #include "debug.h"
-#include <qmap.h>
+#include "linkedtexthandler.h"
+
 
 //----------------------------------------------------------------------
 
@@ -95,7 +98,8 @@ void dochandler_exit()
 //----------------------------------------------------------------------
 
 MarkupHandler::MarkupHandler(QList<DocImpl> &children,QString &curString)
-  : m_children(children), m_curString(curString), m_curMarkup(IDocMarkup::Normal)
+  : m_children(children), m_curString(curString), 
+    m_curMarkup(IDocMarkup::Normal), m_headingLevel(0)
 {
   addStartHandler("bold",this,&MarkupHandler::startBold);
   addEndHandler("bold",this,&MarkupHandler::endBold);
@@ -117,6 +121,27 @@ MarkupHandler::MarkupHandler(QList<DocImpl> &children,QString &curString)
 
   addStartHandler("superscript",this,&MarkupHandler::startSuperscript);
   addEndHandler("superscript",this,&MarkupHandler::endSuperscript);
+
+  addStartHandler("preformatted",this,&MarkupHandler::startPreformatted);
+  addEndHandler("preformatted",this,&MarkupHandler::endPreformatted);
+
+  addStartHandler("heading1",this,&MarkupHandler::startHeading1);
+  addEndHandler("heading1",this,&MarkupHandler::endHeading1);
+  
+  addStartHandler("heading2",this,&MarkupHandler::startHeading2);
+  addEndHandler("heading2",this,&MarkupHandler::endHeading2);
+
+  addStartHandler("heading3",this,&MarkupHandler::startHeading3);
+  addEndHandler("heading3",this,&MarkupHandler::endHeading3);
+
+  addStartHandler("heading4",this,&MarkupHandler::startHeading4);
+  addEndHandler("heading4",this,&MarkupHandler::endHeading4);
+
+  addStartHandler("heading5",this,&MarkupHandler::startHeading5);
+  addEndHandler("heading5",this,&MarkupHandler::endHeading5);
+
+  addStartHandler("heading6",this,&MarkupHandler::startHeading6);
+  addEndHandler("heading6",this,&MarkupHandler::endHeading6);
 }
 
 MarkupHandler::~MarkupHandler()
@@ -127,7 +152,7 @@ void MarkupHandler::addTextNode()
 {
   if (!m_curString.isEmpty())
   {
-    m_children.append(new TextNode(m_curString,m_curMarkup));
+    m_children.append(new TextNode(m_curString,m_curMarkup,m_headingLevel));
     debug(2,"addTextNode() text=%s markup=%x\n",m_curString.data(),m_curMarkup);
     m_curString="";
   }
@@ -231,6 +256,117 @@ void MarkupHandler::endSuperscript()
   m_curMarkup &= ~IDocMarkup::Superscript;
 }
 
+void MarkupHandler::startPreformatted(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Preformatted,TRUE));
+  m_curMarkup |= IDocMarkup::Preformatted;
+}
+
+void MarkupHandler::endPreformatted()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Preformatted,FALSE));
+  m_curMarkup &= ~IDocMarkup::Preformatted;
+}
+
+void MarkupHandler::startHeading1(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,TRUE,1));
+  m_curMarkup |= IDocMarkup::Heading;
+  m_headingLevel=1;
+}
+
+void MarkupHandler::endHeading1()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,FALSE,1));
+  m_curMarkup &= ~IDocMarkup::Heading;
+  m_headingLevel=0;
+}
+
+void MarkupHandler::startHeading2(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,TRUE,2));
+  m_curMarkup |= IDocMarkup::Heading;
+  m_headingLevel=2;
+}
+
+void MarkupHandler::endHeading2()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,FALSE,2));
+  m_curMarkup &= ~IDocMarkup::Heading;
+  m_headingLevel=0;
+}
+
+void MarkupHandler::startHeading3(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,TRUE,3));
+  m_curMarkup |= IDocMarkup::Heading;
+  m_headingLevel=3;
+}
+
+void MarkupHandler::endHeading3()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,FALSE,3));
+  m_curMarkup &= ~IDocMarkup::Heading;
+  m_headingLevel=0;
+}
+
+void MarkupHandler::startHeading4(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,TRUE,4));
+  m_curMarkup |= IDocMarkup::Heading;
+  m_headingLevel=4;
+}
+
+void MarkupHandler::endHeading4()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,FALSE,4));
+  m_curMarkup &= ~IDocMarkup::Heading;
+  m_headingLevel=0;
+}
+
+void MarkupHandler::startHeading5(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,TRUE,5));
+  m_curMarkup |= IDocMarkup::Heading;
+  m_headingLevel=5;
+}
+
+void MarkupHandler::endHeading5()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,FALSE,5));
+  m_curMarkup &= ~IDocMarkup::Heading;
+  m_headingLevel=0;
+}
+
+void MarkupHandler::startHeading6(const QXmlAttributes & /*attrib*/)
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,TRUE,6));
+  m_curMarkup |= IDocMarkup::Heading;
+  m_headingLevel=6;
+}
+
+void MarkupHandler::endHeading6()
+{
+  addTextNode();
+  m_children.append(new MarkupModifierNode(IDocMarkup::Heading,FALSE,6));
+  m_curMarkup &= ~IDocMarkup::Heading;
+  m_headingLevel=0;
+}
+
+
 //----------------------------------------------------------------------
 // ListItemHandler
 //----------------------------------------------------------------------
@@ -347,6 +483,69 @@ IDocIterator *ItemizedListHandler::elements() const
   return new ItemizedListIterator(*this);
 }
 
+//----------------------------------------------------------------------
+// TocListHandler
+//----------------------------------------------------------------------
+
+TocListHandler::TocListHandler(IBaseHandler *parent) : m_parent(parent)
+{
+  m_children.setAutoDelete(TRUE);
+  addEndHandler("toclist",this,&TocListHandler::endTocList);
+  addStartHandler("tocitem",this,&TocListHandler::startTocItem);
+}
+
+TocListHandler::~TocListHandler()
+{
+}
+
+void TocListHandler::startTocList(const QXmlAttributes& /*attrib*/)
+{
+  m_parent->setDelegate(this);
+}
+
+void TocListHandler::endTocList()
+{
+  m_parent->setDelegate(0);
+}
+
+void TocListHandler::startTocItem(const QXmlAttributes& attrib)
+{
+  TocItemHandler *tiHandler = new TocItemHandler(this);
+  tiHandler->startTocItem(attrib);
+  m_children.append(tiHandler);
+}
+
+IDocIterator *TocListHandler::elements() const
+{
+  return new TocListIterator(*this);
+}
+
+//----------------------------------------------------------------------
+// TocItemHandler
+//----------------------------------------------------------------------
+
+TocItemHandler::TocItemHandler(IBaseHandler *parent) : m_parent(parent)
+{
+  addEndHandler("tocitem",this,&TocItemHandler::endTocItem);
+}
+
+TocItemHandler::~TocItemHandler()
+{
+}
+
+void TocItemHandler::startTocItem(const QXmlAttributes& attrib)
+{
+  m_parent->setDelegate(this);
+  m_id = attrib.value("id");
+  m_curString="";
+}
+
+void TocItemHandler::endTocItem()
+{
+  m_title = m_curString;
+  m_parent->setDelegate(0);
+}
+
 //----------------------------------------------------------------------
 // ParameterHandler
 //----------------------------------------------------------------------
@@ -610,7 +809,7 @@ void RefHandler::startRef(const QXmlAttributes& attrib)
 {
   m_parent->setDelegate(this);
   m_refId = attrib.value("refid");
-  m_refId = attrib.value("external");
+  m_extId = attrib.value("external");
   ASSERT(attrib.value("kindref")=="compound" || 
          attrib.value("kindref")=="member");
   m_targetKind = attrib.value("kindref")=="compound" ?  Compound : Member;
@@ -663,9 +862,15 @@ void TitleHandler::addTextNode()
 {
   if (!m_curString.isEmpty())
   {
-    m_children.append(new TextNode(m_curString,m_markupHandler->markup()));
-    debug(2,"addTextNode() text=\"%s\" markup=%x\n",
-        m_curString.data(),m_markupHandler->markup());
+    m_children.append(
+                      new TextNode(
+                                   m_curString,
+                                   m_markupHandler->markup(),
+                                   m_markupHandler->headingLevel()
+                                  )
+                     );
+    debug(2,"addTextNode() text=\"%s\" markup=%x headingLevel=%d\n",
+        m_curString.data(),m_markupHandler->markup(),m_markupHandler->headingLevel());
     m_curString="";
   }
 }
@@ -731,7 +936,7 @@ void SimpleSectHandler::startParagraph(const QXmlAttributes& attrib)
 //----------------------------------------------------------------------
 
 VariableListEntryHandler::VariableListEntryHandler(IBaseHandler *parent)
-   : m_parent(parent), m_description(0)
+   : m_parent(parent), m_description(0), m_linkedTextHandler(0)
 {
   addStartHandler("term",this,&VariableListEntryHandler::startTerm);
   addEndHandler("term",this,&VariableListEntryHandler::endTerm);
@@ -772,12 +977,13 @@ void VariableListEntryHandler::endListItem()
 void VariableListEntryHandler::startTerm(const QXmlAttributes& /*attrib*/)
 {
   m_curString="";
+  m_linkedTextHandler = new LinkedTextHandler(this,m_term);
+  m_linkedTextHandler->start("term");
 }
 
 void VariableListEntryHandler::endTerm()
 {
-  m_term = m_curString;
-  debug(2,"term=%s\n",m_term.data());
+  delete m_linkedTextHandler;
 }
 
 void VariableListEntryHandler::startParagraph(const QXmlAttributes& attrib)
@@ -787,6 +993,10 @@ void VariableListEntryHandler::startParagraph(const QXmlAttributes& attrib)
   m_description->startParagraph(attrib);
 }
 
+ILinkedTextIterator *VariableListEntryHandler::term() const 
+{ 
+  return new LinkedTextIterator(m_term); 
+}
 
 
 //----------------------------------------------------------------------
@@ -847,6 +1057,7 @@ HighlightHandler::HighlightHandler(IBaseHandler *parent)
   m_children.setAutoDelete(TRUE);
   addEndHandler("highlight",this,&HighlightHandler::endHighlight);
   addStartHandler("ref",this,&HighlightHandler::startRef);
+  addStartHandler("sp",this,&HighlightHandler::startSpace);
   m_hl = IDocHighlight::Invalid;
 }
 
@@ -860,12 +1071,13 @@ void HighlightHandler::startHighlight(const QXmlAttributes& attrib)
   m_hl = s_highlightMapper->stringToKind(m_hlString);
   m_curString="";
   m_parent->setDelegate(this);
+  debug(2,"start highlight\n");
 }
 
 void HighlightHandler::endHighlight()
 {
   addTextNode();
-  debug(2,"highlight class=`%s'\n",m_hlString.data());
+  debug(2,"end highlight class=`%s'\n",m_hlString.data());
   m_parent->setDelegate(0);
 }
 
@@ -877,11 +1089,18 @@ void HighlightHandler::startRef(const QXmlAttributes& attrib)
   rh->startRef(attrib);
 }
 
+void HighlightHandler::startSpace(const QXmlAttributes&)
+{
+  m_curString=" ";
+  addTextNode();
+}
+
 void HighlightHandler::addTextNode()
 {
+  printf("m_curString=`%s'\n",m_curString.data());
   if (!m_curString.isEmpty())
   {
-    m_children.append(new TextNode(m_curString,IDocMarkup::Normal));
+    m_children.append(new TextNode(m_curString,IDocMarkup::Normal,0));
     debug(2,"addTextNode() text=\"%s\"\n",
         m_curString.data());
     m_curString="";
@@ -958,7 +1177,7 @@ void CodeLineHandler::addTextNode()
 {
   if (!m_curString.isEmpty())
   {
-    m_children.append(new TextNode(m_curString,IDocMarkup::Normal));
+    m_children.append(new TextNode(m_curString,IDocMarkup::Normal,0));
     debug(2,"addTextNode() text=\"%s\"\n",
         m_curString.data());
     m_curString="";
@@ -1062,6 +1281,32 @@ void FormulaHandler::endFormula()
   m_parent->setDelegate(0);
 }
 
+//----------------------------------------------------------------------
+// AnchorHandler
+//----------------------------------------------------------------------
+
+AnchorHandler::AnchorHandler(IBaseHandler *parent)
+  : m_parent(parent)
+{
+  addEndHandler("anchor",this,&AnchorHandler::endAnchor);
+}
+
+AnchorHandler::~AnchorHandler()
+{
+}
+
+void AnchorHandler::startAnchor(const QXmlAttributes& attrib)
+{
+  m_id = attrib.value("id");
+  m_parent->setDelegate(this);
+}
+
+void AnchorHandler::endAnchor()
+{
+  debug(2,"anchor id=`%s'\n",m_id.data());
+  m_parent->setDelegate(0);
+}
+
 //----------------------------------------------------------------------
 // ImageHandler
 //----------------------------------------------------------------------
@@ -1097,7 +1342,7 @@ void ImageHandler::endImage()
 DotFileHandler::DotFileHandler(IBaseHandler *parent)
   : m_parent(parent)
 {
-  addEndHandler("image",this,&DotFileHandler::endDotFile);
+  addEndHandler("dotfile",this,&DotFileHandler::endDotFile);
 }
 
 DotFileHandler::~DotFileHandler()
@@ -1297,6 +1542,111 @@ IDocIterator *TableHandler::rows() const
   return new TableIterator(*this);
 }
 
+//----------------------------------------------------------------------
+// CopyHandler
+//----------------------------------------------------------------------
+
+CopyHandler::CopyHandler(IBaseHandler *parent) 
+  : m_parent(parent)
+{
+  m_children.setAutoDelete(TRUE);
+  
+  addEndHandler("copydoc",this,&CopyHandler::endCopy);
+
+  addStartHandler("para",this,&CopyHandler::startParagraph);
+}
+
+CopyHandler::~CopyHandler()
+{
+}
+
+void CopyHandler::startCopy(const QXmlAttributes& /*attrib*/)
+{
+  m_parent->setDelegate(this);
+  debug(2,"start copy handler\n");
+}
+
+void CopyHandler::endCopy()
+{
+  debug(2,"end copy handler\n");
+  m_parent->setDelegate(0);
+}
+
+void CopyHandler::startParagraph(const QXmlAttributes& attrib)
+{
+  ParagraphHandler *parHandler = new ParagraphHandler(this);
+  parHandler->startParagraph(attrib);
+  m_children.append(parHandler);
+}
+
+IDocIterator *CopyHandler::contents() const
+{
+  return new CopyIterator(*this);
+}
+
+//----------------------------------------------------------------------
+// VerbatimHandler
+//----------------------------------------------------------------------
+
+VerbatimHandler::VerbatimHandler(IBaseHandler *parent)
+  :  m_parent(parent), m_type(IDocVerbatim::Invalid)
+{
+  addEndHandler("verbatim",this,&VerbatimHandler::endVerbatim);
+  addEndHandler("latexonly",this,&VerbatimHandler::endVerbatim);
+  addEndHandler("htmlonly",this,&VerbatimHandler::endVerbatim);
+}
+
+VerbatimHandler::~VerbatimHandler()
+{
+}
+
+void VerbatimHandler::startVerbatim(const QXmlAttributes&,Types type)
+{
+  m_type = type;
+  m_parent->setDelegate(this);
+  m_curString="";
+}
+
+void VerbatimHandler::endVerbatim()
+{
+  m_text = m_curString;
+  m_parent->setDelegate(0);
+}
+
+
+//----------------------------------------------------------------------
+// SymbolHandler
+//----------------------------------------------------------------------
+
+SymbolHandler::SymbolHandler(IBaseHandler *parent,Types type)
+  :  m_parent(parent), m_letter('\0'), m_type(type)
+{
+  addEndHandler("symbol");
+  switch (type)
+  {
+    case IDocSymbol::Invalid: m_typeString="invalid"; break;
+    case IDocSymbol::Umlaut:  m_typeString="umlaut"; break;
+    case IDocSymbol::Acute:   m_typeString="acute"; break;
+    case IDocSymbol::Grave:   m_typeString="grave"; break;
+    case IDocSymbol::Circ:    m_typeString="circ"; break;
+    case IDocSymbol::Tilde:   m_typeString="tilde"; break;
+    case IDocSymbol::Szlig:   m_typeString="szlig"; break;
+    case IDocSymbol::Cedil:   m_typeString="cedil"; break;
+    case IDocSymbol::Ring:    m_typeString="ring"; break;
+    case IDocSymbol::Nbsp:    m_typeString="nbsp"; break;
+    case IDocSymbol::Copy:    m_typeString="copy"; break;
+  }
+}
+
+SymbolHandler::~SymbolHandler()
+{
+}
+
+void SymbolHandler::startSymbol(const QXmlAttributes& attrib)
+{
+  QString ls = attrib.value("char");
+  if (!ls.isEmpty()) m_letter = ls.latin1()[0];
+}
 
 //----------------------------------------------------------------------
 // ParagraphHandler
@@ -1329,6 +1679,22 @@ ParagraphHandler::ParagraphHandler(IBaseHandler *parent)
   addStartHandler("dotfile",this,&ParagraphHandler::startDotFile);
   addStartHandler("indexentry",this,&ParagraphHandler::startIndexEntry);
   addStartHandler("table",this,&ParagraphHandler::startTable);
+  addStartHandler("verbatim",this,&ParagraphHandler::startVerbatim);
+  addStartHandler("latexonly",this,&ParagraphHandler::startHtmlOnly);
+  addStartHandler("htmlonly",this,&ParagraphHandler::startLatexOnly);
+  addStartHandler("umlaut",this,&ParagraphHandler::startUmlaut);
+  addStartHandler("acute",this,&ParagraphHandler::startAcute);
+  addStartHandler("grave",this,&ParagraphHandler::startGrave);
+  addStartHandler("circ",this,&ParagraphHandler::startCirc);
+  addStartHandler("tilde",this,&ParagraphHandler::startTilde);
+  addStartHandler("szlig",this,&ParagraphHandler::startSzlig);
+  addStartHandler("cedil",this,&ParagraphHandler::startCedil);
+  addStartHandler("ring",this,&ParagraphHandler::startRing);
+  addStartHandler("nbsp",this,&ParagraphHandler::startNbsp);
+  addStartHandler("copy",this,&ParagraphHandler::startCopy);
+  addStartHandler("anchor",this,&ParagraphHandler::startAnchor);
+  addStartHandler("copydoc",this,&ParagraphHandler::startCopyDoc);
+  addStartHandler("toclist",this,&ParagraphHandler::startTocList);
 }
 
 ParagraphHandler::~ParagraphHandler()
@@ -1485,13 +1851,147 @@ void ParagraphHandler::startTable(const QXmlAttributes& attrib)
   m_children.append(th);
 }
 
+void ParagraphHandler::startVerbatim(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  VerbatimHandler *vh = new VerbatimHandler(this);
+  vh->startVerbatim(attrib,IDocVerbatim::Verbatim);
+  m_children.append(vh);
+}
+
+void ParagraphHandler::startHtmlOnly(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  VerbatimHandler *vh = new VerbatimHandler(this);
+  vh->startVerbatim(attrib,IDocVerbatim::HtmlOnly);
+  m_children.append(vh);
+}
+
+void ParagraphHandler::startLatexOnly(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  VerbatimHandler *vh = new VerbatimHandler(this);
+  vh->startVerbatim(attrib,IDocVerbatim::LatexOnly);
+  m_children.append(vh);
+}
+
+void ParagraphHandler::startUmlaut(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Umlaut);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startAcute(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Acute);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startGrave(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Grave);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startCirc(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Circ);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startTilde(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Tilde);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startSzlig(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Szlig);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startCedil(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Cedil);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startRing(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Ring);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startNbsp(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Nbsp);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startCopy(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  SymbolHandler *sh = new SymbolHandler(this,IDocSymbol::Copy);
+  sh->startSymbol(attrib);
+  m_children.append(sh);
+}
+
+void ParagraphHandler::startAnchor(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  AnchorHandler *ah = new AnchorHandler(this);
+  ah->startAnchor(attrib);
+  m_children.append(ah);
+}
+
+void ParagraphHandler::startCopyDoc(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  CopyHandler *ch = new CopyHandler(this);
+  ch->startCopy(attrib);
+  m_children.append(ch);
+}
+
+void ParagraphHandler::startTocList(const QXmlAttributes& attrib)
+{
+  addTextNode();
+  TocListHandler *th = new TocListHandler(this);
+  th->startTocList(attrib);
+  m_children.append(th);
+}
+
 void ParagraphHandler::addTextNode()
 {
   if (!m_curString.isEmpty())
   {
-    m_children.append(new TextNode(m_curString,m_markupHandler->markup()));
-    debug(2,"addTextNode() text=\"%s\" markup=%x\n",
-        m_curString.data(),m_markupHandler->markup());
+    m_children.append(
+                      new TextNode(
+                                   m_curString,
+                                   m_markupHandler->markup(),
+                                   m_markupHandler->headingLevel()
+                                  )
+                     );
+    debug(2,"addTextNode() text=\"%s\" markup=%x headingLevel=%d\n",
+        m_curString.data(),m_markupHandler->markup(),m_markupHandler->headingLevel());
     m_curString="";
   }
 }
@@ -1506,20 +2006,25 @@ IDocIterator *ParagraphHandler::contents() const
 //----------------------------------------------------------------------
 
 DocSectionHandler::DocSectionHandler(IBaseHandler *parent,int level)
-  : m_parent(parent), m_level(level)
+  : m_parent(parent), m_internal(0), m_level(level), m_title(0)
 {
-  m_children.setAutoDelete(TRUE);
-  m_markupHandler = new MarkupHandler(m_children,m_curString);
-  setFallBackHandler(m_markupHandler);
-  addStartHandler("ref",this,&DocSectionHandler::startRef);
   QString sectionKey;
+  m_paragraphs.setAutoDelete(TRUE);
+  m_subsections.setAutoDelete(TRUE);
+  addStartHandler("title",this,&DocSectionHandler::startTitle);
+  addStartHandler("para",this,&DocSectionHandler::startParagraph);
+  if (level<6) 
+  {
+    sectionKey.sprintf("sect%d",level+1);
+    addStartHandler(sectionKey,this,&DocSectionHandler::startSubSection);
+  }
+  addStartHandler("internal",this,&DocSectionHandler::startInternal);
   sectionKey.sprintf("sect%d",level);
   addEndHandler(sectionKey,this,&DocSectionHandler::endDocSection);
 }
 
 DocSectionHandler::~DocSectionHandler()
 {
-  delete m_markupHandler;
 }
 
 void DocSectionHandler::startDocSection(const QXmlAttributes& attrib)
@@ -1527,39 +2032,112 @@ void DocSectionHandler::startDocSection(const QXmlAttributes& attrib)
   m_parent->setDelegate(this);
   debug(2,"Start docsection\n");
   m_id = attrib.value("id");
-  m_curString="";
 }
 
 void DocSectionHandler::endDocSection()
 {
-  addTextNode();
   m_parent->setDelegate(0);
   debug(2,"End docsection\n");
 }
 
-void DocSectionHandler::addTextNode()
+void DocSectionHandler::startSubSection(const QXmlAttributes& attrib)
 {
-  if (!m_curString.isEmpty())
-  {
-    m_children.append(new TextNode(m_curString,m_markupHandler->markup()));
-    debug(2,"addTextNode() text=\"%s\" markup=%x\n",
-        m_curString.data(),m_markupHandler->markup());
-    m_curString="";
-  }
+  DocSectionHandler *secHandler = new DocSectionHandler(this,m_level+1);
+  secHandler->startDocSection(attrib);
+  m_subsections.append(secHandler);
 }
 
-void DocSectionHandler::startRef(const QXmlAttributes& attrib)
+void DocSectionHandler::startParagraph(const QXmlAttributes& attrib)
 {
-  RefHandler *ref = new RefHandler(this);
-  ref->startRef(attrib);
-  m_children.append(ref);
+  ParagraphHandler *parHandler = new ParagraphHandler(this);
+  parHandler->startParagraph(attrib);
+  m_paragraphs.append(parHandler);
 }
 
-IDocIterator *DocSectionHandler::title() const
+void DocSectionHandler::startInternal(const QXmlAttributes& attrib)
 {
-  return new DocSectionIterator(*this);
+  m_internal = new DocInternalHandler(this,m_level);
+  m_internal->startInternal(attrib);
 }
 
+void DocSectionHandler::startTitle(const QXmlAttributes& attrib)
+{
+  m_title = new TitleHandler(this);
+  m_title->startTitle(attrib);
+}
+
+IDocIterator *DocSectionHandler::paragraphs() const
+{
+  return new DocSectionParaIterator(*this);
+}
+
+IDocIterator *DocSectionHandler::subSections() const
+{
+  return new DocSectionSubIterator(*this);
+}
+
+IDocInternal *DocSectionHandler::internal() const 
+{ 
+  return m_internal; 
+}
+
+//----------------------------------------------------------------------
+// DocInternal
+//----------------------------------------------------------------------
+
+DocInternalHandler::DocInternalHandler(IBaseHandler *parent,int level)
+  : m_parent(parent), m_level(level)
+{
+  m_paragraphs.setAutoDelete(TRUE);
+  m_subsections.setAutoDelete(TRUE);
+  addStartHandler("para",this,&DocInternalHandler::startParagraph);
+  QString sectionKey;
+  sectionKey.sprintf("sect%d",level+1);
+  addStartHandler(sectionKey,this,&DocInternalHandler::startSubSection);
+  addEndHandler("internal",this,&DocInternalHandler::endInternal);
+}
+
+DocInternalHandler::~DocInternalHandler()
+{
+}
+
+void DocInternalHandler::startInternal(const QXmlAttributes&)
+{
+  m_parent->setDelegate(this);
+  debug(2,"Start internal\n");
+}
+
+void DocInternalHandler::endInternal()
+{
+  m_parent->setDelegate(0);
+  debug(2,"End internal\n");
+}
+
+void DocInternalHandler::startSubSection(const QXmlAttributes& attrib)
+{
+  DocSectionHandler *secHandler = new DocSectionHandler(this,m_level+1);
+  secHandler->startDocSection(attrib);
+  m_subsections.append(secHandler);
+}
+
+void DocInternalHandler::startParagraph(const QXmlAttributes& attrib)
+{
+  ParagraphHandler *parHandler = new ParagraphHandler(this);
+  parHandler->startParagraph(attrib);
+  m_paragraphs.append(parHandler);
+}
+
+IDocIterator *DocInternalHandler::paragraphs() const
+{
+  return new DocInternalParaIterator(*this);
+}
+
+IDocIterator *DocInternalHandler::subSections() const
+{
+  return new DocInternalSubIterator(*this);
+}
+
+
 //----------------------------------------------------------------------
 // DocHandler
 //----------------------------------------------------------------------
@@ -1570,12 +2148,12 @@ DocHandler::DocHandler(IBaseHandler *parent) : m_parent(parent)
 
   addEndHandler("briefdescription",this,&DocHandler::endDoc);
   addEndHandler("detaileddescription",this,&DocHandler::endDoc);
+  addEndHandler("internal");
 
   addStartHandler("para",this,&DocHandler::startParagraph);
   addStartHandler("sect1",this,&DocHandler::startSect1);
-  addStartHandler("sect2",this,&DocHandler::startSect2);
-  addStartHandler("sect3",this,&DocHandler::startSect3);
   addStartHandler("title",this,&DocHandler::startTitle);
+  addStartHandler("internal");
 }
 
 DocHandler::~DocHandler()
@@ -1608,20 +2186,6 @@ void DocHandler::startSect1(const QXmlAttributes& attrib)
   m_children.append(secHandler);
 }
 
-void DocHandler::startSect2(const QXmlAttributes& attrib)
-{
-  DocSectionHandler *secHandler = new DocSectionHandler(this,2);
-  secHandler->startDocSection(attrib);
-  m_children.append(secHandler);
-}
-
-void DocHandler::startSect3(const QXmlAttributes& attrib)
-{
-  DocSectionHandler *secHandler = new DocSectionHandler(this,3);
-  secHandler->startDocSection(attrib);
-  m_children.append(secHandler);
-}
-
 void DocHandler::startTitle(const QXmlAttributes& attrib)
 {
   TitleHandler *titleHandler = new TitleHandler(this);
