@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id:$
+ * $Id: dochandler.h,v 1.87 2004/05/07 12:55:58 dimitri Exp $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,10 +21,14 @@
 #include <qxml.h>
 
 #include <doxmlintf.h>
+#include "stringimpl.h"
 #include "basehandler.h"
 #include "baseiterator.h"
 
 class ParagraphHandler;
+class DocInternalHandler;
+class LinkedTextImpl;
+class LinkedTextHandler;
 
 //-----------------------------------------------------------------------------
 
@@ -63,6 +67,13 @@ DEFINE_CLS_IMPL(DocTable);
 DEFINE_CLS_IMPL(DocRow);
 DEFINE_CLS_IMPL(DocEntry);
 DEFINE_CLS_IMPL(DocSection);
+DEFINE_CLS_IMPL(DocVerbatim);
+DEFINE_CLS_IMPL(DocCopy);
+DEFINE_CLS_IMPL(DocTocList);
+DEFINE_CLS_IMPL(DocTocItem);
+DEFINE_CLS_IMPL(DocAnchor);
+DEFINE_CLS_IMPL(DocSymbol);
+DEFINE_CLS_IMPL(DocInternal);
 DEFINE_CLS_IMPL(DocRoot);
 
 //-----------------------------------------------------------------------------
@@ -74,18 +85,20 @@ DEFINE_CLS_IMPL(DocRoot);
 class TextNode : public DocTextImpl
 {
   public:
-    TextNode(const QString &t,int markup) 
-      : m_text(t), m_markup(markup) {}
+    TextNode(const QString &t,int markup,int level) 
+      : m_text(t), m_markup(markup), m_headingLevel(level) {}
     virtual ~TextNode() {}
 
     // IDocText
     virtual Kind kind() const { return DocImpl::Text; }
-    virtual QString text() const { return m_text; }
+    virtual const IString *text() const { return &m_text; }
     virtual int markup() const { return m_markup; }
+    virtual int headingLevel() const { return m_headingLevel; }
   
   private:  
-    QString m_text;
+    StringImpl m_text;
     int m_markup;
+    int m_headingLevel;
 };
 
 //-----------------------------------------------------------------------------
@@ -96,18 +109,20 @@ class TextNode : public DocTextImpl
 class MarkupModifierNode : public DocMarkupModifierImpl
 {
   public:
-    MarkupModifierNode(int markup,bool enabled) 
-      : m_markup(markup), m_enabled(enabled) {}
+    MarkupModifierNode(int markup,bool enabled,int level=0) 
+      : m_markup(markup), m_enabled(enabled), m_headingLevel(level) {}
     virtual ~MarkupModifierNode() {}
   
     // IDocMarkupModifier
     virtual Kind kind() const { return DocImpl::MarkupModifier; }
     virtual bool enabled() const { return m_enabled; }
     virtual int markup() const { return m_markup; }
+    virtual int headingLevel() const { return m_headingLevel; }
 
   private:
     int m_markup;
     bool m_enabled;
+    int m_headingLevel;
 };
 
 
@@ -122,6 +137,7 @@ class MarkupHandler : public BaseFallBackHandler<MarkupHandler>
     MarkupHandler(QList<DocImpl> &children,QString &curString);
     virtual ~MarkupHandler();
     int markup() const { return m_curMarkup; }
+    int headingLevel() const { return m_headingLevel; }
 
     virtual void startBold(const QXmlAttributes &attrib);
     virtual void endBold();
@@ -137,6 +153,20 @@ class MarkupHandler : public BaseFallBackHandler<MarkupHandler>
     virtual void endSubscript();
     virtual void startSuperscript(const QXmlAttributes &attrib);
     virtual void endSuperscript();
+    virtual void startPreformatted(const QXmlAttributes &attrib);
+    virtual void endPreformatted();
+    virtual void startHeading1(const QXmlAttributes &attrib);
+    virtual void endHeading1();
+    virtual void startHeading2(const QXmlAttributes &attrib);
+    virtual void endHeading2();
+    virtual void startHeading3(const QXmlAttributes &attrib);
+    virtual void endHeading3();
+    virtual void startHeading4(const QXmlAttributes &attrib);
+    virtual void endHeading4();
+    virtual void startHeading5(const QXmlAttributes &attrib);
+    virtual void endHeading5();
+    virtual void startHeading6(const QXmlAttributes &attrib);
+    virtual void endHeading6();
 
 
   private:
@@ -145,6 +175,7 @@ class MarkupHandler : public BaseFallBackHandler<MarkupHandler>
     QList<DocImpl>  &m_children;
     QString         &m_curString;
     int             m_curMarkup;
+    int             m_headingLevel;
 };
 
 //-----------------------------------------------------------------------------
@@ -186,6 +217,22 @@ class ParagraphHandler : public DocParaImpl,
     virtual void startDotFile(const QXmlAttributes& attrib);
     virtual void startIndexEntry(const QXmlAttributes& attrib);
     virtual void startTable(const QXmlAttributes& attrib);
+    virtual void startVerbatim(const QXmlAttributes& attrib);
+    virtual void startHtmlOnly(const QXmlAttributes& attrib);
+    virtual void startLatexOnly(const QXmlAttributes& attrib);
+    virtual void startUmlaut(const QXmlAttributes& attrib);
+    virtual void startAcute(const QXmlAttributes& attrib);
+    virtual void startGrave(const QXmlAttributes& attrib);
+    virtual void startCirc(const QXmlAttributes& attrib);
+    virtual void startTilde(const QXmlAttributes& attrib);
+    virtual void startSzlig(const QXmlAttributes& attrib);
+    virtual void startCedil(const QXmlAttributes& attrib);
+    virtual void startRing(const QXmlAttributes& attrib);
+    virtual void startNbsp(const QXmlAttributes& attrib);
+    virtual void startCopy(const QXmlAttributes& attrib);
+    virtual void startAnchor(const QXmlAttributes& attrib);
+    virtual void startCopyDoc(const QXmlAttributes& attrib);
+    virtual void startTocList(const QXmlAttributes& attrib);
 
     ParagraphHandler(IBaseHandler *parent);
     virtual ~ParagraphHandler();
@@ -271,6 +318,61 @@ class OrderedListIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,Doc
       BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_children) {}
 };
 
+//-----------------------------------------------------------------------------
+
+/*! \brief Node representing table of contents list.
+ *
+ */
+class TocListHandler : public DocTocListImpl, public BaseHandler<TocListHandler>
+{
+    friend class TocListIterator;
+  public:
+    TocListHandler(IBaseHandler *parent);
+    virtual ~TocListHandler();
+    virtual void startTocList(const QXmlAttributes& attrib);
+    virtual void endTocList();
+    virtual void startTocItem(const QXmlAttributes& attrib);
+
+    // IDocTocList
+    virtual Kind kind() const { return DocImpl::TocList; }
+    virtual IDocIterator *elements() const;
+
+  private:
+    IBaseHandler   *m_parent;
+    QList<DocImpl>  m_children;
+};
+
+class TocListIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
+{
+  public:
+    TocListIterator(const TocListHandler &handler) : 
+      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_children) {}
+};
+
+//-----------------------------------------------------------------------------
+
+/*! \brief Node representing a table of contents item.
+ *
+ */
+class TocItemHandler : public DocTocItemImpl, public BaseHandler<TocItemHandler>
+{
+    friend class TocItemIterator;
+  public:
+    TocItemHandler(IBaseHandler *parent);
+    virtual ~TocItemHandler();
+    virtual void startTocItem(const QXmlAttributes& attrib);
+    virtual void endTocItem();
+
+    // IDocItem
+    virtual Kind kind() const { return DocImpl::TocItem; }
+    virtual const IString *id() const { return &m_id; }
+    virtual const IString *title() const { return &m_title; }
+
+  private:
+    IBaseHandler   *m_parent;
+    StringImpl      m_id;
+    StringImpl      m_title;
+};
 
 //-----------------------------------------------------------------------------
 
@@ -322,12 +424,12 @@ class ParameterHandler : public DocParameterImpl,
 
     // IDocParameter
     virtual Kind kind() const { return DocImpl::Parameter; }
-    virtual QString name() const { return m_name; }
+    virtual const IString *name() const { return &m_name; }
     virtual IDocPara *description() const { return m_description; }
 
   private:
     IBaseHandler     *m_parent;
-    QString           m_name;
+    StringImpl        m_name;
     ParagraphHandler *m_description;
 };
 
@@ -404,13 +506,13 @@ class LinkHandler : public DocLinkImpl, public BaseHandler<LinkHandler>
 
     // IDocLink
     virtual Kind kind() const { return DocImpl::Link; }
-    virtual QString refId() const { return m_ref; }
-    virtual QString text() const { return m_text; }
+    virtual const IString *refId() const { return &m_ref; }
+    virtual const IString *text() const { return &m_text; }
 
   private:
     IBaseHandler   *m_parent;
-    QString         m_ref;
-    QString         m_text;
+    StringImpl      m_ref;
+    StringImpl      m_text;
 };
 
 
@@ -430,11 +532,11 @@ class EMailHandler : public DocEMailImpl, public BaseHandler<EMailHandler>
 
     // IDocEMail
     virtual Kind kind() const { return DocImpl::EMail; }
-    virtual QString address() const { return m_address; }
+    virtual const IString *address() const { return &m_address; }
 
   private:
     IBaseHandler   *m_parent;
-    QString         m_address;
+    StringImpl      m_address;
 };
 
 
@@ -454,13 +556,13 @@ class ULinkHandler : public DocULinkImpl, public BaseHandler<ULinkHandler>
 
     // IDocULink
     virtual Kind kind() const { return DocImpl::ULink; }
-    virtual QString url() const { return m_url; }
-    virtual QString text() const { return m_text; }
+    virtual const IString * url() const { return &m_url; }
+    virtual const IString * text() const { return &m_text; }
 
   private:
     IBaseHandler   *m_parent;
-    QString         m_url;
-    QString         m_text;
+    StringImpl      m_url;
+    StringImpl      m_text;
 };
 
 //-----------------------------------------------------------------------------
@@ -499,16 +601,16 @@ class RefHandler : public DocRefImpl, public BaseHandler<RefHandler>
 
     // IDocRef
     virtual Kind kind() const { return DocImpl::Ref; }
-    virtual QString refId() const { return m_refId; }
+    virtual const IString *refId() const { return &m_refId; }
     virtual TargetKind targetKind() const { return m_targetKind; }
-    virtual QString external() const { return m_extId; }
-    virtual QString text() const { return m_linkText; }
+    virtual const IString *external() const { return &m_extId; }
+    virtual const IString *text() const { return &m_linkText; }
 
   private:
     IBaseHandler   *m_parent;
-    QString         m_refId;
-    QString         m_extId;
-    QString         m_linkText;
+    StringImpl      m_refId;
+    StringImpl      m_extId;
+    StringImpl      m_linkText;
     TargetKind      m_targetKind;
 };
 
@@ -569,7 +671,7 @@ class SimpleSectHandler : public DocSimpleSectImpl,
     // IDocSimpleSect
     virtual Kind kind() const { return DocImpl::SimpleSect; }
     virtual Types type() const { return m_type; }
-    virtual QString typeString() const { return m_typeString; }
+    virtual const IString *typeString() const { return &m_typeString; }
     virtual IDocTitle *title() const { return m_title; }
     virtual IDocPara *description() const { return m_paragraph; }
 
@@ -577,7 +679,7 @@ class SimpleSectHandler : public DocSimpleSectImpl,
     IBaseHandler            *m_parent;
     ParagraphHandler        *m_paragraph;
     Types                    m_type;
-    QString                  m_typeString;
+    StringImpl               m_typeString;
     TitleHandler            *m_title;
 };
 
@@ -603,13 +705,14 @@ class VariableListEntryHandler : public DocVariableListEntryImpl,
 
     // IDocVariableListEntry
     virtual Kind kind() const { return DocImpl::VariableListEntry; }
-    virtual QString term() const { return m_term; }
+    virtual ILinkedTextIterator *term() const;
     virtual IDocPara *description() const { return m_description; }
 
   private:
-    IBaseHandler     *m_parent;
-    QString           m_term;
-    ParagraphHandler *m_description;
+    IBaseHandler*         m_parent;
+    QList<LinkedTextImpl> m_term;
+    ParagraphHandler*     m_description;
+    LinkedTextHandler*    m_linkedTextHandler;
 };
 
 //-----------------------------------------------------------------------------
@@ -664,6 +767,7 @@ class HighlightHandler : public DocHighlightImpl, public BaseHandler<HighlightHa
     void startHighlight(const QXmlAttributes& attrib);
     void endHighlight();
     virtual void startRef(const QXmlAttributes&);
+    virtual void startSpace(const QXmlAttributes&);
 
     // IDocHighlight
     virtual Kind kind() const { return DocImpl::Highlight; }
@@ -710,7 +814,7 @@ class CodeLineHandler : public DocCodeLineImpl, public BaseHandler<CodeLineHandl
     // IDocCodeLine
     virtual Kind kind() const { return DocImpl::CodeLine; }
     virtual int lineNumber() const { return m_lineNumber; }
-    virtual QString refId() const { return m_refId; }
+    virtual const IString *refId() const { return &m_refId; }
     virtual IDocIterator *codeElements() const;
 
   private:
@@ -718,7 +822,7 @@ class CodeLineHandler : public DocCodeLineImpl, public BaseHandler<CodeLineHandl
 
     IBaseHandler   *m_parent;
     int            m_lineNumber;
-    QString        m_refId;
+    StringImpl     m_refId;
     QList<DocImpl> m_children;
 };
 
@@ -782,13 +886,13 @@ class FormulaHandler : public DocFormulaImpl, public BaseHandler<FormulaHandler>
 
     // IDocFormula
     virtual Kind kind() const { return DocImpl::Formula; }
-    virtual QString id() const { return m_id; }
-    virtual QString text() const { return m_text; }
+    virtual const IString *id() const { return &m_id; }
+    virtual const IString *text() const { return &m_text; }
 
   private:
     IBaseHandler  *m_parent;
-    QString        m_id;
-    QString        m_text;
+    StringImpl     m_id;
+    StringImpl     m_text;
 };
 
 //-----------------------------------------------------------------------------
@@ -807,13 +911,36 @@ class ImageHandler : public DocImageImpl, public BaseHandler<ImageHandler>
 
     // IDocImage
     virtual Kind kind() const { return DocImpl::Image; }
-    virtual QString name() const { return m_name; }
-    virtual QString caption() const { return m_caption; }
+    virtual const IString *name() const { return &m_name; }
+    virtual const IString *caption() const { return &m_caption; }
 
   private:
     IBaseHandler  *m_parent;
-    QString        m_name;
-    QString        m_caption;
+    StringImpl     m_name;
+    StringImpl     m_caption;
+};
+
+
+//-----------------------------------------------------------------------------
+/*! \brief Node representing an anchor.
+ *
+ */
+// children: -
+class AnchorHandler : public DocAnchorImpl, public BaseHandler<AnchorHandler>
+{
+  public:
+    AnchorHandler(IBaseHandler *parent);
+    virtual ~AnchorHandler();
+    void startAnchor(const QXmlAttributes& attrib);
+    void endAnchor();
+
+    // IDocAnchor
+    virtual Kind kind() const { return DocImpl::Anchor; }
+    virtual const IString *id() const { return &m_id; }
+
+  private:
+    IBaseHandler  *m_parent;
+    StringImpl     m_id;
 };
 
 //-----------------------------------------------------------------------------
@@ -832,13 +959,13 @@ class DotFileHandler : public DocDotFileImpl, public BaseHandler<DotFileHandler>
 
     // IDocDotFile
     virtual Kind kind() const { return DocImpl::DotFile; }
-    virtual QString name() const { return m_name; }
-    virtual QString caption() const { return m_caption; }
+    virtual const IString *name() const { return &m_name; }
+    virtual const IString *caption() const { return &m_caption; }
 
   private:
     IBaseHandler  *m_parent;
-    QString        m_name;
-    QString        m_caption;
+    StringImpl     m_name;
+    StringImpl     m_caption;
 };
 
 //-----------------------------------------------------------------------------
@@ -861,13 +988,13 @@ class IndexEntryHandler : public DocIndexEntryImpl, public BaseHandler<IndexEntr
 
     // IDocIndexEntry
     virtual Kind kind() const { return DocImpl::IndexEntry; }
-    virtual QString primary() const { return m_primary; }
-    virtual QString secondary() const { return m_secondary; }
+    virtual const IString *primary() const { return &m_primary; }
+    virtual const IString *secondary() const { return &m_secondary; }
 
   private:
     IBaseHandler  *m_parent;
-    QString        m_primary;
-    QString        m_secondary;
+    StringImpl     m_primary;
+    StringImpl     m_secondary;
 };
 
 //-----------------------------------------------------------------------------
@@ -956,13 +1083,13 @@ class TableHandler : public DocTableImpl, public BaseHandler<TableHandler>
     virtual Kind kind() const { return DocImpl::Table; }
     virtual IDocIterator *rows() const;
     virtual int numColumns() const { return m_numColumns; }
-    virtual QString caption() const { return m_caption; }
+    virtual const IString *caption() const { return &m_caption; }
 
   private:
     IBaseHandler      *m_parent;
     QList<RowHandler>  m_children;
     int                m_numColumns;
-    QString            m_caption;
+    StringImpl         m_caption;
 };
 
 class TableIterator : public BaseIteratorVia<IDocIterator,IDoc,RowHandler,DocImpl>
@@ -972,48 +1099,184 @@ class TableIterator : public BaseIteratorVia<IDocIterator,IDoc,RowHandler,DocImp
       BaseIteratorVia<IDocIterator,IDoc,RowHandler,DocImpl>(handler.m_children) {}
 };
 
+//-----------------------------------------------------------------------------
+
+/*! \brief Node representing a copied piece of documentation.
+ *
+ */
+class CopyHandler : public DocCopyImpl, public BaseHandler<CopyHandler>
+{
+    friend class CopyIterator;
+  public:
+    CopyHandler(IBaseHandler *parent);
+    virtual ~CopyHandler();
+    virtual void startCopy(const QXmlAttributes& attrib);
+    virtual void endCopy();
+    virtual void startParagraph(const QXmlAttributes& attrib);
+
+    // IDocCopy
+    virtual Kind kind() const { return DocImpl::Copy; }
+    virtual IDocIterator *contents() const;
+
+  private:
+    IBaseHandler   *m_parent;
+    QList<DocImpl>  m_children;
+};
+
+class CopyIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
+{
+  public:
+    CopyIterator(const CopyHandler &handler) : 
+      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_children) {}
+};
+
+//-----------------------------------------------------------------------------
+
+/*! \brief Node representing an preformatted section
+ */
+class VerbatimHandler : public DocVerbatimImpl, 
+                        public BaseHandler<VerbatimHandler>
+{
+  public:
+    VerbatimHandler(IBaseHandler *parent);
+    virtual ~VerbatimHandler();
+    void startVerbatim(const QXmlAttributes& attrib,Types type);
+    void endVerbatim();
+
+    // IDocVerbatim
+    virtual Kind kind() const { return DocImpl::Verbatim; }
+    virtual const IString *text() const { return &m_text; }
+    virtual Types type() const { return m_type; }
+
+  private:
+    IBaseHandler *m_parent;
+    StringImpl    m_text;
+    Types         m_type;
+};
+
+
+//-----------------------------------------------------------------------------
+
+/*! \brief Node representing an special symbol.
+ *
+ */
+// children: -
+class SymbolHandler : public DocSymbolImpl, public BaseHandler<SymbolHandler>
+{
+  public:
+    SymbolHandler(IBaseHandler *parent,Types type);
+    virtual ~SymbolHandler();
+    void startSymbol(const QXmlAttributes& attrib);
+    void endSymbol();
+
+    // IDocSymbol
+    virtual Kind kind() const   { return DocImpl::Symbol; }
+    virtual Types type() const  { return m_type; }
+    virtual const IString *typeString() const { return &m_typeString; }
+    virtual char letter() const { return m_letter; }
+
+  private:
+    IBaseHandler  *m_parent;
+    char           m_letter;
+    Types          m_type;
+    StringImpl     m_typeString;
+};
+
 
 //-----------------------------------------------------------------------------
 
 /*! \brief Node representing a section.
  *
  */
-// children: text, ref
-// children handled by MarkupHandler: 
-//           bold, computeroutput, emphasis, center,
-//           small, subscript, superscript. 
+// children: title, para, sect(n+1)
 class DocSectionHandler : public DocSectionImpl, public BaseHandler<DocSectionHandler>
 {
-    friend class DocSectionIterator;
+    friend class DocSectionParaIterator;
+    friend class DocSectionSubIterator;
   public:
     DocSectionHandler(IBaseHandler *parent,int level);
     virtual ~DocSectionHandler();
     virtual void startDocSection(const QXmlAttributes& attrib);
     virtual void endDocSection();
-    virtual void startRef(const QXmlAttributes& attrib);
-    void addTextNode();
+    virtual void startTitle(const QXmlAttributes& attrib);
+    virtual void startSubSection(const QXmlAttributes& attrib);
+    virtual void startParagraph(const QXmlAttributes& attrib);
+    virtual void startInternal(const QXmlAttributes& attrib);
 
     // IDocSection
     virtual Kind kind() const { return DocImpl::Section; }
-    virtual QString id() const { return m_id; }
+    virtual const IString *id() const { return &m_id; }
     virtual int level() const { return m_level; }
-    virtual IDocIterator *title() const;
+    virtual IDocTitle *title() const { return m_title; }
+    virtual IDocIterator *paragraphs() const;
+    virtual IDocIterator *subSections() const;
+    virtual IDocInternal *internal() const;
 
   private:
-    IBaseHandler   *m_parent;
-    QList<DocImpl>  m_children;
-    MarkupHandler  *m_markupHandler;
-    QString         m_id;
-    int             m_level;
+    IBaseHandler    *m_parent;
+    QList<DocImpl>   m_paragraphs;
+    QList<DocImpl>   m_subsections;
+    DocInternalHandler *m_internal;
+    StringImpl       m_id;
+    int              m_level;
+    TitleHandler    *m_title;
 };
 
-class DocSectionIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
+class DocSectionParaIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
 {
   public:
-    DocSectionIterator(const DocSectionHandler &handler) : 
-      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_children) {}
+    DocSectionParaIterator(const DocSectionHandler &handler) : 
+      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_paragraphs) {}
 };
 
+class DocSectionSubIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
+{
+  public:
+    DocSectionSubIterator(const DocSectionHandler &handler) : 
+      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_subsections) {}
+};
+
+//-----------------------------------------------------------------------------
+
+class DocInternalHandler : public DocInternalImpl, public BaseHandler<DocInternalHandler>
+{
+  public:
+    friend class DocInternalParaIterator;
+    friend class DocInternalSubIterator;
+    DocInternalHandler(IBaseHandler *parent,int level);
+    virtual ~DocInternalHandler();
+    virtual void startInternal(const QXmlAttributes& attrib);
+    virtual void endInternal();
+    virtual void startSubSection(const QXmlAttributes& attrib);
+    virtual void startParagraph(const QXmlAttributes& attrib);
+
+    // IDocInternal
+    virtual Kind kind() const { return DocImpl::Internal; }
+    virtual IDocIterator *paragraphs() const;
+    virtual IDocIterator *subSections() const;
+
+  private:
+    IBaseHandler    *m_parent;
+    QList<DocImpl>   m_paragraphs;
+    QList<DocImpl>   m_subsections;
+    int              m_level;
+};
+
+class DocInternalParaIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
+{
+  public:
+    DocInternalParaIterator(const DocInternalHandler &handler) : 
+      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_paragraphs) {}
+};
+
+class DocInternalSubIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
+{
+  public:
+    DocInternalSubIterator(const DocInternalHandler &handler) : 
+      BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_subsections) {}
+};
+
+
 //-----------------------------------------------------------------------------
 
 /*! \brief Node representing a documentation block.
@@ -1028,8 +1291,6 @@ class DocHandler : public DocRootImpl, public BaseHandler<DocHandler>
     virtual void endDoc();
     virtual void startParagraph(const QXmlAttributes& attrib);
     virtual void startSect1(const QXmlAttributes& attrib);
-    virtual void startSect2(const QXmlAttributes& attrib);
-    virtual void startSect3(const QXmlAttributes& attrib);
     virtual void startTitle(const QXmlAttributes& attrib);
 
     DocHandler(IBaseHandler *parent);
@@ -1051,6 +1312,8 @@ class DocIterator : public BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>
       BaseIteratorVia<IDocIterator,IDoc,DocImpl,DocImpl>(handler.m_children) {}
 };
 
+//-----------------------------------------------------------------------------
+
 void dochandler_init();
 void dochandler_exit();
 
