@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * $Id: mainhandler.cpp,v 1.87 2004/05/07 12:55:58 dimitri Exp $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,8 +18,11 @@
 #include "compoundhandler.h"
 #include "sectionhandler.h"
 #include "graphhandler.h"
+#include "dochandler.h"
+#include "memberhandler.h"
 #include "debug.h"
 
+
 class ErrorHandler : public QXmlErrorHandler
 {
     public:
@@ -91,8 +94,8 @@ MainHandler::MainHandler() : m_compoundDict(2999), m_compoundNameDict(2999),
 {
   m_compounds.setAutoDelete(TRUE);
   m_memberNameDict.setAutoDelete(TRUE);
-  addStartHandler("doxygen"); 
-  addEndHandler("doxygen");
+  addStartHandler("doxygenindex"); 
+  addEndHandler("doxygenindex");
   addStartHandler("compound",this,&MainHandler::startCompound);
   addEndHandler("compound");
   addStartHandler("member",this,&MainHandler::startMember);
@@ -100,6 +103,7 @@ MainHandler::MainHandler() : m_compoundDict(2999), m_compoundNameDict(2999),
   addStartHandler("name",this,&MainHandler::startName);
   addEndHandler("name",this,&MainHandler::endName);
   m_curCompound = 0;
+  m_insideMember = FALSE;
 }
 
 MainHandler::~MainHandler()
@@ -110,7 +114,7 @@ MainHandler::~MainHandler()
 void MainHandler::startCompound(const QXmlAttributes& attrib)
 {
   m_curCompound = new CompoundEntry(257);
-  m_curCompound->id = attrib.value("id");
+  m_curCompound->id = attrib.value("refid");
   m_compounds.append(m_curCompound);
   m_compoundDict.insert(m_curCompound->id,m_curCompound);
 }
@@ -122,21 +126,27 @@ void MainHandler::startName(const QXmlAttributes& /*attrib*/)
 
 void MainHandler::endName()
 {
-  m_curCompound->name = m_curString;
+  if (m_insideMember)
+  {
+    m_curMember->name = m_curString;
+  }
+  else
+  {
+    m_curCompound->name = m_curString;
+  }
 }
 
 void MainHandler::startMember(const QXmlAttributes& attrib)
 {
-  m_curString = "";
+  m_insideMember = TRUE;
   m_curMember = new MemberEntry;
-  m_curMember->id = attrib.value("id");
+  m_curMember->id = attrib.value("refid");
   m_curMember->compound = m_curCompound;
   m_memberDict.insert(m_curMember->id,m_curMember);
 }
 
 void MainHandler::endMember()
 {
-  m_curMember->name = m_curString;
   m_curCompound->memberDict.insert(m_curString,m_curMember);
   QList<CompoundEntry> *cel=0;
   if ((cel=m_memberNameDict.find(m_curString))==0)
@@ -145,6 +155,7 @@ void MainHandler::endMember()
     m_memberNameDict.insert(m_curString,cel);
   }
   cel->append(m_curCompound);
+  m_insideMember = FALSE;
 }
 
 void MainHandler::setDebugLevel(int level)
@@ -194,16 +205,17 @@ ICompoundIterator *MainHandler::compounds() const
   return new CompoundEntryIterator(this,m_compounds);
 }
 
-ICompound *MainHandler::compoundById(const QString &id) const
+ICompound *MainHandler::compoundById(const char *id) const
 {
-  if (id.isEmpty()) return 0;
-  CompoundHandler *ch = m_compoundsLoaded[id];
+  QString ids = id;
+  if (ids.isEmpty()) return 0;
+  CompoundHandler *ch = m_compoundsLoaded[ids];
   if (ch) // compound already in memory
   {
     ch->addref(); // returning alias -> increase reference counter
     return ch->toICompound(); 
   }
-  CompoundEntry *ce = m_compoundDict.find(id);
+  CompoundEntry *ce = m_compoundDict.find(ids);
   if (ce==0) return 0; // id not found
   // create and load a new compound
   ch = new CompoundHandler(m_xmlDirName);
@@ -218,34 +230,40 @@ ICompound *MainHandler::compoundById(const QString &id) const
   // unchanged.
   MainHandler *that = (MainHandler *)this;
   ch->initialize(that);
+  printf("loading compound %s in memory\n",id);
   that->m_compoundsLoaded.insert(id,ch);
   return ch->toICompound();
 }
 
 void MainHandler::unloadCompound(CompoundHandler *ch)
 {
-  m_compoundsLoaded.remove(ch->id()); 
+  printf("unloading compound %s from memory\n",ch->id()->latin1());
+  bool result = m_compoundsLoaded.remove(ch->id()->latin1()); 
+  if (!result) printf("Failed to unload!\n");
 }
 
-ICompound *MainHandler::compoundByName(const QString &name) const
+ICompound *MainHandler::compoundByName(const char *name) const
 {
-  if (name.isEmpty()) return 0;
+  QString nameStr = name;
+  if (nameStr.isEmpty()) return 0;
   CompoundEntry *ce = m_compoundNameDict[name];
   if (ce==0) return 0; // name not found
   return compoundById(ce->id);
 }
 
-ICompound *MainHandler::memberById(const QString &id) const
+ICompound *MainHandler::memberById(const char *id) const
 {
-  if (id.isEmpty()) return 0;
+  QString ids = id;
+  if (ids.isEmpty()) return 0;
   MemberEntry *me = m_memberDict[id];
   if (me==0) return 0; // id not found
   return compoundById(me->id);
 }
 
-ICompoundIterator *MainHandler::memberByName(const QString &name) const
+ICompoundIterator *MainHandler::memberByName(const char *name) const
 {
-  if (name.isEmpty()) return 0;
+  QString nameStr = name;
+  if (nameStr.isEmpty()) return 0;
   QList<CompoundEntry> *cel = m_memberNameDict[name];
   if (cel==0) return 0; // name not found
   return new CompoundEntryIterator(this,*cel);
@@ -263,11 +281,12 @@ IDoxygen *createObjectModel()
 
 void MainHandler::release()
 {
+  //printf("MainHandler::release()\n");
   QDictIterator<CompoundHandler> chi(m_compoundsLoaded);
   CompoundHandler *ch;
   for (chi.toFirst();(ch=chi.current());++chi)
   {
-    debug(1,"Compound %s not released\n",ch->name().data());
+    debug(1,"Compound %s not released\n",ch->name()->latin1());
   }
   graphhandler_exit();
   dochandler_exit();
