@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: $
+ * $Id: sectionhandler.cpp,v 1.87 2004/05/07 12:55:58 dimitri Exp $
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -16,6 +16,7 @@
 #include "mainhandler.h"
 #include "compoundhandler.h"
 #include "sectionhandler.h"
+#include "memberhandler.h"
 #include "debug.h"
 
 class SectionTypeMap
@@ -40,6 +41,11 @@ class SectionTypeMap
       m_map.insert("protected-slot",new int(ISection::ProSlots));
       m_map.insert("protected-static-func",new int(ISection::ProStatFuncs));
       m_map.insert("protected-static-attrib",new int(ISection::ProStatAttribs));
+      m_map.insert("package-type",new int(ISection::PacTypes));
+      m_map.insert("package-func",new int(ISection::PacFuncs));
+      m_map.insert("package-attrib",new int(ISection::PacAttribs));
+      m_map.insert("package-static-func",new int(ISection::PacStatFuncs));
+      m_map.insert("package-static-attrib",new int(ISection::PacStatAttribs));
       m_map.insert("private-type",new int(ISection::PriTypes));
       m_map.insert("private-func",new int(ISection::PriFuncs));
       m_map.insert("private-attrib",new int(ISection::PriAttribs));
@@ -87,6 +93,8 @@ SectionHandler::SectionHandler(IBaseHandler *parent) : m_parent(parent)
   m_members.setAutoDelete(TRUE);
   addEndHandler("sectiondef",this,&SectionHandler::endSection);
   addStartHandler("memberdef",this,&SectionHandler::startMember);
+  addStartHandler("header",this,&SectionHandler::startHeader);
+  addEndHandler("header",this,&SectionHandler::endHeader);
 }
 
 SectionHandler::~SectionHandler()
@@ -114,6 +122,18 @@ void SectionHandler::startMember(const QXmlAttributes& attrib)
   m_members.append(memHandler);
 }
 
+void SectionHandler::startHeader(const QXmlAttributes&)
+{
+  m_header="";
+  m_curString="";
+}
+
+void SectionHandler::endHeader()
+{
+  m_header = m_curString.stripWhiteSpace();
+  debug(2,"member header=`%s'\n",m_header.data());
+}
+
 void SectionHandler::initialize(CompoundHandler *ch)
 {
   QListIterator<MemberHandler> mli(m_members);
@@ -126,4 +146,8 @@ void SectionHandler::initialize(CompoundHandler *ch)
   }
 }
 
+IMemberIterator *SectionHandler::members() const 
+{ 
+  return new MemberIterator(m_members); 
+}
 
