@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: bufstr.h,v 1.85 2000/11/18 12:58:19 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -33,7 +33,7 @@ class BufStr : public QCString
       {
         QCString::resize(size()+spareRoom);
       }
-      data()[offset++]=c;
+      QCString::data()[offset++]=c;
     }
     void addArray(const char *a,int len)
     {
@@ -41,7 +41,7 @@ class BufStr : public QCString
       {
         QCString::resize(size()+len+spareRoom);
       }
-      memcpy(data()+offset,a,len);
+      memcpy(QCString::data()+offset,a,len);
       offset+=len;
     }
     uint curPos() { return offset; }
