@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: classdef.h,v 1.39 2001/03/19 19:27:39 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -63,6 +63,8 @@ class ClassDef : public Definition
                         Struct=Entry::STRUCT_SEC, 
                         Union=Entry::UNION_SEC,
                         Interface=Entry::INTERFACE_SEC,
+                        Protocol=Entry::PROTOCOL_SEC,
+                        Category=Entry::CATEGORY_SEC,
                         Exception=Entry::EXCEPTION_SEC
                       };
     DefType definitionType() { return TypeClass; }
@@ -72,6 +74,7 @@ class ClassDef : public Definition
     QCString getSourceFileBase() const; 
     QCString getReference() const;
     bool isReference() const;
+    bool isLocal() const { return m_isLocal; }
 
     bool hasDocumentation() const;
 
@@ -102,6 +105,7 @@ class ClassDef : public Definition
     void writeDocumentationForInnerClasses(OutputList &ol);
     void writeMemberList(OutputList &ol);
     void writeDeclaration(OutputList &ol,MemberDef *md,bool inGroup);
+    void writeDetailedDescription(OutputList &ol,const QCString &pageType,bool exampleFlag);
 
     /*! Return the protection level (Public,Protected,Private) in which 
      *  this compound was found.
@@ -125,12 +129,6 @@ class ClassDef : public Definition
      */
     ArgumentList *templateArguments() const { return m_tempArgs; }
 
-    /*! Returns the template arguments that this nested class "inherits" 
-     *  from its outer class (doxygen assumes there is only one!). 
-     *  Will return 0 if not applicable.
-     */
-    ArgumentList *outerTemplateArguments() const;
-    
     /*! Returns the namespace this compound is in, or 0 if it has a global
      *  scope.
      */
@@ -143,7 +141,7 @@ class ClassDef : public Definition
 
     /*! Returns the Java package this class is in or 0 if not applicable. 
      */ 
-    PackageDef   *packageDef() const;
+    //PackageDef   *packageDef() const;
 
     MemberDef    *getMemberByName(const QCString &);
     
@@ -151,7 +149,7 @@ class ClassDef : public Definition
      *  class. This function will recusively traverse all branches of the
      *  inheritance tree.
      */
-    bool isBaseClass(ClassDef *bcd,bool followInstances);
+    bool isBaseClass(ClassDef *bcd,bool followInstances,int level=0);
 
     /*! Returns a sorted dictionary with all template instances found for
      *  this template class. Returns 0 if not a template or no instances.
@@ -162,11 +160,18 @@ class ClassDef : public Definition
      */
     ClassDef *templateMaster() const { return m_templateMaster; } 
 
+    IncludeInfo *includeInfo() const { return m_incInfo; }
+    
     UsesClassDict *usedImplementationClasses() const 
     { 
       return m_usesImplClassDict; 
     }
 
+    UsesClassDict *usedByImplementationClasses() const 
+    { 
+      return m_usedByImplClassDict; 
+    }
+
     UsesClassDict *usedInterfaceClasses() const
     {
       return m_usesIntfClassDict;
@@ -194,35 +199,46 @@ class ClassDef : public Definition
      */
     bool isAbstract() const { return m_isAbstract; }
 
+    /*! Returns TRUE if this class is implemented in Objective-C */
+    bool isObjectiveC() const { return m_isObjC; }
+
+    ClassDef *categoryOf() const { return m_categoryOf; }
+
     /*! returns the name of the class including outer classes, but not
      *  including namespaces.
      */
     QCString className() const;
 
     /* member lists by protection */
-    MemberList pubMembers;
-    MemberList proMembers;
-    MemberList priMembers;
-    MemberList pubStaticMembers;
-    MemberList proStaticMembers;
-    MemberList priStaticMembers;
+    MemberList pubMethods;
+    MemberList proMethods;
+    MemberList pacMethods;
+    MemberList priMethods;
+    MemberList pubStaticMethods;
+    MemberList proStaticMethods;
+    MemberList pacStaticMethods;
+    MemberList priStaticMethods;
     MemberList pubSlots;
     MemberList proSlots;
     MemberList priSlots;
     MemberList pubAttribs;
     MemberList proAttribs;
+    MemberList pacAttribs;
     MemberList priAttribs;
     MemberList pubStaticAttribs;
     MemberList proStaticAttribs;
+    MemberList pacStaticAttribs;
     MemberList priStaticAttribs;
     MemberList pubTypes;
     MemberList proTypes;
+    MemberList pacTypes;
     MemberList priTypes;
     MemberList related;
     MemberList signals;
     MemberList friends;
     MemberList dcopMethods;
     MemberList properties;
+    MemberList events;
     
     /* member list by types */
     MemberList constructors;
@@ -233,6 +249,7 @@ class ClassDef : public Definition
     MemberList relatedMembers;
     MemberList variableMembers;
     MemberList propertyMembers;
+    MemberList eventMembers;
 
     /* user defined member groups */
     MemberGroupSDict *memberGroupSDict;
@@ -244,7 +261,7 @@ class ClassDef : public Definition
      */
     void insertBaseClass(ClassDef *,const char *name,Protection p,Specifier s,const char *t=0);
     void insertSubClass(ClassDef *,Protection p,Specifier s,const char *t=0);
-    void setIncludeFile(FileDef *fd,const char *incName,bool local); 
+    void setIncludeFile(FileDef *fd,const char *incName,bool local,bool force); 
     void insertMember(MemberDef *);
     void insertUsedFile(const char *);
     void computeAnchors();
@@ -254,9 +271,11 @@ class ClassDef : public Definition
     bool addExample(const char *anchor,const char *name, const char *file);
     void addMembersToMemberGroup();
     void distributeMemberGroupDocumentation();
+    void findSectionsInDocumentation();
     void setNamespace(NamespaceDef *nd) { m_nspace = nd; }
     void setTemplateArguments(ArgumentList *al);
     void mergeMembers();
+    void mergeCategory(ClassDef *category);
     void setFileDef(FileDef *fd) { m_fileDef=fd; }
     //void determineImplUsageRelation();
     //void determineIntfUsageRelation();
@@ -265,6 +284,7 @@ class ClassDef : public Definition
     void setGroupDefForAllMembers(GroupDef *g,Grouping::GroupPri_t pri,const QCString &fileName,int startLine,bool hasDocs);
     void addInnerCompound(Definition *d);
     void addUsedClass(ClassDef *cd,const char *accessName);
+    void addUsedByClass(ClassDef *cd,const char *accessName);
     //void initTemplateMapping();
     //void setTemplateArgumentMapping(const char *formal,const char *actual);
     //QCString getTemplateArgumentMapping(const char *formal) const;
@@ -276,6 +296,7 @@ class ClassDef : public Definition
     void addMembersToTemplateInstance(ClassDef *cd,const char *templSpec);
     void setClassIsArtificial() { m_artificial = TRUE; }
     void setIsStatic(bool b) { m_isStatic=b; }
+    void setIsObjectiveC(bool b) { m_isObjC=b; }
     void addListReferences();
 
     /*! Creates a new compound definition.
@@ -290,10 +311,13 @@ class ClassDef : public Definition
      *  \param fName     the file name as found in the tag file. 
      *                   This overwrites the file that doxygen normally 
      *                   generates based on the compound type & name.
+     *  \param isSymbol  If TRUE this class name is added as a publicly 
+     *                   visible (and referencable) symbol.
      */
     ClassDef(const char *fileName,int startLine,
              const char *name,CompoundType ct,
-             const char *ref=0,const char *fName=0);
+             const char *ref=0,const char *fName=0,
+             bool isSymbol=TRUE);
     /*! Destroys a compound definition. */
    ~ClassDef();
 
@@ -384,6 +408,7 @@ class ClassDef : public Definition
 
     /* classes for the collaboration diagram */
     UsesClassDict *m_usesImplClassDict;
+    UsesClassDict *m_usedByImplClassDict;
     UsesClassDict *m_usesIntfClassDict;
 
     /*! Template instances that exists of this class, the key in the
@@ -409,11 +434,22 @@ class ClassDef : public Definition
     /*! Is the class part of an unnamed namespace? */
     bool m_isStatic;
 
+    /*! Is the class part implemented in Objective C? */
+    bool m_isObjC;
+
     /*! TRUE if classes members are merged with those of the base classes. */
     bool m_membersMerged;
 
     /*! TRUE if the class is defined in a source file rather than a header file. */
     bool m_isLocal;
+
+    /*! class name with outer class scope, but without namespace scope. */
+    QCString m_className;
+
+    /*! If this class is a Objective-C category, then this points to the
+     *  class which is extended.
+     */
+    ClassDef *m_categoryOf;
 };
 
 /*! \brief Class that contains information about a usage relation. 
