@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: classlist.cpp,v 1.14 2001/03/19 19:27:39 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -20,7 +20,6 @@
 #include "util.h"
 #include "outputlist.h"
 #include "language.h"
-#include "doc.h"
 #include "doxygen.h"
 
 ClassList::ClassList() : QList<ClassDef>()
@@ -31,26 +30,29 @@ ClassList::~ClassList()
 {
 }
 
-int ClassList::compareItems(GCI item1, GCI item2)
+static int compItems(void *item1,void *item2)
 {
   ClassDef *c1=(ClassDef *)item1;
   ClassDef *c2=(ClassDef *)item2;
-  //return stricmp(c1->localName().data()+getPrefixIndex(c1->localName()),
-  //               c2->localName().data()+getPrefixIndex(c2->localName())
-  //              );
-  return stricmp(c1->className().data()+getPrefixIndex(c1->className()),
-                 c2->className().data()+getPrefixIndex(c2->className()));
+  int p1=0,p2=0;
+  static bool b = Config_getBool("SORT_BY_SCOPE_NAME");
+  if (!b) 
+  { 
+    p1=getPrefixIndex(c1->className()); 
+    p2=getPrefixIndex(c2->className()); 
+  }
+  return stricmp(c1->className().data()+p1,
+                 c2->className().data()+p2);
+}
+
+int ClassList::compareItems(GCI item1, GCI item2)
+{
+  return compItems(item1,item2);
 }
 
 int ClassSDict::compareItems(GCI item1, GCI item2)
 {
-  ClassDef *c1=(ClassDef *)item1;
-  ClassDef *c2=(ClassDef *)item2;
-  //return stricmp(c1->localName().data()+getPrefixIndex(c1->localName()),
-  //               c2->localName().data()+getPrefixIndex(c2->localName())
-  //              );
-  return stricmp(c1->className().data()+getPrefixIndex(c1->className()),
-                 c2->className().data()+getPrefixIndex(c2->className()));
+  return compItems(item1,item2);
 }
 
 ClassListIterator::ClassListIterator(const ClassList &cllist) :
@@ -72,18 +74,22 @@ void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *f
          )
       {
         bool isLink = cd->isLinkable();
-        if (isLink || !Config_getBool("HIDE_UNDOC_CLASSES"))
+        if (isLink || 
+             (!Config_getBool("HIDE_UNDOC_CLASSES") && 
+              (!cd->isLocal() || Config_getBool("EXTRACT_LOCAL_CLASSES"))
+             )
+           )
         {
           if (!found)
           {
             ol.startMemberHeader();
             if (header)
             {
-              parseText(ol,header);
+              ol.parseText(header);
             }
             else
             {
-              parseText(ol,theTranslator->trCompounds());
+              ol.parseText(theTranslator->trCompounds());
             }
             ol.endMemberHeader();
             ol.startMemberList();
@@ -96,6 +102,15 @@ void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *f
           }
           ol.startMemberItem(FALSE);
           QCString tmp = cd->compoundTypeString();
+          QCString cname = cd->displayName();
+          //if (Config_getBool("OPTIMIZE_OUTPUT_JAVA"))
+          //{
+          //  cname = substitute(cd->className(),"::",".");
+          //}
+          //else
+          //{
+          //  cname = cd->className();
+          //}
           ol.writeString(tmp);
           ol.writeString(" ");
           ol.insertMemberAlign();
@@ -104,31 +119,34 @@ void ClassSDict::writeDeclaration(OutputList &ol,const ClassDef::CompoundType *f
             ol.writeObjectLink(cd->getReference(),
                 cd->getOutputFileBase(),
                 0,
-                cd->localName()
+                cname
                );
           }
           else 
           {
             ol.startBold();
-            ol.docify(cd->localName());
+            ol.docify(cname);
             ol.endBold();
           }
-          ol.endMemberItem(FALSE);
+          ol.endMemberItem();
           if (!cd->briefDescription().isEmpty())
           {
             ol.startMemberDescription();
-            parseDoc(ol,cd->getDefFileName(),cd->getDefLine(),cd->name(),0,cd->briefDescription());
-            if ((!cd->briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) ||
-                !cd->documentation().isEmpty())
+            ol.parseDoc(cd->briefFile(),cd->briefLine(),cd,0,
+                        cd->briefDescription(),FALSE,FALSE);
+            if (//(!cd->briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) ||
+                //!cd->documentation().isEmpty())
+                cd->isLinkableInProject()
+               )
             {
               ol.pushGeneratorState();
               ol.disableAllBut(OutputGenerator::Html);
-              ol.endEmphasis();
+              //ol.endEmphasis();
               ol.docify(" ");
               ol.startTextLink(cd->getOutputFileBase(),"_details");
-              parseText(ol,theTranslator->trMore());
+              ol.parseText(theTranslator->trMore());
               ol.endTextLink();
-              ol.startEmphasis();
+              //ol.startEmphasis();
               ol.popGeneratorState();
             }
             ol.endMemberDescription();
