@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: classlist.h,v 1.8 2001/03/19 19:27:39 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -49,7 +49,7 @@ class ClassDict : public QDict<ClassDef>
 class ClassSDict : public SDict<ClassDef>
 {
   public:
-    ClassSDict(int size) : SDict<ClassDef>(size) {}
+    ClassSDict(int size=17) : SDict<ClassDef>(size) {}
    ~ClassSDict() {}
    int compareItems(GCI item1,GCI item2);
    void writeDeclaration(OutputList &ol,const ClassDef::CompoundType *filter=0,
