@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: debug.h,v 1.6 2001/03/19 19:27:40 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -27,7 +27,11 @@ class Debug
                      Functions    = 0x00000002, 
                      Variables    = 0x00000004,
                      Preprocessor = 0x00000008,
-                     Classes      = 0x00000010
+                     Classes      = 0x00000010,
+                     CommentCnv   = 0x00000020,
+                     Validate     = 0x00000040,
+                     PrintTree    = 0x00000080,
+                     Time         = 0x00000100
                    };
     static void print(DebugMask mask,int prio,const char *fmt,...);
     static void setFlag(const char *label);
