@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: index.cpp,v 1.30 1999/03/06 22:15:39 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,27 +18,78 @@
 #include "qtbc.h"
 #include <ctype.h>
 #include <qregexp.h>
+#include <stdio.h>
+#include <stdlib.h>
 #include "config.h"
 #include "definition.h"
 #include "doxygen.h"
 #include "language.h"
 #include "message.h"
 #include "outputlist.h"
-#include "doc.h"
 #include "code.h"
 #include "util.h"
 #include "groupdef.h"
+#include "section.h"
+
+#if defined(_MSC_VER) || defined(__BORLANDC__)
+#define popen _popen
+#define pclose _pclose
+#endif
+
+static void addToMap(const char *name,Definition *d)
+{
+  QCString symbolName = name;
+  int index=computeQualifiedIndex(symbolName);
+  if (index!=-1) symbolName=symbolName.mid(index+2);
+  if (!symbolName.isEmpty()) 
+  {
+    DefinitionList *dl=Doxygen::symbolMap->find(symbolName);
+    if (dl==0)
+    {
+      dl = new DefinitionList;
+      Doxygen::symbolMap->append(symbolName,dl);
+    }
+    //printf("******* adding symbol `%s'\n",symbolName.data());
+    dl->append(d);
+    d->setSymbolName(symbolName);
+  }
+}
+
+static void removeFromMap(Definition *d)
+{
+  QCString symbolName = d->symbolName();
+  int index=computeQualifiedIndex(symbolName);
+  if (index!=-1) symbolName=symbolName.mid(index+2);
+  if (!symbolName.isEmpty()) 
+  {
+    //printf("******* removing symbol `%s'\n",symbolName.data());
+    DefinitionList *dl=Doxygen::symbolMap->find(symbolName);
+    if (dl)
+    {
+      ASSERT(dl!=0);
+      bool b = dl->removeRef(d);
+      ASSERT(b==TRUE);
+    }
+  }
+}
 
 Definition::Definition(const char *df,int dl,
-                       const char *name,const char *b,const char *d)
+                       const char *name,const char *b,
+                       const char *d,bool isSymbol)
 {
+  //QCString ns;
   m_defFileName = df;
   m_defLine = dl;
   m_name=name; 
   if (m_name!="<globalScope>") 
+  {
+    //extractNamespaceName(m_name,m_localName,ns);
     m_localName=stripScope(name);
+  }
   else
+  {
     m_localName=name;
+  }
   //printf("m_localName=%s\n",m_localName.data());
   m_brief=b; 
   m_doc=d; 
@@ -50,40 +101,53 @@ Definition::Definition(const char *df,int dl,
   m_todoId=0;
   m_testId=0;
   m_bugId=0;
+  m_deprecatedId=0;
   m_outerScope=Doxygen::globalScope;
   m_partOfGroups=0;
+  m_xrefListItems=0;
+  m_briefLine=1;
+  m_briefFile=(QCString)"<"+name+">";
+  m_docLine=1;
+  m_docFile=(QCString)"<"+name+">";
+  m_isSymbol = isSymbol;
+  if (m_isSymbol) addToMap(name,this);
 }
 
 Definition::~Definition()
 {
+  if (m_isSymbol) removeFromMap(this);
   delete m_sectionDict;
   delete m_sourceRefByDict;
   delete m_sourceRefsDict;
   delete m_partOfGroups;
+  delete m_xrefListItems;
 }
 
-void Definition::addSectionsToDefinition(QList<QCString> *anchorList)
+void Definition::addSectionsToDefinition(QList<SectionInfo> *anchorList)
 {
   if (!anchorList) return;
-  QCString *s=anchorList->first();
-  while (s)
+  //printf("%s: addSectionsToDefinition(%d)\n",name().data(),anchorList->count());
+  SectionInfo *si=anchorList->first();
+  while (si)
   {
-    SectionInfo *si=0;
-    if (!s->isEmpty() && (si=Doxygen::sectionDict[*s]))
+    //printf("Add section `%s' to definition `%s'\n",
+    //    si->label.data(),name().data());
+    SectionInfo *gsi=Doxygen::sectionDict.find(si->label);
+    if (gsi==0)
     {
-      //printf("Add section `%s' to definition `%s'\n",
-      //    si->label.data(),n.data());
-      if (m_sectionDict==0) 
-      {
-        m_sectionDict = new SectionDict(17);
-      }
-      if (m_sectionDict->find(*s)==0)
-      {
-        m_sectionDict->insert(*s,si);
-      }
-      si->definition = this;
+      gsi = new SectionInfo(*si);
+      Doxygen::sectionDict.insert(si->label,gsi);
     }
-    s=anchorList->next();
+    if (m_sectionDict==0) 
+    {
+      m_sectionDict = new SectionDict(17);
+    }
+    if (m_sectionDict->find(gsi->label)==0)
+    {
+      m_sectionDict->insert(gsi->label,gsi);
+      gsi->definition = this;
+    }
+    si=anchorList->next();
   }
 }
 
@@ -91,12 +155,14 @@ void Definition::writeDocAnchorsToTagFile()
 {
   if (!Config_getString("GENERATE_TAGFILE").isEmpty() && m_sectionDict)
   {
+    //printf("%s: writeDocAnchorsToTagFile(%d)\n",name().data(),m_sectionDict->count());
     QDictIterator<SectionInfo> sdi(*m_sectionDict);
     SectionInfo *si;
     for (;(si=sdi.current());++sdi)
     {
       if (!si->generated)
       {
+        //printf("write an entry!\n");
         if (definitionType()==TypeMember) Doxygen::tagFile << "  ";
         Doxygen::tagFile << "    <docanchor>" << si->label << "</docanchor>" << endl;
       }
@@ -106,24 +172,59 @@ void Definition::writeDocAnchorsToTagFile()
 
 void Definition::setDocumentation(const char *d,const char *docFile,int docLine,bool stripWhiteSpace) 
 { 
+  if (d==0) return;
+  //printf("Definition::setDocumentation(%s,%s,%d,%d)\n",d,docFile,docLine,stripWhiteSpace);
+  QCString doc;
   if (stripWhiteSpace)
-    m_doc=((QCString)d).stripWhiteSpace();
-  else
-    m_doc=d;
+  {
+    // strip leading empty lines in front of the text, but not the 
+    // leading spaces in front of the first line, so list items are 
+    // parsed with the correct indent
+    const char *p=d;
+    char c;
+    int s=0,so=0;
+    while ((c=*p) && (c==' ' || c=='\r' || c=='\n'))
+    {
+      if (c=='\n') so=s; 
+      p++;
+      s++;
+    }
+    if (c=='\0') return;
+    doc=d+so;
+    // zero any trailing whitespace
+    int e=doc.length()-1;
+    while (e>=0 && (c=doc.at(e)) && (c==' ' || c=='\r' || c=='\n'))
+    {
+      doc.at(e)='\0';
+      e--;
+    }
+  }
+  else // don't strip whitespace
+  {
+    doc=d;
+  }
+  //printf("setting docs for %s: `%s'\n",name().data(),m_doc.data());
+  m_doc=doc;
   m_docFile = docFile;
   m_docLine = docLine;
 }
 
+#define uni_isupper(c) (QChar(c).category()==QChar::Letter_Uppercase)
+
 void Definition::setBriefDescription(const char *b,const char *briefFile,int briefLine) 
 { 
-  m_brief=QCString(b).stripWhiteSpace(); 
+  if (b==0) return;
+  //printf("Definition::setBriefDescription(%s,%s,%d)\n",b,briefFile,briefLine);
+  m_brief=QCString(b).stripWhiteSpace();
   int bl=m_brief.length(); 
   if (bl>0) // add puntuation if needed
   {
     switch(m_brief.at(bl-1))
     {
       case '.': case '!': case '?': break;
-      default: m_brief+='.'; break;
+      default: 
+        if (uni_isupper(m_brief.at(0))) m_brief+='.'; 
+        break;
     }
   }
   m_briefFile = briefFile;
@@ -145,27 +246,29 @@ static bool readCodeFragment(const char *fileName,
 {
   //printf("readCodeFragment(%s,%d,%d)\n",fileName,startLine,endLine);
   if (fileName==0 || fileName[0]==0) return FALSE; // not a valid file name
-  QFile f(fileName);
-  if (f.open(IO_ReadOnly))
+  QCString cmd=Config_getString("INPUT_FILTER")+" \""+fileName+"\"";
+  FILE *f = Config_getBool("FILTER_SOURCE_FILES") ? popen(cmd,"r") : fopen(fileName,"r");
+  bool found=FALSE;
+  if (f)
   {
     int c=0;
     int col=0;
     int lineNr=1;
     // skip until the startLine has reached
-    while (lineNr<startLine && !f.atEnd())
+    while (lineNr<startLine && !feof(f))
     {
-      while ((c=f.getch())!='\n' && c!=-1) /* skip */;
+      while ((c=fgetc(f))!='\n' && c!=EOF) /* skip */;
       lineNr++; 
     }
-    if (!f.atEnd())
+    if (!feof(f))
     {
       // skip until the opening bracket or lonely : is found
-      bool found=FALSE;
       char cn=0;
-      while (lineNr<=endLine && !f.atEnd() && !found)
+      while (lineNr<=endLine && !feof(f) && !found)
       {
-        while ((c=f.getch())!='{' && c!=':' && c!=-1) 
+        while ((c=fgetc(f))!='{' && c!=':' && c!=EOF) 
         {
+          //printf("parsing char `%c'\n",c);
           if (c=='\n') 
           {
             lineNr++,col=0; 
@@ -181,7 +284,7 @@ static bool readCodeFragment(const char *fileName,
         }
         if (c==':')
         {
-          cn=f.getch();
+          cn=fgetc(f);
           if (cn!=':') found=TRUE;
         }
         else if (c=='{')
@@ -189,6 +292,7 @@ static bool readCodeFragment(const char *fileName,
           found=TRUE;
         }
       }
+      //printf(" -> readCodeFragment(%s,%d,%d) lineNr=%d\n",fileName,startLine,endLine,lineNr);
       if (found) 
       {
         // For code with more than one line,
@@ -208,15 +312,26 @@ static bool readCodeFragment(const char *fileName,
         char lineStr[maxLineLength];
         do 
         {
+          //printf("reading line %d in range %d-%d\n",lineNr,startLine,endLine);
           int size_read;
-          do {
+          do 
+          {
             // read up to maxLineLength-1 bytes, the last byte being zero
-            size_read = f.readLine(lineStr, maxLineLength);
+            char *p = fgets(lineStr, maxLineLength,f);
+            //printf("  read %s",p);
+            if (p) 
+            {
+              size_read=qstrlen(p); 
+            }
+            else 
+            {
+              size_read=-1;
+            }
             result+=lineStr;
           } while (size_read == (maxLineLength-1));
 
           lineNr++; 
-        } while (lineNr<=endLine && !f.atEnd());
+        } while (lineNr<=endLine && !feof(f));
 
         // strip stuff after closing bracket
         int newLineIndex = result.findRev('\n');
@@ -226,11 +341,11 @@ static bool readCodeFragment(const char *fileName,
           result.truncate(braceIndex+1);
         }
         endLine=lineNr-1;
-        return TRUE;
       }
     }
+    if (Config_getBool("FILTER_SOURCE_FILES")) pclose(f); else fclose(f);
   }
-  return FALSE;
+  return found;
 }
 
 /*! Write a reference to the source code defining this definition */
@@ -240,9 +355,9 @@ void Definition::writeSourceDef(OutputList &ol,const char *)
   //printf("Definition::writeSourceRef %d %p\n",bodyLine,bodyDef);
   if (Config_getBool("SOURCE_BROWSER") && m_startBodyLine!=-1 && m_bodyDef)
   {
-    //ol.disable(OutputGenerator::RTF);
+    ol.disable(OutputGenerator::RTF);
     ol.newParagraph();
-    //ol.enableAll();
+    ol.enableAll();
 
     QCString refText = theTranslator->trDefinedAtLineInSourceFile();
     int lineMarkerPos = refText.find("@0");
@@ -255,7 +370,7 @@ void Definition::writeSourceDef(OutputList &ol,const char *)
       if (lineMarkerPos<fileMarkerPos) // line marker before file marker
       {
         // write text left from linePos marker
-        parseText(ol,refText.left(lineMarkerPos)); 
+        ol.parseText(refText.left(lineMarkerPos)); 
         ol.disableAllBut(OutputGenerator::Html); 
         // write line link (HTML only)
         ol.writeObjectLink(0,m_bodyDef->getSourceFileBase(),
@@ -267,7 +382,7 @@ void Definition::writeSourceDef(OutputList &ol,const char *)
         ol.enableAll();
         
         // write text between markers
-        parseText(ol,refText.mid(lineMarkerPos+2,
+        ol.parseText(refText.mid(lineMarkerPos+2,
               fileMarkerPos-lineMarkerPos-2));
 
         ol.disableAllBut(OutputGenerator::Html); 
@@ -281,13 +396,13 @@ void Definition::writeSourceDef(OutputList &ol,const char *)
         ol.enableAll();
         
         // write text right from file marker
-        parseText(ol,refText.right(
+        ol.parseText(refText.right(
               refText.length()-fileMarkerPos-2)); 
       }
       else // file marker before line marker
       {
         // write text left from file marker
-        parseText(ol,refText.left(fileMarkerPos)); 
+        ol.parseText(refText.left(fileMarkerPos)); 
         ol.disableAllBut(OutputGenerator::Html); 
         // write file link (HTML only)
         ol.writeObjectLink(0,m_bodyDef->getSourceFileBase(),
@@ -299,7 +414,7 @@ void Definition::writeSourceDef(OutputList &ol,const char *)
         ol.enableAll();
         
         // write text between markers
-        parseText(ol,refText.mid(fileMarkerPos+2,
+        ol.parseText(refText.mid(fileMarkerPos+2,
               lineMarkerPos-fileMarkerPos-2)); 
 
         ol.disableAllBut(OutputGenerator::Html); 
@@ -313,7 +428,7 @@ void Definition::writeSourceDef(OutputList &ol,const char *)
         ol.enableAll();
 
         // write text right from linePos marker
-        parseText(ol,refText.right(
+        ol.parseText(refText.right(
               refText.length()-lineMarkerPos-2)); 
       }
     }
@@ -334,7 +449,7 @@ void Definition::writeInlineCode(OutputList &ol,const char *scopeName)
 {
   ol.pushGeneratorState();
   //printf("Source Fragment %s: %d-%d bodyDef=%p\n",name().data(),
-  //        startBodyLine,endBodyLine,bodyDef);
+  //        m_startBodyLine,m_endBodyLine,m_bodyDef);
   if (Config_getBool("INLINE_SOURCES") && m_startBodyLine!=-1 && 
       m_endBodyLine>=m_startBodyLine && m_bodyDef)
   {
@@ -367,7 +482,7 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
   if (Config_getBool("SOURCE_BROWSER") && members)
   {
     ol.newParagraph();
-    parseText(ol,text);
+    ol.parseText(text);
     ol.docify(" ");
 
     QCString ldefLine=theTranslator->trWriteList(members->count());
@@ -378,7 +493,7 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
     while ((newIndex=marker.match(ldefLine,index,&matchLen))!=-1)
     {
       bool ok;
-      parseText(ol,ldefLine.mid(index,newIndex-index));
+      ol.parseText(ldefLine.mid(index,newIndex-index));
       uint entryIndex = ldefLine.mid(newIndex+1,matchLen-1).toUInt(&ok);
       MemberDef *md=members->at(entryIndex);
       if (ok && md)
@@ -397,8 +512,13 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
             name.prepend(scope+"::");
           }
         }
-        Definition *d = md->getOuterScope();
-        if (d==Doxygen::globalScope) d=md->getBodyDef();
+        if (!md->isObjCMethod() &&
+            (md->isFunction() || md->isSlot() || 
+             md->isPrototype() || md->isSignal()
+            )
+           ) name+="()";
+        //Definition *d = md->getOutputFileBase();
+        //if (d==Doxygen::globalScope) d=md->getBodyDef();
         if (md->getStartBodyLine()!=-1 && md->getBodyDef()) 
         {
           //printf("md->getBodyDef()=%p global=%p\n",md->getBodyDef(),Doxygen::globalScope); 
@@ -417,12 +537,13 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
           ol.docify(name);
           ol.popGeneratorState();
         }
-        else if (md->isLinkable() && d && d->isLinkable())
+        else if (md->isLinkable() /*&& d && d->isLinkable()*/)
         {
           // for HTML write a real link
           ol.pushGeneratorState();
           ol.disableAllBut(OutputGenerator::Html);
-          ol.writeObjectLink(d->getReference(),d->getOutputFileBase(),
+          ol.writeObjectLink(md->getReference(),
+                             md->getOutputFileBase(),
                              md->anchor(),name);
           ol.popGeneratorState();
 
@@ -436,11 +557,10 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
         {
           ol.docify(name);
         }
-        if (md->isFunction() || md->isSlot() || md->isPrototype() || md->isSignal()) ol.docify("()");
       }
       index=newIndex+matchLen;
     } 
-    parseText(ol,ldefLine.right(ldefLine.length()-index));
+    ol.parseText(ldefLine.right(ldefLine.length()-index));
     ol.writeString(".");
   }
   ol.popGeneratorState();
@@ -458,9 +578,10 @@ void Definition::writeSourceRefs(OutputList &ol,const char *scopeName)
 
 bool Definition::hasDocumentation() const
 { 
+  static bool extractAll = Config_getBool("EXTRACT_ALL"); 
   return !m_doc.isEmpty() ||             // has detailed docs
          !m_brief.isEmpty() ||           // has brief description
-         Config_getBool("EXTRACT_ALL");       // extract everything
+         extractAll;                     // extract everything
 }
 
 void Definition::addSourceReferencedBy(MemberDef *md)
@@ -504,6 +625,7 @@ void Definition::addSourceReferences(MemberDef *md)
     }
     if (m_sourceRefsDict->find(name)==0)
     {
+      //printf("Adding reference %s->%s\n",md->name().data(),name.data());
       m_sourceRefsDict->inSort(name,md);
     }
   }
@@ -519,8 +641,10 @@ void Definition::addInnerCompound(Definition *)
   err("Error: Definition::addInnerCompound() called\n");
 }
 
-QCString Definition::qualifiedName() const
+QCString Definition::qualifiedName() 
 {
+  if (!m_qualifiedName.isEmpty()) return m_qualifiedName;
+  
   //printf("start Definition::qualifiedName() localName=%s\n",m_localName.data());
   if (m_outerScope==0) 
   {
@@ -528,17 +652,16 @@ QCString Definition::qualifiedName() const
     else return m_localName; 
   }
 
-  QCString qualifiedName;
   if (m_outerScope->name()=="<globalScope>")
   {
-    qualifiedName = m_localName.copy();
+    m_qualifiedName = m_localName.copy();
   }
   else
   {
-    qualifiedName = m_outerScope->qualifiedName()+"::"+m_localName;
+    m_qualifiedName = m_outerScope->qualifiedName()+"::"+m_localName;
   }
   //printf("end Definition::qualifiedName()=%s\n",qualifiedName.data());
-  return qualifiedName;
+  return m_qualifiedName;
 };
 
 QCString Definition::localName() const
@@ -559,4 +682,79 @@ void Definition::makePartOfGroup(GroupDef *gd)
   m_partOfGroups->append(gd);
 }
 
+void Definition::setRefItems(const QList<ListItemInfo> *sli)
+{
+  if (sli)
+  {
+    // deep copy the list
+    if (m_xrefListItems==0) 
+    {
+      m_xrefListItems=new QList<ListItemInfo>;
+      m_xrefListItems->setAutoDelete(TRUE);
+    }
+    QListIterator<ListItemInfo> slii(*sli);
+    ListItemInfo *lii;
+    for (slii.toFirst();(lii=slii.current());++slii)
+    {
+      m_xrefListItems->append(new ListItemInfo(*lii));
+    } 
+  }
+}
+
+void Definition::mergeRefItems(Definition *d)
+{
+  if (d->xrefListItems())
+  {
+    // deep copy the list
+    if (m_xrefListItems==0) 
+    {
+      m_xrefListItems=new QList<ListItemInfo>;
+      m_xrefListItems->setAutoDelete(TRUE);
+    }
+    QListIterator<ListItemInfo> slii(*d->xrefListItems());
+    ListItemInfo *lii;
+    for (slii.toFirst();(lii=slii.current());++slii)
+    {
+      if (getXRefListId(lii->type)==-1)
+      {
+        m_xrefListItems->append(new ListItemInfo(*lii));
+      }
+    } 
+  }
+}
+
+int Definition::getXRefListId(const char *listName) const
+{
+  if (m_xrefListItems)
+  {
+    QListIterator<ListItemInfo> slii(*m_xrefListItems);
+    ListItemInfo *lii;
+    for (slii.toFirst();(lii=slii.current());++slii)
+    {
+      if (strcmp(lii->type,listName)==0)
+      {
+        return lii->itemId;
+      }
+    }
+  }
+  return -1;
+}
+
+const QList<ListItemInfo> *Definition::xrefListItems() const
+{
+  return m_xrefListItems;
+}
+
+
+QCString Definition::convertNameToFile(const char *name,bool allowDots) const
+{
+  if (!m_ref.isEmpty())
+  {
+    return name;
+  }
+  else
+  {
+    return ::convertNameToFile(name,allowDots);
+  }
+}
 
