@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: diagram.cpp,v 1.30 2001/03/19 19:27:40 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -30,8 +30,8 @@
 #include "config.h"
 #include "message.h"
 #include "util.h"
-#include "latexgen.h"
-#include "htmlgen.h"
+//#include "latexgen.h"
+//#include "htmlgen.h"
 #include "doxygen.h"
 
 //-----------------------------------------------------------------------------
@@ -62,6 +62,7 @@ static uint protToMask(Protection p)
   switch(p)
   {
     case Public:    return 0xffffffff;
+    case Package: // package is not possible!
     case Protected: return 0xcccccccc;
     case Private:   return 0xaaaaaaaa;
   }
@@ -73,6 +74,7 @@ static uint protToColor(Protection p)
   switch(p)
   {
     case Public:    return 6;
+    case Package: // package is not possible!
     case Protected: return 5;
     case Private:   return 4;
   }
@@ -84,6 +86,7 @@ static QCString protToString(Protection p)
   switch(p)
   {
     case Public:    return "solid";
+    case Package: // package is not possible!
     case Protected: return "dashed";
     case Private:   return "dotted";
   }
@@ -166,7 +169,7 @@ static void writeMapArea(QTextStream &t,ClassDef *cd,int x,int y,int w,int h)
     {
       if ((dest=Doxygen::tagDestinationDict[ref])) t << *dest << "/";
     }
-    t << cd->getOutputFileBase() << htmlFileExtension << "\" ";
+    t << cd->getOutputFileBase() << Doxygen::htmlFileExtension << "\" ";
     t << "alt=\"" << cd->displayName(); 
     t << "\" shape=\"rect\" coords=\"" << x << "," << y << ",";
     t << (x+w) << "," << (y+h) << "\">" << endl;
