@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: dot.h,v 1.14 2001/03/19 19:27:40 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -16,6 +16,9 @@
  *
  */
 
+#ifndef _DOT_H
+#define _DOT_H
+
 #include "qtbc.h"
 #include <qlist.h>
 #include <qdict.h>
@@ -26,6 +29,7 @@ class FileDef;
 class QTextStream;
 class DotNodeList;
 class ClassSDict;
+class MemberDef;
 
 enum GraphOutputFormat { BITMAP , EPS };
 
@@ -44,21 +48,9 @@ struct EdgeInfo
 
 class DotNode
 {
-  friend class DotGfxHierarchyTable;
-  friend class DotClassGraph;
-  friend class DotInclDepGraph;
-  friend class DotNodeList;
-  friend void writeDotGraph(
-                      DotNode *root,
-                      GraphOutputFormat f,
-                      const QCString &baseName,
-                      bool lrRank,
-                      bool renderParents,
-                      int distance,
-                      bool backArrow
-                     );
   public:
-    DotNode(int n,const char *lab,const char *url,int distance = 0,bool rootNode=FALSE);
+    enum GraphType { Dependency, Inheritance, Collaboration, Hierarchy, CallGraph };
+    DotNode(int n,const char *lab,const char *url,int distance = 0,bool rootNode=FALSE,ClassDef *cd=0);
    ~DotNode();
     void addChild(DotNode *n,
                   int edgeColor=EdgeInfo::Purple,
@@ -69,23 +61,23 @@ class DotNode
                  );
     void setDistance(int distance);
     void addParent(DotNode *n);
-    void deleteNode(DotNodeList &deletedList,SIntDict<DotNode> *skipNodes=0);
+    void deleteNode(DotNodeList &deletedList,SDict<DotNode> *skipNodes=0);
     void removeChild(DotNode *n);
     void removeParent(DotNode *n);
-    void write(QTextStream &t,GraphOutputFormat f,bool topDown,bool toChildren,
-               int maxDistance=1000,bool backArrows=TRUE);
+    void write(QTextStream &t,GraphType gt,GraphOutputFormat f,
+               bool topDown,bool toChildren,int maxDistance,bool backArrows);
     int  m_subgraphId;
     void clearWriteFlag();
-    void writeXML(QTextStream &t);
+    void writeXML(QTextStream &t,bool isClassGraph);
     void writeDEF(QTextStream &t);
     QCString label() const { return m_label; }
     int  number() const { return m_number; }
 
   private:
     void colorConnectedNodes(int curColor);
-    void writeBox(QTextStream &t,GraphOutputFormat f,
+    void writeBox(QTextStream &t,GraphType gt,GraphOutputFormat f,
                   bool hasNonReachableChildren);
-    void writeArrow(QTextStream &t,GraphOutputFormat f,DotNode *cn,
+    void writeArrow(QTextStream &t,GraphType gt,GraphOutputFormat f,DotNode *cn,
                     EdgeInfo *ei,bool topDown, bool pointBack=TRUE);
     const DotNode   *findDocNode() const; // only works for acyclic graphs!
     int              m_number;
@@ -99,6 +91,25 @@ class DotNode
     bool             m_hasDoc;    //!< used to mark a node as documented
     int              m_distance;  //!< distance to the root node
     bool             m_isRoot;    //!< indicates if this is a root node
+    ClassDef *       m_classDef;  //!< class representing this node (can be 0)
+
+    friend class DotGfxHierarchyTable;
+    friend class DotClassGraph;
+    friend class DotInclDepGraph;
+    friend class DotNodeList;
+    friend class DotCallGraph;
+    friend void writeDotGraph(
+                      DotNode *root, GraphType gt,
+                      GraphOutputFormat f, const QCString &baseName,
+                      bool lrRank, bool renderParents,
+                      int distance, bool backArrows
+                     );
+    friend QCString computeMd5Signature(
+                      DotNode *root, GraphType gt,
+                      GraphOutputFormat f, 
+                      bool lrRank, bool renderParents,
+                      int distance, bool backArrows
+                     );
 };
 
 class DotGfxHierarchyTable
@@ -121,12 +132,12 @@ class DotGfxHierarchyTable
 class DotClassGraph
 {
   public:
-    enum GraphType { Interface, Implementation, Inheritance };
-    DotClassGraph(ClassDef *cd,GraphType t,int maxRecusionDepth=1000);
+    //enum GraphType { Interface, Implementation, Inheritance };
+    DotClassGraph(ClassDef *cd,DotNode::GraphType t,int maxRecusionDepth);
    ~DotClassGraph();
     bool isTrivial() const;
     QCString writeGraph(QTextStream &t,GraphOutputFormat f,const char *path,
-                    bool TBRank=TRUE,bool imageMap=TRUE);
+                    const char *relPath, bool TBRank=TRUE,bool imageMap=TRUE);
 
     void writeXML(QTextStream &t);
     void writeDEF(QTextStream &t);
@@ -137,21 +148,22 @@ class DotClassGraph
     void addClass(ClassDef *cd,DotNode *n,int prot,const char *label,
                   int level,const char *usedName,const char *templSpec,
                   bool base);
-    DotNode        *m_startNode;
-    QDict<DotNode> *m_usedNodes;
-    static int      m_curNodeNumber;
-    GraphType       m_graphType;
-    int             m_recDepth;
-    QCString        m_diskName;
-    int             m_maxDistance;
+    DotNode        *   m_startNode;
+    QDict<DotNode> *   m_usedNodes;
+    static int         m_curNodeNumber;
+    DotNode::GraphType m_graphType;
+    int                m_recDepth;
+    QCString           m_diskName;
+    int                m_maxDistance;
 };
 
 class DotInclDepGraph
 {
   public:
-    DotInclDepGraph(FileDef *fd,bool inverse=FALSE);
+    DotInclDepGraph(FileDef *fd,int maxRecusionDepth,bool inverse);
    ~DotInclDepGraph();
     QCString writeGraph(QTextStream &t, GraphOutputFormat f,const char *path,
+                    const char *relPath,
                     bool writeImageMap=TRUE);
     bool isTrivial() const;
     QCString diskName() const;
@@ -165,9 +177,32 @@ class DotInclDepGraph
     QCString        m_diskName;
     int             m_maxDistance;
     bool            m_inverse;
+    int             m_recDepth;
+};
+
+class DotCallGraph
+{
+  public:
+    DotCallGraph(MemberDef *md,int maxRecursionDepth);
+   ~DotCallGraph();
+    QCString writeGraph(QTextStream &t, GraphOutputFormat f,
+                        const char *path,const char *relPath,bool writeImageMap=TRUE);
+    void buildGraph(DotNode *n,MemberDef *md,int distance);
+    bool isTrivial() const;
+    
+  private:
+    DotNode        *m_startNode;
+    static int      m_curNodeNumber;
+    QDict<DotNode> *m_usedNodes;
+    int             m_maxDistance;
+    int             m_recDepth;
+    QCString        m_diskName;
 };
 
 void generateGraphLegend(const char *path);
-void writeDotGraphFromFile(const char *inFile,const char *outFile,
-                      GraphOutputFormat format);
+void writeDotGraphFromFile(const char *inFile,const char *outDir,
+                           const char *outFile,GraphOutputFormat format);
+QString getDotImageMapFromFile(const QString& inFile, const QString& outDir);
 
+
+#endif
