@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: filedef.cpp,v 1.55 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -19,7 +19,6 @@
 #include "memberlist.h"
 #include "classlist.h"
 #include "filedef.h"
-#include "doc.h"
 #include "doxygen.h"
 #include "memberdef.h"
 #include "classdef.h"
@@ -30,20 +29,46 @@
 #include "dot.h"
 #include "message.h"
 #include "code.h"
+#include "docparser.h"
+#include "ftvhelp.h"
+#include "searchindex.h"
 //#include "xml.h"
 
+class DevNullCodeDocInterface : public BaseCodeDocInterface
+{
+  public:
+    virtual void codify(const char *) {}
+    virtual void writeCodeLink(const char *ref,const char *file,
+                               const char *anchor,const char *name) 
+    { ref=ref; file=file; anchor=anchor; name=name; }
+    virtual void writeLineNumber(const char *,const char *,
+                                 const char *,int) {}
+    virtual void startCodeLine() {}
+    virtual void endCodeLine() {}
+    virtual void startCodeAnchor(const char *) {}
+    virtual void endCodeAnchor() {}
+    virtual void startFontClass(const char *) {}
+    virtual void endFontClass() {}
+    virtual void writeCodeAnchor(const char *) {}
+};
+
+
+
 /*! create a new file definition, where \a p is the file path, 
-    \a the file name, and \a ref is an HTML anchor name if the
+    \a nm the file name, and \a ref is an HTML anchor name if the
     file was read from a tag file or 0 otherwise
 */
-FileDef::FileDef(const char *p,const char *nm,const char *lref)
+FileDef::FileDef(const char *p,const char *nm,
+                 const char *lref,const char *dn)
    : Definition((QCString)p+nm,1,nm)
 {
   path=p;
   filepath=path+nm;
   filename=nm;
-  diskname=nm;
+  diskname=dn;
+  if (diskname.isEmpty()) diskname=nm;
   setReference(lref);
+  //printf("new FileDef(path=%s,name=%s,ref=%s)\n",p,nm,lref);
   classSDict     = new ClassSDict(17);
   includeList   = new QList<IncludeInfo>;
   includeList->setAutoDelete(TRUE);
@@ -73,6 +98,8 @@ FileDef::~FileDef()
   delete classSDict;
   delete includeDict;
   delete includeList;
+  delete includedByDict;
+  delete includedByList;
   delete namespaceSDict;
   delete srcDefDict;
   delete srcMemberDict;
@@ -89,6 +116,7 @@ void FileDef::computeAnchors()
 
 void FileDef::distributeMemberGroupDocumentation()
 {
+  //printf("FileDef::distributeMemberGroupDocumentation()\n");
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
   MemberGroup *mg;
   for (;(mg=mgli.current());++mgli)
@@ -97,6 +125,77 @@ void FileDef::distributeMemberGroupDocumentation()
   }
 }
 
+void FileDef::findSectionsInDocumentation()
+{
+  docFindSections(documentation(),this,0,docFile());
+  MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+  MemberGroup *mg;
+  for (;(mg=mgli.current());++mgli)
+  {
+    mg->findSectionsInDocumentation();
+  }
+  decDefineMembers.findSectionsInDocumentation();
+  decProtoMembers.findSectionsInDocumentation();
+  decTypedefMembers.findSectionsInDocumentation();
+  decEnumMembers.findSectionsInDocumentation();
+  decFuncMembers.findSectionsInDocumentation();
+  decVarMembers.findSectionsInDocumentation();
+}
+
+void FileDef::writeDetailedDocumentation(OutputList &ol)
+{
+  if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) || 
+      !documentation().isEmpty() 
+     )
+  {
+    ol.writeRuler();
+    ol.pushGeneratorState();
+    ol.disable(OutputGenerator::Latex);
+    ol.disable(OutputGenerator::RTF);
+    ol.writeAnchor(0,"_details"); 
+    ol.popGeneratorState();
+    ol.startGroupHeader();
+    ol.parseText(theTranslator->trDetailedDescription());
+    ol.endGroupHeader();
+    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
+    {
+      ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
+    }
+    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") && 
+        !documentation().isEmpty())
+    {
+      ol.pushGeneratorState();
+        ol.disable(OutputGenerator::Man);
+        ol.newParagraph();
+        ol.enableAll();
+        ol.disableAllBut(OutputGenerator::Man);
+        ol.writeString("\n\n");
+      ol.popGeneratorState();
+    }
+    if (!documentation().isEmpty())
+    {
+      //if (doc.at(dl-1)!='.' && doc.at(dl-1)!='!' && doc.at(dl-1)!='?') 
+      //  doc+='.';
+      ol.parseDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
+    }
+    //printf("Writing source ref for file %s\n",name().data());
+    if (Config_getBool("SOURCE_BROWSER")) 
+    {
+      ol.newParagraph();
+      QCString refText = theTranslator->trDefinedInSourceFile();
+      int fileMarkerPos = refText.find("@0");
+      if (fileMarkerPos!=-1) // should always pass this.
+      {
+        ol.parseText(refText.left(fileMarkerPos)); //text left from marker 1
+        ol.writeObjectLink(0,getSourceFileBase(),
+            0,name());
+        ol.parseText(refText.right(
+              refText.length()-fileMarkerPos-2)); // text right from marker 2
+      }
+    }
+  }
+}
+
 /*! Write the documentation page for this file to the file of output
     generators \a ol. 
 */
@@ -115,33 +214,40 @@ void FileDef::writeDocumentation(OutputList &ol)
   QCString pageTitle=name()+" File Reference";
   startFile(ol,getOutputFileBase(),name(),pageTitle);
   startTitle(ol,getOutputFileBase());
-  parseText(ol,theTranslator->trFileReference(docname));
+  ol.parseText(theTranslator->trFileReference(docname));
   addGroupListToTitle(ol,this);
   endTitle(ol,getOutputFileBase(),docName());
   //ol.newParagraph();
   
+  if (Config_getBool("SEARCHENGINE"))
+  {
+    Doxygen::searchIndex->setCurrentDoc(pageTitle,getOutputFileBase());
+    Doxygen::searchIndex->addWord(localName().lower());
+  }
+  
   if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
   {
     Doxygen::tagFile << "  <compound kind=\"file\">" << endl;
     Doxygen::tagFile << "    <name>" << convertToXML(name()) << "</name>" << endl;
     Doxygen::tagFile << "    <path>" << convertToXML(getPath()) << "</path>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << htmlFileExtension << "</filename>" << endl;
+    Doxygen::tagFile << "    <filename>" 
+                     << convertToXML(getOutputFileBase()) 
+                     << "</filename>" << endl;
   }
   
   ol.startTextBlock();
-  //brief=brief.stripWhiteSpace();
-  //int bl=brief.length();
-  OutputList briefOutput(&ol); 
-  if (briefDescription()) 
+  if (Config_getBool("DETAILS_AT_TOP"))
   {
-    //if (brief.at(bl-1)!='.' && brief.at(bl-1)!='!' && brief.at(bl!='?')) 
-    //  brief+='.';
-    parseDoc(briefOutput,filepath,1,0,0,briefDescription());
-    ol+=briefOutput;
+    writeDetailedDocumentation(ol);
+    ol.newParagraph();
+  }
+  else if (briefDescription()) 
+  {
+    ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
     ol.writeString(" \n");
     ol.disableAllBut(OutputGenerator::Html);
     ol.startTextLink(0,"_details");
-    parseText(ol,theTranslator->trMore());
+    ol.parseText(theTranslator->trMore());
     ol.endTextLink();
     ol.enableAll();
     ol.disable(OutputGenerator::Man);
@@ -162,6 +268,7 @@ void FileDef::writeDocumentation(OutputList &ol)
       if (fd)
       {
         isIDLorJava = fd->name().right(4)==".idl" || 
+                      fd->name().right(5)==".pidl" ||
                       fd->name().right(5)==".java";
       }
       ol.startTypewriter();
@@ -185,9 +292,23 @@ void FileDef::writeDocumentation(OutputList &ol)
       // Here we use the include file name as it appears in the file.
       // we could also we the name as it is used within doxygen,
       // then we should have used fd->docName() instead of ii->includeName
-      if (fd && fd->isLinkable() && fd->generateSourceFile())
+      if (fd && fd->isLinkable())
       {
-        ol.writeObjectLink(fd->getReference(),fd->includeName(),0,ii->includeName);
+        ol.writeObjectLink(fd->getReference(),
+            fd->generateSourceFile() ? fd->includeName() : fd->getOutputFileBase(),
+            0,ii->includeName);
+        if (!Config_getString("GENERATE_TAGFILE").isEmpty() && !fd->isReference()) 
+        {
+          const char *locStr = (ii->local || isIDLorJava) ? "yes" : "no";
+          Doxygen::tagFile << "    <includes id=\"" 
+                           << convertToXML(fd->getOutputFileBase()) 
+                           << "\" name=\"" 
+                           << convertToXML(fd->name())
+                           << "\" local=\"" << locStr << "\">" 
+                           << convertToXML(ii->includeName)
+                           << "</includes>" 
+                           << endl;
+        }
       }
       else
       {
@@ -215,13 +336,13 @@ void FileDef::writeDocumentation(OutputList &ol)
   if (Config_getBool("HAVE_DOT") && Config_getBool("INCLUDE_GRAPH"))
   {
     //printf("Graph for file %s\n",name().data());
-    DotInclDepGraph incDepGraph(this,FALSE);
+    DotInclDepGraph incDepGraph(this,Config_getInt("MAX_DOT_GRAPH_DEPTH"),FALSE);
     if (!incDepGraph.isTrivial())
     {
       ol.disable(OutputGenerator::Man);
       ol.newParagraph();
       ol.startInclDepGraph();
-      parseText(ol,theTranslator->trInclDepGraph(name()));
+      ol.parseText(theTranslator->trInclDepGraph(name()));
       ol.endInclDepGraph(incDepGraph);
       ol.enableAll();
     }
@@ -231,13 +352,13 @@ void FileDef::writeDocumentation(OutputList &ol)
   if (Config_getBool("HAVE_DOT") && Config_getBool("INCLUDED_BY_GRAPH"))
   {
     //printf("Graph for file %s\n",name().data());
-    DotInclDepGraph incDepGraph(this,TRUE);
+    DotInclDepGraph incDepGraph(this,Config_getInt("MAX_DOT_GRAPH_DEPTH"),TRUE);
     if (!incDepGraph.isTrivial())
     {
       ol.disable(OutputGenerator::Man);
       ol.newParagraph();
       ol.startInclDepGraph();
-      parseText(ol,theTranslator->trInclByDepGraph());
+      ol.parseText(theTranslator->trInclByDepGraph());
       ol.endInclDepGraph(incDepGraph);
       ol.enableAll();
     }
@@ -250,18 +371,11 @@ void FileDef::writeDocumentation(OutputList &ol)
     ol.disableAllBut(OutputGenerator::Html);
     ol.newParagraph();
     ol.startTextLink(includeName(),0);
-    parseText(ol,theTranslator->trGotoSourceCode());
+    ol.parseText(theTranslator->trGotoSourceCode());
     ol.endTextLink();
     ol.enableAll();
   }
   
-  //ol.disableAllBut(OutputGenerator::Html);
-  //ol.writeString((QCString)"<p>Interface collaboration diagram for "
-  //               "<a href=\"usage_intf_graph_"+name()+htmlFileExtension+"\">here</a>");
-  //ol.writeString((QCString)"<p>Include dependency diagram for "+fn+" can be found "+
-  //               "<a href=\""+diskname+"_incldep+htmlFileExtension+"\">here</a>.");
-  //ol.enableAll();
-
   ol.endTextBlock();
   
   ol.startMemberSections();
@@ -278,7 +392,7 @@ void FileDef::writeDocumentation(OutputList &ol)
         if (!found)
         {
           ol.startMemberHeader();
-          parseText(ol,theTranslator->trNamespaces());
+          ol.parseText(theTranslator->trNamespaces());
           ol.endMemberHeader();
           ol.startMemberList();
           found=TRUE;
@@ -304,7 +418,7 @@ void FileDef::writeDocumentation(OutputList &ol)
           ol.docify(nd->displayName());
           ol.endBold();
         }
-        ol.endMemberItem(FALSE);
+        ol.endMemberItem();
       }
     }
     if (found) ol.endMemberList();
@@ -332,56 +446,9 @@ void FileDef::writeDocumentation(OutputList &ol)
   decVarMembers.writeDeclarations(ol,0,0,this,0,theTranslator->trVariables(),0);
   ol.endMemberSections();
 
-  //doc=doc.stripWhiteSpace();
-  //int bl=brief.length();
-  //int dl=doc.length();
-  if ((!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF")) || 
-      !documentation().isEmpty() 
-      /* || startBodyLine!=-1 */
-     )
+  if (!Config_getBool("DETAILS_AT_TOP"))
   {
-    ol.writeRuler();
-    ol.pushGeneratorState();
-    //bool latexOn = ol.isEnabled(OutputGenerator::Latex);
-    //if (latexOn) ol.disable(OutputGenerator::Latex);
-    ol.disable(OutputGenerator::Latex);
-    ol.disable(OutputGenerator::RTF);
-    ol.writeAnchor(0,"_details"); 
-    //if (latexOn) ol.enable(OutputGenerator::Latex);
-    ol.popGeneratorState();
-    ol.startGroupHeader();
-    parseText(ol,theTranslator->trDetailedDescription());
-    ol.endGroupHeader();
-    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
-    {
-      ol+=briefOutput;
-    }
-    if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") && 
-        !documentation().isEmpty())
-    {
-      ol.newParagraph();
-    }
-    if (!documentation().isEmpty())
-    {
-      //if (doc.at(dl-1)!='.' && doc.at(dl-1)!='!' && doc.at(dl-1)!='?') 
-      //  doc+='.';
-      parseDoc(ol,filepath,1,0,0,documentation()+"\n");
-    }
-    //printf("Writing source ref for file %s\n",name().data());
-    if (Config_getBool("SOURCE_BROWSER")) 
-    {
-      ol.newParagraph();
-      QCString refText = theTranslator->trDefinedInSourceFile();
-      int fileMarkerPos = refText.find("@0");
-      if (fileMarkerPos!=-1) // should always pass this.
-      {
-        parseText(ol,refText.left(fileMarkerPos)); //text left from marker 1
-        ol.writeObjectLink(0,getSourceFileBase(),
-            0,name());
-        parseText(ol,refText.right(
-              refText.length()-fileMarkerPos-2)); // text right from marker 2
-      }
-    }
+    writeDetailedDocumentation(ol);
   }
 
   docDefineMembers.writeDocumentation(ol,name(),this,
@@ -405,9 +472,9 @@ void FileDef::writeDocumentation(OutputList &ol)
   // write Author section (Man only)
   ol.disableAllBut(OutputGenerator::Man);
   ol.startGroupHeader();
-  parseText(ol,theTranslator->trAuthor(TRUE,TRUE));
+  ol.parseText(theTranslator->trAuthor(TRUE,TRUE));
   ol.endGroupHeader();
-  parseText(ol,theTranslator->trGeneratedAutomatically(Config_getString("PROJECT_NAME")));
+  ol.parseText(theTranslator->trGeneratedAutomatically(Config_getString("PROJECT_NAME")));
   ol.enableAll();
 
   if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
@@ -425,13 +492,13 @@ void FileDef::writeSource(OutputList &ol)
   ol.disableAllBut(OutputGenerator::Html);
   startFile(ol,getSourceFileBase(),0,docname+" Source File");
   startTitle(ol,0);
-  parseText(ol,docname);
+  ol.parseText(docname);
   endTitle(ol,0,0);
 
   if (isLinkable())
   {
     ol.startTextLink(getOutputFileBase(),0);
-    parseText(ol,theTranslator->trGotoDocumentation());
+    ol.parseText(theTranslator->trGotoDocumentation());
     ol.endTextLink();
   }
 
@@ -446,15 +513,23 @@ void FileDef::writeSource(OutputList &ol)
   ol.enableAll();
 }
 
+void FileDef::parseSource()
+{
+  DevNullCodeDocInterface devNullIntf;
+  parseCode(devNullIntf,0,
+            fileToString(absFilePath(),Config_getBool("FILTER_SOURCE_FILES")),
+            FALSE,0,this
+           );
+}
 
 void FileDef::addMembersToMemberGroup()
 {
-  ::addMembersToMemberGroup(&decDefineMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decProtoMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decTypedefMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decEnumMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decFuncMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decVarMembers,memberGroupSDict);
+  ::addMembersToMemberGroup(&decDefineMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decProtoMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decTypedefMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decEnumMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decFuncMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decVarMembers,memberGroupSDict,this);
 }
 
 /*! Adds member definition \a md to the list of all members of this file */
@@ -462,33 +537,46 @@ void FileDef::insertMember(MemberDef *md)
 {
   //printf("%s:FileDef::insertMember(%s)\n",name().data(),md->name().data());
   allMemberList.append(md); 
+  bool sortBriefDocs = Config_getBool("SORT_BRIEF_DOCS");
   bool sortMemberDocs = Config_getBool("SORT_MEMBER_DOCS");
   switch(md->memberType())
   {
     case MemberDef::Variable:     
     case MemberDef::Property:     
-      decVarMembers.append(md);
+      if (sortBriefDocs)
+        decVarMembers.inSort(md);
+      else
+        decVarMembers.append(md);
       if (sortMemberDocs)
         docVarMembers.inSort(md); 
       else
         docVarMembers.append(md);
       break;
     case MemberDef::Function: 
-      decFuncMembers.append(md);
+      if (sortBriefDocs)
+        decFuncMembers.inSort(md);
+      else
+        decFuncMembers.append(md);
       if (sortMemberDocs)    
         docFuncMembers.inSort(md); 
       else
         docFuncMembers.append(md);
       break;
     case MemberDef::Typedef:      
-      decTypedefMembers.append(md);
+      if (sortBriefDocs)
+        decTypedefMembers.inSort(md);
+      else
+        decTypedefMembers.append(md);
       if (sortMemberDocs)
         docTypedefMembers.inSort(md); 
       else
         docTypedefMembers.append(md);
       break;
     case MemberDef::Enumeration:  
-      decEnumMembers.append(md);
+      if (sortBriefDocs)
+        decEnumMembers.inSort(md);
+      else
+        decEnumMembers.append(md);
       if (sortMemberDocs)
         docEnumMembers.inSort(md); 
       else
@@ -497,14 +585,20 @@ void FileDef::insertMember(MemberDef *md)
     case MemberDef::EnumValue:    // enum values are shown inside their enums
       break;
     case MemberDef::Prototype:    
-      decProtoMembers.append(md);
+      if (sortBriefDocs)
+        decProtoMembers.inSort(md);
+      else
+        decProtoMembers.append(md);
       if (sortMemberDocs)
         docProtoMembers.inSort(md); 
       else
         docProtoMembers.append(md);
       break;
     case MemberDef::Define:       
-      decDefineMembers.append(md);
+      if (sortBriefDocs)
+        decDefineMembers.inSort(md);
+      else
+        decDefineMembers.append(md);
       if (sortMemberDocs)
         docDefineMembers.inSort(md); 
       else
@@ -523,7 +617,7 @@ void FileDef::insertMember(MemberDef *md)
 /*! Adds compound definition \a cd to the list of all compounds of this file */
 void FileDef::insertClass(ClassDef *cd)
 {
-  if (Config_getBool("SORT_MEMBER_DOCS"))
+  if (Config_getBool("SORT_BRIEF_DOCS"))
     classSDict->inSort(cd->name(),cd);
   else
     classSDict->append(cd->name(),cd);
@@ -534,7 +628,7 @@ void FileDef::insertNamespace(NamespaceDef *nd)
 {
   if (!nd->name().isEmpty() && namespaceSDict->find(nd->name())==0)
   {
-    if (Config_getBool("SORT_MEMBER_DOCS"))
+    if (Config_getBool("SORT_BRIEF_DOCS"))
       namespaceSDict->inSort(nd->name(),nd);
     else
       namespaceSDict->append(nd->name(),nd);
@@ -574,22 +668,29 @@ MemberDef *FileDef::getSourceMember(int lineNr)
   return result;
 }
 
+
 void FileDef::addUsingDirective(NamespaceDef *nd)
 {
   if (usingDirList==0)
   {
-    usingDirList = new NamespaceList;
+    usingDirList = new NamespaceSDict;
+  }
+  if (usingDirList->find(nd->qualifiedName())==0)
+  {
+    usingDirList->append(nd->qualifiedName(),nd);
   }
-  usingDirList->append(nd);
 }
 
-void FileDef::addUsingDeclaration(ClassDef *cd)
+void FileDef::addUsingDeclaration(Definition *d)
 {
   if (usingDeclList==0)
   {
-    usingDeclList = new ClassList;
+    usingDeclList = new SDict<Definition>(17);
+  }
+  if (usingDeclList->find(d->qualifiedName())==0)
+  {
+    usingDeclList->append(d->qualifiedName(),d);
   }
-  usingDeclList->append(cd);
 }
 
 void FileDef::addIncludeDependency(FileDef *fd,const char *incName,bool local)
@@ -607,6 +708,57 @@ void FileDef::addIncludeDependency(FileDef *fd,const char *incName,bool local)
   }
 }
 
+void FileDef::addIncludedUsingDirectives()
+{
+  if (visited) return;
+  visited=TRUE;
+  //printf("( FileDef::addIncludedUsingDirectives for file %s\n",name().data());
+
+  NamespaceList nl;
+  if (includeList) // file contains #includes
+  {
+    {
+      QListIterator<IncludeInfo> iii(*includeList);
+      IncludeInfo *ii;
+      for (iii.toFirst();(ii=iii.current());++iii) // foreach #include...
+      {
+        if (ii->fileDef && !ii->fileDef->visited) // ...that is a known file
+        {
+          // recurse into this file
+          ii->fileDef->addIncludedUsingDirectives();
+        }
+      }
+    }
+    {
+      QListIterator<IncludeInfo> iii(*includeList);
+      IncludeInfo *ii;
+      // iterate through list from last to first
+      for (iii.toLast();(ii=iii.current());--iii)
+      {
+        if (ii->fileDef && ii->fileDef!=this)
+        {
+          NamespaceSDict *unl = ii->fileDef->usingDirList;
+          if (unl)
+          {
+            NamespaceSDict::Iterator nli(*unl);
+            NamespaceDef *nd;
+            for (nli.toLast();(nd=nli.current());--nli)
+            {
+              // append each using directive found in a #include file
+              if (usingDirList==0) usingDirList = new NamespaceSDict;
+              //printf("Prepending used namespace %s to the list of file %s\n",
+              //    nd->name().data(),name().data());
+              usingDirList->prepend(nd->qualifiedName(),nd);
+            }
+          }
+        }
+      }
+    }
+  }
+  //printf(") end FileDef::addIncludedUsingDirectives for file %s\n",name().data());
+}
+
+
 void FileDef::addIncludedByDependency(FileDef *fd,const char *incName,bool local)
 {
   //printf("FileDef::addIncludedByDependency(%p,%s,%d)\n",fd,incName,local);
@@ -622,6 +774,12 @@ void FileDef::addIncludedByDependency(FileDef *fd,const char *incName,bool local
   }
 }
 
+bool FileDef::isIncluded(const QCString &name) const
+{
+  if (name.isEmpty()) return FALSE;
+  return includeDict->find(name)!=0;
+}
+
 bool FileDef::generateSourceFile() const 
 { 
   QCString extension = name().right(4);
@@ -635,7 +793,7 @@ bool FileDef::generateSourceFile() const
 
 void FileDef::addListReferences()
 {
-  addRefItem(todoId(),testId(),bugId(),
+  addRefItem(xrefListItems(),
              theTranslator->trFile(TRUE,TRUE),
              getOutputFileBase(),name()
             );
@@ -653,3 +811,258 @@ void FileDef::addListReferences()
   docVarMembers.addListReferences(this);
 }
 
+//-------------------------------------------------------------------
+
+static int findMatchingPart(const QCString &path,const QCString dir)
+{
+  int si1;
+  int pos1=0,pos2=0;
+  while ((si1=path.find('/',pos1))!=-1)
+  {
+    int si2=dir.find('/',pos2);
+    //printf("  found slash at pos %d in path %d: %s<->%s\n",si1,si2,
+    //    path.mid(pos1,si1-pos2).data(),dir.mid(pos2).data());
+    if (si2==-1 && path.mid(pos1,si1-pos2)==dir.mid(pos2)) // match at end
+    {
+      return dir.length();
+    }
+    if (si1!=si2 || path.mid(pos1,si1-pos2)!=dir.mid(pos2,si2-pos2)) // no match in middle
+    {
+      return QMAX(pos1-1,0);
+    }
+    pos1=si1+1;
+    pos2=si2+1;
+  }
+  return 0;
+}
+
+static Directory *findDirNode(Directory *root,const QCString &name)
+{
+  QListIterator<DirEntry> dli(root->children());
+  DirEntry *de;
+  for (dli.toFirst();(de=dli.current());++dli)
+  {
+    if (de->kind()==DirEntry::Dir)
+    {
+      Directory *dir = (Directory *)de;
+      QCString dirName=dir->name();
+      int sp=findMatchingPart(name,dirName);
+      //printf("findMatchingPart(%s,%s)=%d\n",name.data(),dirName.data(),sp);
+      if (sp>0) // match found
+      {
+        if ((uint)sp==dirName.length()) // whole directory matches
+        {
+          // recurse into the directory
+          return findDirNode(dir,name.mid(dirName.length()+1));
+        } 
+        else // partial match => we need to split the path into three parts
+        {
+          QCString baseName     =dirName.left(sp);
+          QCString oldBranchName=dirName.mid(sp+1);
+          QCString newBranchName=name.mid(sp+1);
+          // strip file name from path
+          int newIndex=newBranchName.findRev('/');
+          if (newIndex>0) newBranchName=newBranchName.left(newIndex);
+
+          //printf("Splitting off part in new branch \n"
+          //    "base=%s old=%s new=%s\n",
+          //    baseName.data(),
+          //    oldBranchName.data(),
+          //    newBranchName.data()
+          //      );
+          Directory *base = new Directory(root,baseName);
+          Directory *newBranch = new Directory(base,newBranchName);
+          dir->reParent(base);
+          dir->rename(oldBranchName);
+          base->addChild(dir);
+          base->addChild(newBranch);
+          dir->setLast(FALSE);
+          // remove DirEntry container from list (without deleting it)
+          root->children().setAutoDelete(FALSE);
+          root->children().removeRef(dir);
+          root->children().setAutoDelete(TRUE);
+          // add new branch to the root
+          if (!root->children().isEmpty())
+          {
+            root->children().last()->setLast(FALSE); 
+          }
+          root->addChild(base);
+          return newBranch;
+        }
+      }
+    }
+  }
+  int si=name.findRev('/');
+  if (si==-1) // no subdir
+  {
+    return root; // put the file under the root node.
+  }
+  else // need to create a subdir 
+  {
+    QCString baseName = name.left(si);
+    //printf("new subdir %s\n",baseName.data());
+    Directory *newBranch = new Directory(root,baseName);
+    if (!root->children().isEmpty())
+    {
+      root->children().last()->setLast(FALSE); 
+    }
+    root->addChild(newBranch);
+    return newBranch;
+  }
+}
+
+static void mergeFileDef(Directory *root,FileDef *fd)
+{
+  QCString rootPath = root->name();
+  QCString filePath = fd->absFilePath();
+  //printf("merging %s\n",filePath.data());
+  Directory *dirNode = findDirNode(root,filePath);
+  if (!dirNode->children().isEmpty())
+  {
+    dirNode->children().last()->setLast(FALSE); 
+  }
+  DirEntry *e=new DirEntry(dirNode,fd);
+  dirNode->addChild(e);
+}
+
+static void generateIndent(QTextStream &t,DirEntry *de,int level)
+{
+  if (de->parent())
+  {
+    generateIndent(t,de->parent(),level+1);
+  }
+  // from the root up to node n do...
+  if (level==0) // item before a dir or document
+  {
+    if (de->isLast())
+    {
+      if (de->kind()==DirEntry::Dir)
+      {
+        t << "<img " << FTV_IMGATTRIBS(plastnode) << "/>";
+      }
+      else
+      {
+        t << "<img " << FTV_IMGATTRIBS(lastnode) << "/>";
+      }
+    }
+    else
+    {
+      if (de->kind()==DirEntry::Dir)
+      {
+        t << "<img " << FTV_IMGATTRIBS(pnode) << "/>";
+      }
+      else
+      {
+        t << "<img " << FTV_IMGATTRIBS(node) << "/>";
+      }
+    }
+  }
+  else // item at another level
+  {
+    if (de->isLast())
+    {
+      t << "<img " << FTV_IMGATTRIBS(blank) << "/>";
+    }
+    else
+    {
+      t << "<img " << FTV_IMGATTRIBS(vertline) << "/>";
+    }
+  }
+}
+
+
+static void writeDirTreeNode(QTextStream &t,Directory *root,int level)
+{
+  QCString indent;
+  indent.fill(' ',level*2);
+  QListIterator<DirEntry> dli(root->children());
+  DirEntry *de;
+  for (dli.toFirst();(de=dli.current());++dli)
+  {
+    t << indent << "<p>";
+    generateIndent(t,de,0);
+    if (de->kind()==DirEntry::Dir)
+    {
+      Directory *dir=(Directory *)de;
+      //printf("%s [dir]: %s (last=%d,dir=%d)\n",indent.data(),dir->name().data(),dir->isLast(),dir->kind()==DirEntry::Dir);
+      t << "<img " << FTV_IMGATTRIBS(folderclosed) << "/>";
+      t << dir->name();
+      t << "</p>\n";
+      t << indent << "<div>\n";
+      writeDirTreeNode(t,dir,level+1);
+      t << indent << "</div>\n";
+    }
+    else
+    {
+      //printf("%s [file]: %s (last=%d,dir=%d)\n",indent.data(),de->file()->name().data(),de->isLast(),de->kind()==DirEntry::Dir);
+      t << "<img " << FTV_IMGATTRIBS(doc) << "/>";
+      t << de->file()->name();
+      t << "</p>\n";
+    }
+  }
+}
+
+void generateFileTree(QTextStream &t)
+{
+  FTVHelp::generateTreeViewImages();
+  
+  Directory *root=new Directory(0,"");
+  root->setLast(TRUE);
+  FileNameListIterator fnli(Doxygen::inputNameList); 
+  FileName *fn;
+  for (fnli.toFirst();(fn=fnli.current());++fnli)
+  {
+    FileNameIterator fni(*fn);
+    FileDef *fd;
+    for (;(fd=fni.current());++fni)
+    {
+      mergeFileDef(root,fd);
+    }
+  }
+  t << "<div class=\"directory\">\n";
+  writeDirTreeNode(t,root,0);
+  t << "</div>\n";
+  delete root;
+}
+
+void FileDef::combineUsingRelations()
+{
+  if (visited) return; // already done
+  visited=TRUE;
+  if (usingDirList)
+  {
+    NamespaceSDict::Iterator nli(*usingDirList);
+    NamespaceDef *nd;
+    for (nli.toFirst();(nd=nli.current());++nli)
+    {
+      nd->combineUsingRelations();
+    }
+    for (nli.toFirst();(nd=nli.current());++nli)
+    {
+      // add used namespaces of namespace nd to this namespace
+      if (nd->getUsedNamespaces())
+      {
+        NamespaceSDict::Iterator unli(*nd->getUsedNamespaces());
+        NamespaceDef *und;
+        for (unli.toFirst();(und=unli.current());++unli)
+        {
+          //printf("Adding namespace %s to the using list of %s\n",und->qualifiedName().data(),qualifiedName().data());
+          addUsingDirective(und);
+        }
+      }
+      // add used classes of namespace nd to this namespace
+      if (nd->getUsedClasses())
+      {
+        SDict<Definition>::Iterator cli(*nd->getUsedClasses());
+        Definition *ucd;
+        for (cli.toFirst();(ucd=cli.current());++cli)
+        {
+          //printf("Adding class %s to the using list of %s\n",cd->qualifiedName().data(),qualifiedName().data());
+          addUsingDeclaration(ucd);
+        }
+      }
+    }
+  }
+}
+
+
