@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: filedef.h,v 1.32 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -65,7 +65,7 @@ class FileDef : public Definition
   public:
     //enum FileType { Source, Header, Unknown };
 
-    FileDef(const char *p,const char *n,const char *ref=0);
+    FileDef(const char *p,const char *n,const char *ref=0,const char *dn=0);
    ~FileDef();
     DefType definitionType() { return TypeFile; }
 
@@ -117,9 +117,12 @@ class FileDef : public Definition
     {
       return isLinkableInProject() || isReference();
     }
+    bool isIncluded(const QCString &name) const;
 
+    void writeDetailedDocumentation(OutputList &ol);
     void writeDocumentation(OutputList &ol);
     void writeSource(OutputList &ol);
+    void parseSource();
     friend void generatedFileNames();
     void insertMember(MemberDef *md);
     void insertClass(ClassDef *cd);
@@ -130,9 +133,10 @@ class FileDef : public Definition
     PackageDef *packageDef() const { return package; }
     
     void addUsingDirective(NamespaceDef *nd);
-    NamespaceList *getUsedNamespaces() const { return usingDirList; }
-    void addUsingDeclaration(ClassDef *cd);
-    ClassList *getUsedClasses() const { return usingDeclList; }
+    NamespaceSDict *getUsedNamespaces() const { return usingDirList; }
+    void addUsingDeclaration(Definition *def);
+    SDict<Definition> *getUsedClasses() const { return usingDeclList; }
+    void combineUsingRelations();
 
     bool generateSourceFile() const;
 
@@ -143,6 +147,8 @@ class FileDef : public Definition
 
     void addMembersToMemberGroup();
     void distributeMemberGroupDocumentation();
+    void findSectionsInDocumentation();
+    void addIncludedUsingDirectives();
 
     void addListReferences();
 
@@ -172,6 +178,8 @@ class FileDef : public Definition
 
     NamespaceSDict *namespaceSDict;
     ClassSDict *classSDict;
+    
+    bool visited;
 
   private: 
     
@@ -179,8 +187,8 @@ class FileDef : public Definition
     QList<IncludeInfo> *includeList;
     QDict<IncludeInfo> *includedByDict;
     QList<IncludeInfo> *includedByList;
-    NamespaceList *usingDirList;
-    ClassList *usingDeclList;
+    NamespaceSDict *usingDirList;
+    SDict<Definition> *usingDeclList;
     //DefineList *defineList;
     QCString path;
     QCString filepath;
@@ -234,4 +242,51 @@ class OutputNameDict : public QDict<FileList>
    ~OutputNameDict() {}
 };
 
+class Directory;
+
+class DirEntry
+{
+  public:
+    enum EntryKind { Dir, File };
+    DirEntry(DirEntry *parent,FileDef *fd)  
+       : m_parent(parent), m_kind(File), m_fd(fd), m_isLast(FALSE) { }
+    DirEntry(DirEntry *parent)              
+       : m_parent(parent), m_kind(Dir), m_fd(0), m_isLast(FALSE) { }
+    virtual ~DirEntry() { }
+    EntryKind kind() const { return m_kind; }
+    FileDef *file()  const { return m_fd; }
+    bool isLast() const    { return m_isLast; }
+    void setLast(bool b)   { m_isLast=b; }
+    DirEntry *parent() const { return m_parent; }
+
+  protected:
+    DirEntry *m_parent;
+  private:
+    EntryKind m_kind;
+    FileDef   *m_fd;
+    int num;
+    bool m_isLast;
+};
+
+class Directory : public DirEntry
+{
+  public:
+    Directory(Directory *parent,const QCString &name) 
+       : DirEntry(parent), m_name(name)
+    { m_children.setAutoDelete(TRUE); }
+    virtual ~Directory()              {}
+    void addChild(DirEntry *d)        { m_children.append(d); d->setLast(TRUE); }
+    QList<DirEntry> &children()       { return m_children; }
+    void rename(const QCString &name) { m_name=name; }
+    void reParent(Directory *parent)  { m_parent=parent; }
+    QCString name() const             { return m_name; }
+
+  private:
+    QCString m_name;
+    QList<DirEntry> m_children;
+};
+
+void generateFileTree(QTextStream &t);
+
 #endif
+
