@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: groupdef.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,7 +22,6 @@
 #include "filedef.h"
 #include "classlist.h"
 #include "outputlist.h"
-#include "doc.h"
 #include "namespacedef.h"
 #include "language.h"
 #include "util.h"
@@ -30,10 +29,12 @@
 #include "message.h"
 #include "membergroup.h"
 #include "doxygen.h"
-#include "page.h"
+#include "pagedef.h"
+#include "docparser.h"
+#include "searchindex.h"
 
-GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t) : 
-   Definition(df,dl,na)
+GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t,
+                   const char *refFileName) : Definition(df,dl,na)
 {
   fileList = new FileList;
   classSDict = new ClassSDict(257);
@@ -43,7 +44,14 @@ GroupDef::GroupDef(const char *df,int dl,const char *na,const char *t) :
   exampleDict = new PageSDict(257);
   allMemberList = new MemberList;
   allMemberNameInfoSDict = new MemberNameInfoSDict(17);
-  fileName = (QCString)"group_"+na;
+  if (refFileName)
+  {
+    fileName=stripExtension(refFileName);
+  }
+  else
+  {
+    fileName = (QCString)"group_"+na;
+  }
   setGroupTitle( t );
   memberGroupSDict = new MemberGroupSDict;
   memberGroupSDict->setAutoDelete(TRUE);
@@ -105,9 +113,26 @@ void GroupDef::distributeMemberGroupDocumentation()
   }
 }
 
+void GroupDef::findSectionsInDocumentation()
+{
+  docFindSections(documentation(),this,0,docFile());
+  MemberGroupSDict::Iterator mgli(*memberGroupSDict);
+  MemberGroup *mg;
+  for (;(mg=mgli.current());++mgli)
+  {
+    mg->findSectionsInDocumentation();
+  }
+  decDefineMembers.findSectionsInDocumentation();
+  decProtoMembers.findSectionsInDocumentation();
+  decTypedefMembers.findSectionsInDocumentation();
+  decEnumMembers.findSectionsInDocumentation();
+  decFuncMembers.findSectionsInDocumentation();
+  decVarMembers.findSectionsInDocumentation();
+}
+
 void GroupDef::addFile(const FileDef *def)
 {
-  if (Config_getBool("SORT_MEMBER_DOCS"))
+  if (Config_getBool("SORT_BRIEF_DOCS"))
     fileList->inSort(def);
   else
     fileList->append(def);
@@ -115,7 +140,7 @@ void GroupDef::addFile(const FileDef *def)
 
 void GroupDef::addClass(const ClassDef *cd)
 {
-  if (Config_getBool("SORT_MEMBER_DOCS"))
+  if (Config_getBool("SORT_BRIEF_DOCS"))
     classSDict->inSort(cd->name(),cd);
   else
     classSDict->append(cd->name(),cd);
@@ -123,33 +148,34 @@ void GroupDef::addClass(const ClassDef *cd)
 
 void GroupDef::addNamespace(const NamespaceDef *def)
 {
-  if (Config_getBool("SORT_MEMBER_DOCS"))
+  if (Config_getBool("SORT_BRIEF_DOCS"))
     namespaceList->inSort(def);  
   else
     namespaceList->append(def);
 }
 
-void GroupDef::addPage(PageInfo *def)
+void GroupDef::addPage(PageDef *def)
 {
-  pageDict->append(def->name,def);
+  //printf("Making page %s part of a group\n",def->name.data());
+  pageDict->append(def->name(),def);
   def->makePartOfGroup(this);
 }
 
-void GroupDef::addExample(const PageInfo *def)
+void GroupDef::addExample(const PageDef *def)
 {
-  exampleDict->append(def->name,def);
+  exampleDict->append(def->name(),def);
 }
 
 
 void GroupDef::addMembersToMemberGroup()
 {
-  ::addMembersToMemberGroup(&decDefineMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decProtoMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decTypedefMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decEnumMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decEnumValMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decFuncMembers,memberGroupSDict);
-  ::addMembersToMemberGroup(&decVarMembers,memberGroupSDict);
+  ::addMembersToMemberGroup(&decDefineMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decProtoMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decTypedefMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decEnumMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decEnumValMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decFuncMembers,memberGroupSDict,this);
+  ::addMembersToMemberGroup(&decVarMembers,memberGroupSDict,this);
 
   //printf("GroupDef::addMembersToMemberGroup() memberGroupList=%d\n",memberGroupList->count());
   MemberGroupSDict::Iterator mgli(*memberGroupSDict);
@@ -161,7 +187,7 @@ void GroupDef::addMembersToMemberGroup()
 }
 
 
-void GroupDef::insertMember(MemberDef *md,bool docOnly)
+bool GroupDef::insertMember(MemberDef *md,bool docOnly)
 {
   //printf("GroupDef(%s)::insertMember(%s)\n", title.data(), md->name().data());
   MemberNameInfo *mni=0;
@@ -172,10 +198,27 @@ void GroupDef::insertMember(MemberDef *md,bool docOnly)
     for ( ; (srcMi=srcMnii.current()) ; ++srcMnii )
     {
       MemberDef *srcMd = srcMi->memberDef;
-      if (matchArguments(srcMd->argumentList(),md->argumentList()) && 
-          srcMd->getOuterScope()==md->getOuterScope())
+      if (srcMd==md) return FALSE; // already added before!
+
+      bool sameScope = srcMd->getOuterScope()==md->getOuterScope() || // same class or namespace
+          // both inside a file => definition and declaration do not have to be in the same file
+           (srcMd->getOuterScope()->definitionType()==Definition::TypeFile &&
+            md->getOuterScope()->definitionType()==Definition::TypeFile); 
+      
+      if (srcMd->isFunction() && md->isFunction() && 
+          matchArguments(srcMd->argumentList(),md->argumentList()) &&
+          sameScope
+         )
       {
-        return; // member already added
+        if (srcMd->getGroupAlias()==0) 
+        {
+          md->setGroupAlias(srcMd); 
+        }
+        else
+        {
+          md->setGroupAlias(srcMd->getGroupAlias()); 
+        }
+        return FALSE; // member is the same as one that is already added
       }
     }
     mni->append(new MemberInfo(md,md->protection(),md->virtualness(),FALSE));
@@ -186,32 +229,57 @@ void GroupDef::insertMember(MemberDef *md,bool docOnly)
     mni->append(new MemberInfo(md,md->protection(),md->virtualness(),FALSE));
     allMemberNameInfoSDict->append(mni->memberName(),mni);
   }
+  //printf("Added member!\n");
   allMemberList->append(md); 
   switch(md->memberType())
   {
     case MemberDef::Variable:     
-      if (!docOnly) decVarMembers.append(md); 
+      if (!docOnly)
+      {
+        if (Config_getBool("SORT_BRIEF_DOCS"))
+          decVarMembers.inSort(md);
+        else
+          decVarMembers.append(md);
+      }
       if (Config_getBool("SORT_MEMBER_DOCS"))
         docVarMembers.inSort(md); 
       else
         docVarMembers.append(md);
       break;
     case MemberDef::Function: 
-      if (!docOnly) decFuncMembers.append(md);
+      if (!docOnly)
+      {
+        if (Config_getBool("SORT_BRIEF_DOCS"))
+          decFuncMembers.inSort(md);
+        else
+          decFuncMembers.append(md);
+      }
       if (Config_getBool("SORT_MEMBER_DOCS"))    
         docFuncMembers.inSort(md); 
       else
         docFuncMembers.append(md);
       break;
     case MemberDef::Typedef:      
-      if (!docOnly) decTypedefMembers.append(md);
+      if (!docOnly)
+      {
+        if (Config_getBool("SORT_BRIEF_DOCS"))
+          decTypedefMembers.inSort(md);
+        else
+          decTypedefMembers.append(md);
+      }
       if (Config_getBool("SORT_MEMBER_DOCS"))
         docTypedefMembers.inSort(md); 
       else
         docTypedefMembers.append(md);
       break;
     case MemberDef::Enumeration:  
-      if (!docOnly) decEnumMembers.append(md);
+      if (!docOnly)
+      {
+        if (Config_getBool("SORT_BRIEF_DOCS"))
+          decEnumMembers.inSort(md);
+        else
+          decEnumMembers.append(md);
+      }
       if (Config_getBool("SORT_MEMBER_DOCS"))
         docEnumMembers.inSort(md); 
       else
@@ -220,14 +288,26 @@ void GroupDef::insertMember(MemberDef *md,bool docOnly)
     case MemberDef::EnumValue:    
       break;
     case MemberDef::Prototype:    
-      if (!docOnly) decProtoMembers.append(md);
+      if (!docOnly)
+      {
+        if (Config_getBool("SORT_BRIEF_DOCS"))
+          decProtoMembers.inSort(md);
+        else
+          decProtoMembers.append(md);
+      }
       if (Config_getBool("SORT_MEMBER_DOCS"))
         docProtoMembers.inSort(md); 
       else
         docProtoMembers.append(md);
       break;
     case MemberDef::Define:       
-      if (!docOnly) decDefineMembers.append(md);
+      if (!docOnly)
+      {
+        if (Config_getBool("SORT_BRIEF_DOCS"))
+          decDefineMembers.inSort(md);
+        else
+          decDefineMembers.append(md);
+      }
       if (Config_getBool("SORT_MEMBER_DOCS"))
         docDefineMembers.inSort(md); 
       else
@@ -240,7 +320,7 @@ void GroupDef::insertMember(MemberDef *md,bool docOnly)
            md->getClassDef() ? md->getClassDef()->name().data() : "",
            name().data());
   }
-  //addMemberToGroup(md,groupId);
+  return TRUE;
 }
 
 void GroupDef::removeMember(MemberDef *md)
@@ -330,12 +410,47 @@ int GroupDef::countMembers() const
          exampleDict->count();
 }
 
-/*! Compute the HTML anchor names for all members in the class */ 
+/*! Compute the HTML anchor names for all members in the group */ 
 void GroupDef::computeAnchors()
 {
+  //printf("GroupDef::computeAnchors()\n");
   setAnchors(0,'a',allMemberList);
 }
 
+void GroupDef::writeDetailedDocumentation(OutputList &ol)
+{
+  if (!briefDescription().isEmpty() || !documentation().isEmpty())
+  {
+    
+    if (pageDict->count()!=countMembers()) // classical layout
+    {
+      ol.writeRuler();
+      ol.pushGeneratorState();
+      ol.disable(OutputGenerator::Latex);
+      ol.disable(OutputGenerator::RTF);
+      ol.writeAnchor(0,"_details");
+      ol.popGeneratorState();
+      ol.startGroupHeader();
+      ol.parseText(theTranslator->trDetailedDescription());
+      ol.endGroupHeader();
+
+      // repeat brief description
+      if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
+      {
+        ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
+        ol.newParagraph();
+      }
+    }
+
+    // write documentation
+    if (!documentation().isEmpty())
+    {
+      ol.parseDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
+    }
+  }
+}
+
+
 void GroupDef::writeDocumentation(OutputList &ol)
 {
   ol.pushGeneratorState();
@@ -346,21 +461,31 @@ void GroupDef::writeDocumentation(OutputList &ol)
   addGroupListToTitle(ol,this);
   endTitle(ol,getOutputFileBase(),title);
 
-  //brief=brief.stripWhiteSpace();
-  //int bl=brief.length();
-  OutputList briefOutput(&ol);
-  if (!briefDescription().isEmpty())
+  if (Config_getBool("SEARCHENGINE"))
   {
-    parseDoc(briefOutput,m_defFileName,m_defLine,name(),0,briefDescription());
-    ol+=briefOutput;
+    Doxygen::searchIndex->setCurrentDoc(title,getOutputFileBase());
+    Doxygen::searchIndex->addWord(localName().lower());
+  }
+
+  if (Config_getBool("DETAILS_AT_TOP"))
+  {
+    writeDetailedDocumentation(ol);
+    ol.newParagraph();
+  }
+  else if (!briefDescription().isEmpty())
+  {
+    ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
     ol.writeString(" \n");
     ol.pushGeneratorState();
     ol.disable(OutputGenerator::Latex);
     ol.disable(OutputGenerator::RTF);
+    ol.disable(OutputGenerator::Man);
     ol.startTextLink(0,"_details");
-    parseText(ol,theTranslator->trMore());
+    ol.parseText(theTranslator->trMore());
     ol.endTextLink();
-    //ol.enable(OutputGenerator::Latex);
+    ol.enableAll();
+    ol.disableAllBut(OutputGenerator::Man);
+    ol.newParagraph();
     ol.popGeneratorState();
   }
 
@@ -369,7 +494,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
     Doxygen::tagFile << "  <compound kind=\"group\">" << endl;
     Doxygen::tagFile << "    <name>" << convertToXML(name()) << "</name>" << endl;
     Doxygen::tagFile << "    <title>" << convertToXML(title) << "</title>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << htmlFileExtension << "</filename>" << endl;
+    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
   }
   
   ol.startMemberSections();
@@ -378,7 +503,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
   if (fileList->count()>0)
   {
     ol.startMemberHeader();
-    parseText(ol,theTranslator->trFile(TRUE,FALSE));
+    ol.parseText(theTranslator->trFile(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
     FileDef *fd=fileList->first();
@@ -392,11 +517,11 @@ void GroupDef::writeDocumentation(OutputList &ol)
       {
         Doxygen::tagFile << "    <file>" << convertToXML(fd->name()) << "</file>" << endl;
       }
-      ol.endMemberItem(FALSE);
+      ol.endMemberItem();
       if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
         ol.startMemberDescription();
-        parseDoc(ol,m_defFileName,m_defLine,0,0,fd->briefDescription());
+        ol.parseDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),FALSE,FALSE);
         ol.endMemberDescription();
         ol.newParagraph();
       }
@@ -409,7 +534,7 @@ void GroupDef::writeDocumentation(OutputList &ol)
   if (namespaceList->count()>0)
   {
     ol.startMemberHeader();
-    parseText(ol,theTranslator->trNamespaces());
+    ol.parseText(theTranslator->trNamespaces());
     ol.endMemberHeader();
     ol.startMemberList();
     NamespaceDef *nd=namespaceList->first();
@@ -423,11 +548,11 @@ void GroupDef::writeDocumentation(OutputList &ol)
       {
         Doxygen::tagFile << "    <namespace>" << convertToXML(nd->name()) << "</namespace>" << endl;
       }
-      ol.endMemberItem(FALSE);
+      ol.endMemberItem();
       if (!nd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
         ol.startMemberDescription();
-        parseDoc(ol,m_defFileName,m_defLine,0,0,nd->briefDescription());
+        ol.parseDoc(briefFile(),briefLine(),nd,0,nd->briefDescription(),FALSE,FALSE);
         ol.endMemberDescription();
         ol.newParagraph();
       }
@@ -440,24 +565,26 @@ void GroupDef::writeDocumentation(OutputList &ol)
   if (groupList->count()>0)
   {
     ol.startMemberHeader();
-    parseText(ol,theTranslator->trModules());
+    ol.parseText(theTranslator->trModules());
     ol.endMemberHeader();
     ol.startMemberList();
     GroupDef *gd=groupList->first();
     while (gd)
     {
       ol.startMemberItem(0);
-      //ol.insertMemberAlign();
+      ol.docify(theTranslator->trGroup(FALSE,TRUE));
+      ol.docify(" ");
+      ol.insertMemberAlign();
       ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),0,gd->groupTitle());
       if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
       {
         Doxygen::tagFile << "    <subgroup>" << convertToXML(gd->name()) << "</subgroup>" << endl;
       }
-      ol.endMemberItem(FALSE);
+      ol.endMemberItem();
       if (!gd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
         ol.startMemberDescription();
-        parseDoc(ol,m_defFileName,m_defLine,0,0,gd->briefDescription());
+        ol.parseDoc(briefFile(),briefLine(),gd,0,gd->briefDescription(),FALSE,FALSE);
         ol.endMemberDescription();
         ol.newParagraph();
       }
@@ -490,43 +617,18 @@ void GroupDef::writeDocumentation(OutputList &ol)
   }
   ol.endMemberSections();
 
-
-  if (!briefDescription().isEmpty() || !documentation().isEmpty())
+  if (!Config_getBool("DETAILS_AT_TOP"))
   {
-    
-    if (pageDict->count()!=countMembers()) // classical layout
-    {
-      ol.writeRuler();
-      ol.pushGeneratorState();
-      ol.disable(OutputGenerator::Latex);
-      ol.disable(OutputGenerator::RTF);
-      ol.writeAnchor(0,"_details");
-      ol.popGeneratorState();
-      ol.startGroupHeader();
-      parseText(ol,theTranslator->trDetailedDescription());
-      ol.endGroupHeader();
-
-      // repeat brief description
-      if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
-      {
-        ol+=briefOutput;
-        ol.newParagraph();
-      }
-    }
-
-    // write documentation
-    if (!documentation().isEmpty())
-    {
-      parseDoc(ol,m_defFileName,m_defLine,name(),0,documentation()+"\n");
-    }
+    writeDetailedDocumentation(ol);
   }
-  PageInfo *pi=0;
+
+  PageDef *pd=0;
   PageSDict::Iterator pdi(*pageDict);
-  for (pdi.toFirst();(pi=pdi.current());++pdi)
+  for (pdi.toFirst();(pd=pdi.current());++pdi)
   {
-    if (!pi->isReference())
+    if (!pd->isReference())
     {
-      QCString pageName = pi->getOutputFileBase();
+      QCString pageName = pd->getOutputFileBase();
 
       if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
       {
@@ -534,15 +636,15 @@ void GroupDef::writeDocumentation(OutputList &ol)
       }
 
       SectionInfo *si=0;
-      if (!pi->title.isEmpty() && !pi->name.isEmpty() &&
-          (si=Doxygen::sectionDict[pi->name])!=0)
+      if (!pd->title().isEmpty() && !pd->name().isEmpty() &&
+          (si=Doxygen::sectionDict[pd->name()])!=0)
       {
-        ol.startSection(si->label,si->title,TRUE);
+        ol.startSection(si->label,si->title,SectionInfo::Subsection);
         ol.docify(si->title);
-        ol.endSection(si->label,TRUE);
+        ol.endSection(si->label,SectionInfo::Subsection);
       }
       ol.startTextBlock();
-      parseDoc(ol,pi->defFileName,pi->defLine,0,0,pi->doc);
+      ol.parseDoc(pd->docFile(),pd->docLine(),pd,0,pd->documentation(),TRUE,FALSE);
       ol.endTextBlock();
     }
   }
@@ -630,7 +732,8 @@ void addGroupToGroups(Entry *root,GroupDef *subGroup)
 /*! Add a member to the group with the highest priority */
 void addMemberToGroups(Entry *root,MemberDef *md)
 {
-  //printf("  Root %p = %s, md %p=%s groups=%d\n", root, root->name.data(), md, md->name().data(), root->groups->count() );
+  //printf("addMemberToGroups:  Root %p = %s, md %p=%s groups=%d\n", 
+  //    root, root->name.data(), md, md->name().data(), root->groups->count() );
   QListIterator<Grouping> gli(*root->groups);
   Grouping *g;
 
@@ -705,6 +808,7 @@ void addMemberToGroups(Entry *root,MemberDef *md)
 
       if (moveit)
       {
+        //printf("removeMember\n");
         mgd->removeMember(md);
         insertit = TRUE;
       }
@@ -712,17 +816,21 @@ void addMemberToGroups(Entry *root,MemberDef *md)
 
     if (insertit)
     {
-      //printf("insertMember\n");
-      fgd->insertMember(md);
-      md->setGroupDef(fgd,pri,root->fileName,root->startLine,root->doc.length() != 0);
-      ClassDef *cd = md->getClassDefOfAnonymousType();
-      if (cd) cd->setGroupDefForAllMembers(fgd,pri,root->fileName,root->startLine,root->doc.length() != 0);
+      //printf("insertMember found at %s line %d\n",md->getDefFileName().data(),md->getDefLine());
+      bool success = fgd->insertMember(md);
+      if (success)
+      {
+        //printf("insertMember successful\n");
+        md->setGroupDef(fgd,pri,root->fileName,root->startLine,!root->doc.isEmpty());
+        ClassDef *cd = md->getClassDefOfAnonymousType();
+        if (cd) cd->setGroupDefForAllMembers(fgd,pri,root->fileName,root->startLine,root->doc.length() != 0);
+      }
     }
   }
 }
 
 
-void addExampleToGroups(Entry *root,PageInfo *eg)
+void addExampleToGroups(Entry *root,PageDef *eg)
 {
   QListIterator<Grouping> gli(*root->groups);
   Grouping *g;
@@ -740,12 +848,19 @@ void addExampleToGroups(Entry *root,PageInfo *eg)
 
 QCString GroupDef::getOutputFileBase() const 
 { 
-  return convertNameToFile(fileName); 
+  if (isReference())
+  {
+    return fileName;
+  }
+  else
+  {
+    return convertNameToFile(fileName); 
+  }
 }
 
 void GroupDef::addListReferences()
 {
-  addRefItem(todoId(),testId(),bugId(),
+  addRefItem(xrefListItems(),
              theTranslator->trGroup(TRUE,TRUE),
              getOutputFileBase(),name()
             );
