@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: htmlgen.h,v 1.51 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -33,9 +33,6 @@ class HtmlGenerator : public OutputGenerator
     static void writeHeaderFile(QFile &f);
     static void writeFooterFile(QFile &f);
    
-    OutputGenerator *copy() { return new HtmlGenerator; }
-    //OutputGenerator *clone() { return new HtmlGenerator(*this); }
-    void append(const OutputGenerator *o);
     void enable() { active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==Html) active=TRUE;  }
@@ -43,15 +40,14 @@ class HtmlGenerator : public OutputGenerator
     void disableIfNot(OutputType o) { if (o!=Html) active=FALSE; }
     bool isEnabled(OutputType o) { return (o==Html && active); } 
     OutputGenerator *get(OutputType o) { return (o==Html) ? this : 0; }
-    
-    //void generateExternalIndex();
-    void startFile(const char *name,const char *manName,
-                   const char *title,bool external);
-    void writeFooter(int,bool);
+
+    void printDoc(DocNode *);
+
+    void startFile(const char *name,const char *manName,const char *title);
+    void writeFooter();
     void endFile();
     void clearBuffer();
 
-    //void writeIndex();
     void startIndexSection(IndexSections) {}
     void endIndexSection(IndexSections) {}
     void startProjectNumber();
@@ -72,11 +68,6 @@ class HtmlGenerator : public OutputGenerator
     void endIndexValue(const char *,bool);
     void startItemList()  { t << "<ul>"  << endl; }
     void endItemList()    { t << "</ul>" << endl; }
-    void startEnumList()  { t << "<ol>"  << endl; }
-    void endEnumList()    { t << "</ol>" << endl; }
-    void startAlphabeticalIndexList();
-    void endAlphabeticalIndexList();
-    void writeIndexHeading(const char *s);
     void writeIndexItem(const char *ref,const char *file,const char *name);
     void docify(const char *text);
     void codify(const char *text);
@@ -88,7 +79,6 @@ class HtmlGenerator : public OutputGenerator
     void endTextLink();
     void startHtmlLink(const char *url);
     void endHtmlLink();
-    void writeMailLink(const char *url);
     void startTypewriter() { t << "<code>"; }
     void endTypewriter()   { t << "</code>"; }
     void startGroupHeader();
@@ -103,7 +93,10 @@ class HtmlGenerator : public OutputGenerator
     void endMemberSubtitle();
     void startMemberList();
     void endMemberList();
+    void startAnonTypeScope(int) {}
+    void endAnonTypeScope(int) {}
     void startMemberItem(int);
+    void endMemberItem();
 
     void startMemberGroupHeader(bool);
     void endMemberGroupHeader();
@@ -113,22 +106,17 @@ class HtmlGenerator : public OutputGenerator
     void endMemberGroup(bool);
 
     void insertMemberAlign();
-    void endMemberItem(bool);
     void startMemberDescription();
     void endMemberDescription();
 
     void writeRuler()    { t << "<hr>"; }
     void writeAnchor(const char *,const char *name) 
                          { t << "<a name=\"" << name <<"\"></a>"; }
-    void startCodeFragment() { t << "<div class=\"fragment\"><pre>"; }
-    void endCodeFragment()   { t << "</pre></div>"; } 
-    void startPreFragment() { t << "<pre>"; }
-    void endPreFragment()   { t << "</pre>"; }
+    void startCodeFragment() { t << "<pre class=\"fragment\"><div>"; }
+    void endCodeFragment()   { t << "</div></pre>"; } 
     void writeLineNumber(const char *,const char *,const char *,int);
     void startCodeLine() { col=0; }
     void endCodeLine()   { codify("\n"); }
-    //void writeBoldString(const char *text) 
-    //                     { t << "<b>"; docify(text); t << "</b>"; }
     void startEmphasis() { t << "<em>";  }
     void endEmphasis()   { t << "</em>"; }
     void startBold()     { t << "<b>"; }
@@ -140,16 +128,13 @@ class HtmlGenerator : public OutputGenerator
     void lineBreak() { t << "<br>" << endl; }
     void writeChar(char c);
     void startMemberDoc(const char *,const char *,const char *,const char *);
-    void endMemberDoc(); 
-    //void writeDoxyAnchor(const char *fName,const char *clName,
-    //                     const char *anchor,const char *name);
+    void endMemberDoc(bool); 
     void startDoxyAnchor(const char *fName,const char *manName,
                          const char *anchor,const char *name);
     void endDoxyAnchor(const char *fName,const char *anchor);
     void startCodeAnchor(const char *label) { t << "<a name=\"" << label << "\"></a>"; }
     void endCodeAnchor() { }
     void writeLatexSpacing() {}
-    //void writeLatexLabel(const char *,const char *) {}
     void writeStartAnnoItem(const char *type,const char *file,
                             const char *path,const char *name);
     void writeEndAnnoItem(const char *) { t << endl; }
@@ -161,61 +146,27 @@ class HtmlGenerator : public OutputGenerator
     void endCenter()          { t << "</center>" << endl; }
     void startSmall()         { t << "<small>" << endl; }
     void endSmall()           { t << "</small>" << endl; }
-    void startSubscript()     { t << "<sub>"; }
-    void endSubscript()       { t << "</sub>"; }
-    void startSuperscript()   { t << "<sup>"; }
-    void endSuperscript()     { t << "</sup>"; }
-    void startTable(bool,int) 
-                              { t << "<table border=1 cellspacing=3 cellpadding=3>"; }
-    void endTable(bool hasCaption) 
-                              { if (!hasCaption) t << "</tr>"; 
-                                t << "</table>" << endl; 
-                              }
-    void startCaption()       { t << "</tr><caption align=\"bottom\">"; }
-    void endCaption()         { t << "</caption>" << endl; }
-    void nextTableRow()       { t << "<tr><td>"; }
-    void endTableRow()        { t << "</tr>" << endl; }
-    void nextTableColumn()    { t << "<td>"; }
-    void endTableColumn()     { t << "</td>"; }
-    void writeCopyright()     { t << "&copy;"; }
-    void writeQuote()         { t << "&quot;"; }
-    void writeUmlaut(char c)  { t << "&" << c << "uml;"; }
-    void writeAcute(char c)   { t << "&" << c << "acute;"; }
-    void writeGrave(char c)   { t << "&" << c << "grave;"; }
-    void writeCirc(char c)    { t << "&" << c << "circ;"; }
-    void writeTilde(char c)   { t << "&" << c << "tilde;"; } 
-    void writeRing(char c)    { t << "&" << c << "ring;"; }
-    void writeSharpS()        { t << "&szlig;"; }
-    void writeCCedil(char c)  { t << "&" << c << "cedil;"; }
     void startDescList(SectionTypes)      { t << "<dl compact><dt><b>" << endl; }
     void endDescList()        { t << "</dl>"; }
-    void startParamList(ParamListTypes) { t << "<dl compact><dt><b>" << endl; }    
-    void endParamList()       { t << "</dl>"; }
-    void endDescTitle()       { t << "</b>"; }
+    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
+    void endSimpleSect();
+    void startParamList(ParamListTypes,const char *);
+    void endParamList();
     void writeDescItem()      { t << "<dd>" << endl; }
-    void startSection(const char *,const char *,bool);
-    void endSection(const char *,bool);
-    void writeSectionRef(const char *,const char *,const char *,const char *);
-    void writeSectionRefItem(const char *,const char *,const char *);
-    //void writeSectionRefAnchor(const char *,const char *,const char *);
+    void startSection(const char *,const char *,SectionInfo::SectionType);
+    void endSection(const char *,SectionInfo::SectionType);
     void addIndexItem(const char *,const char *);
     void startIndent();
     void endIndent();
     void writeSynopsis() {}
     void startClassDiagram();
     void endClassDiagram(ClassDiagram &,const char *,const char *);
-    //void startColorFont(uchar r,uchar g,uchar b);
-    //void endColorFont();
     void startPageRef() {}
     void endPageRef(const char *,const char *) {}
-    void startQuickIndexItem(const char *,const char *);
-    void endQuickIndexItem();
-    void writeFormula(const char *,const char *);
+    //void startQuickIndexItem(const char *,const char *);
+    //void endQuickIndexItem();
+    void writeQuickLinks(bool compact,HighlightedItem hli);
     void writeNonBreakableSpace(int);
-    void startImage(const char *,const char *,bool);
-    void endImage(bool);
-    void startDotFile(const char *,bool);
-    void endDotFile(bool);
     
     void startDescTable()
     { t << "<table border=0 cellspacing=2 cellpadding=0>" << endl; }
@@ -229,12 +180,13 @@ class HtmlGenerator : public OutputGenerator
     { t << "<td>" << endl; }
     void endDescTableData()
     { t << "</td></tr>" << endl; }
-    //static void docifyStatic(QTextStream &t,const char *str);
     
     void startDotGraph();
     void endDotGraph(DotClassGraph &g);
     void startInclDepGraph();
     void endInclDepGraph(DotInclDepGraph &g);
+    void startCallGraph();
+    void endCallGraph(DotCallGraph &g);
     void writeGraphicalHierarchy(DotGfxHierarchyTable &g);
 
     void startTextBlock(bool) {}
@@ -243,26 +195,26 @@ class HtmlGenerator : public OutputGenerator
 
     void startMemberDocPrefixItem();
     void endMemberDocPrefixItem();
-    void startMemberDocName();
+    void startMemberDocName(bool);
     void endMemberDocName();
-    void startParameterType(bool first);
+    void startParameterType(bool first,const char *key);
     void endParameterType();
     void startParameterName(bool);
-    void endParameterName(bool last,bool emptyList);
-    void startParameterList();
+    void endParameterName(bool last,bool emptyList,bool closeBracket);
+    void startParameterList(bool);
     void endParameterList();
 
-    void startFontClass(const char *s) { t << "<font class=\"" << s << "\">"; }
-    void endFontClass() { t << "</font>"; }
+    void startFontClass(const char *s) { t << "<span class=\"" << s << "\">"; }
+    void endFontClass() { t << "</span>"; }
 
-    void startHtmlOnly()  {}
-    void endHtmlOnly()    {}
-    void startLatexOnly() {}
-    void endLatexOnly()   {}
+    void writeCodeAnchor(const char *anchor) 
+    { t << "<a name=\"" << anchor << "\"></a>"; }
+    static void writeSearchPage();
 
   private:
     QCString lastTitle;
     QCString lastFile;
+    QCString relPath;
 
     HtmlGenerator &operator=(const HtmlGenerator &g);
     HtmlGenerator(const HtmlGenerator &g);
