@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: index.h,v 1.28 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -26,14 +26,14 @@ enum IndexSections
   isTitlePageStart,
   isTitlePageAuthor,
   isMainPage,
-  isPackageIndex,
+//  isPackageIndex,
   isModuleIndex,
   isNamespaceIndex,
   isClassHierarchyIndex,
   isCompoundIndex,
   isFileIndex,
   isPageIndex,
-  isPackageDocumentation,
+//  isPackageDocumentation,
   isModuleDocumentation,
   isNamespaceDocumentation,
   isClassDocumentation,
@@ -69,25 +69,77 @@ void writePackageIndex(OutputList &ol);
 
 void countDataStructures();
 
+enum HighlightedItem
+{
+  HLI_None=0,
+  HLI_Main,
+  HLI_Modules,
+  HLI_Namespaces,
+  HLI_Hierarchy,
+  HLI_Classes,
+  HLI_Annotated,
+  HLI_Files,
+  HLI_NamespaceMembers,
+  HLI_Functions,
+  HLI_Globals,
+  HLI_Pages,
+  HLI_Examples,
+  HLI_Search
+};
+
+enum ClassMemberHighlight
+{
+  CMHL_All = 0,
+  CMHL_Functions,
+  CMHL_Variables,
+  CMHL_Typedefs,
+  CMHL_Enums,
+  CMHL_EnumValues,
+  CMHL_Related,
+  CMHL_Properties,
+  CMHL_Events,
+  CMHL_Total = CMHL_Events+1
+};
+
+enum FileMemberHighlight
+{
+  FMHL_All = 0,
+  FMHL_Functions,
+  FMHL_Variables,
+  FMHL_Typedefs,
+  FMHL_Enums,
+  FMHL_EnumValues,
+  FMHL_Defines,
+  FMHL_Total = FMHL_Defines+1
+};
+
+enum NamespaceMemberHighlight
+{
+  NMHL_All = 0,
+  NMHL_Functions,
+  NMHL_Variables,
+  NMHL_Typedefs,
+  NMHL_Enums,
+  NMHL_EnumValues,
+  NMHL_Total = FMHL_EnumValues+1
+};
+
 extern int annotatedClasses;
 extern int hierarchyClasses;
-extern int documentedFunctions;
-extern int documentedMembers;
-extern int documentedDefines;
 extern int documentedFiles;
-extern int documentedHtmlFiles;
 extern int documentedGroups;
 extern int documentedNamespaces;
-extern int documentedNamespaceMembers;
-extern int documentedIncludeFiles;
-extern int documentedPages;
 extern int indexedPages;
-extern int documentedPackages;
+extern int documentedClassMembers[CMHL_Total];
+extern int documentedFileMembers[FMHL_Total];
+extern int documentedNamespaceMembers[NMHL_Total];
+extern int documentedHtmlFiles;
+extern int documentedPages;
 
 void startTitle(OutputList &ol,const char *fileName);
 void endTitle(OutputList &ol,const char *fileName,const char *name);
 void startFile(OutputList &ol,const char *name,const char *manName,
-               const char *title,bool external=FALSE);
+               const char *title,HighlightedItem hli=HLI_None);
 void endFile(OutputList &ol,bool external=FALSE);
 
 #endif
