@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: latexgen.h,v 1.50 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,8 +15,8 @@
  *
  */
 
-#ifndef OUTPUT_H
-#define OUTPUT_H
+#ifndef LATEXGEN_H
+#define LATEXGEN_H
 
 #include "outputgen.h"
 
@@ -31,9 +31,9 @@ class LatexGenerator : public OutputGenerator
     static void writeStyleSheetFile(QFile &f);
     static void writeHeaderFile(QFile &f);
 
-    OutputGenerator *copy();
+    //OutputGenerator *copy();
     //OutputGenerator *clone() { return new LatexGenerator(*this); }
-    void append(const OutputGenerator *o);
+    //void append(const OutputGenerator *o);
     void enable() { active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==Latex) active=TRUE;  }
@@ -42,9 +42,10 @@ class LatexGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==Latex && active); } 
     OutputGenerator *get(OutputType o) { return (o==Latex) ? this : 0; }
 
-    void startFile(const char *name,const char *manName,
-                   const char *title, bool external);
-    void writeFooter(int,bool) {}
+    void printDoc(DocNode *);
+
+    void startFile(const char *name,const char *manName,const char *title);
+    void writeFooter() {}
     void endFile();
     void clearBuffer();
     
@@ -69,11 +70,11 @@ class LatexGenerator : public OutputGenerator
     void endIndexValue(const char *,bool);
     void startItemList()  { t << "\\begin{CompactItemize}" << endl; }
     void endItemList()    { t << "\\end{CompactItemize}"   << endl; }
-    void startEnumList()  { t << "\\begin{enumerate}"      << endl; }
-    void endEnumList()    { t << "\\end{enumerate}"        << endl; }
-    void startAlphabeticalIndexList() {}
-    void endAlphabeticalIndexList() {} 
-    void writeIndexHeading(const char *) {}
+    //void startEnumList()  { t << "\\begin{enumerate}"      << endl; }
+    //void endEnumList()    { t << "\\end{enumerate}"        << endl; }
+    //void startAlphabeticalIndexList() {}
+    //void endAlphabeticalIndexList() {} 
+    //void writeIndexHeading(const char *) {}
     void writeIndexItem(const char *ref,const char *file,const char *name);
     void docify(const char *text);
     void codify(const char *text);
@@ -85,7 +86,7 @@ class LatexGenerator : public OutputGenerator
     void endTextLink();
     void startHtmlLink(const char *url);
     void endHtmlLink();
-    void writeMailLink(const char *url);
+    //void writeMailLink(const char *url);
     void startTypewriter() { t << "{\\tt "; }
     void endTypewriter()   { t << "}";      }
     void startGroupHeader();
@@ -100,8 +101,10 @@ class LatexGenerator : public OutputGenerator
     void endMemberSubtitle() {}
     void startMemberList();
     void endMemberList();
+    void startAnonTypeScope(int);
+    void endAnonTypeScope(int);
     void startMemberItem(int);
-    void endMemberItem(bool);
+    void endMemberItem();
 
     void startMemberGroupHeader(bool);
     void endMemberGroupHeader();
@@ -116,12 +119,14 @@ class LatexGenerator : public OutputGenerator
     void writeAnchor(const char *fileName,const char *name);
     void startCodeFragment() { t << endl << endl << "\\footnotesize\\begin{verbatim}"; }
     void endCodeFragment()   { t << "\\end{verbatim}\\normalsize " << endl; }
-    void startPreFragment()  { t << "\\small\\begin{alltt}"; 
-                               insidePre=TRUE; 
-                             }
-    void endPreFragment()    { t << "\\end{alltt}\\normalsize " << endl; 
-                               insidePre=FALSE; 
-                             }
+    //void startPreFragment()  { t << "\\small\\begin{alltt}"; 
+    //                           insidePre=TRUE; 
+    //                         }
+    //void endPreFragment()    { t << "\\end{alltt}\\normalsize " << endl; 
+    //                           insidePre=FALSE; 
+    //                         }
+    //void startVerbatimFragment() { t << endl << endl << "\\footnotesize\\begin{verbatim}"; }
+    //void endVerbatimFragment()   { t << "\\end{verbatim}\\normalsize " << endl; }
     void writeLineNumber(const char *,const char *,const char *,int l) { t << l << " "; }
     void startCodeLine() { col=0; }
     void endCodeLine() { codify("\n"); }
@@ -137,7 +142,7 @@ class LatexGenerator : public OutputGenerator
     void endDescItem();
     void lineBreak() { t << "\\par\n"; }
     void startMemberDoc(const char *,const char *,const char *,const char *);
-    void endMemberDoc();
+    void endMemberDoc(bool);
     void startDoxyAnchor(const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *);
     void startCodeAnchor(const char *) {}
@@ -156,64 +161,66 @@ class LatexGenerator : public OutputGenerator
     void endCenter()        { t << "\\end{center}" << endl; }
     void startSmall()       { t << "\\footnotesize "; }
     void endSmall()         { t << "\\normalsize "; }
-    void startSubscript()   { t << "$_{\\mbox{"; }
-    void endSubscript()     { t << "}}$"; }
-    void startSuperscript() { t << "$^{\\mbox{"; }
-    void endSuperscript()   { t << "}}$"; }
-    void startTable(bool hasCaption,int c)  
-                            { 
-                              if (hasCaption) t << "\\begin{table}[h]";
-                              t << "\\begin{TabularC}{" << c << "}\n\\hline\n"; 
-                            }
-    void endTable(bool hasCaption)         
-                            { 
-                              if (hasCaption)
-                              {
-                                t << "\\end{table}\n";
-                              }
-                              else
-                              {
-                                t << "\\\\\\hline\n\\end{TabularC}\n"; 
-                              }
-                            }
-    void startCaption()     { t << "\\\\\\hline\n\\end{TabularC}\n"
-                                   "\\centering\n\\caption{"; 
-                            }
-    void endCaption()       { t << "}\n"; }
-    void nextTableRow()     {}
-    void endTableRow()      { t << "\\\\\\hline\n"; }
-    void nextTableColumn()  { t << "&"; }
-    void endTableColumn()   {}
-    void writeCopyright()    { t << "\\copyright"; }
-    void writeQuote()        { t << "''"; }
-    void writeUmlaut(char c) { if (c=='i') t << "\\\"{\\i}"; else 
-                                           t << "\\\"{" << c << "}"; 
-                             }
-    void writeAcute(char c)  { if (c=='i') t << "\\'{\\i}"; else
-                                           t << "\\'{"  << c << "}"; 
-                             }
-    void writeGrave(char c)  { if (c=='i') t << "\\`{\\i}"; else
-                                           t << "\\`{"  << c << "}"; 
-                             }
-    void writeCirc(char c)   { if (c=='i') t << "\\^{\\i}"; else
-                                           t << "\\^{"  << c << "}"; 
-                             }
-    void writeTilde(char c)  { t << "\\~{"  << c << "}"; }
-    void writeRing(char c)   { t << "\\" << c << c; }
-    void writeSharpS()       { t << "\"s"; }
-    void writeCCedil(char c) { t << "\\c{" << c << "}"; }
+    //void startSubscript()   { t << "$_{\\mbox{"; }
+    //void endSubscript()     { t << "}}$"; }
+    //void startSuperscript() { t << "$^{\\mbox{"; }
+    //void endSuperscript()   { t << "}}$"; }
+    //void startTable(bool hasCaption,int c)  
+    //                        { 
+    //                          if (hasCaption) t << "\\begin{table}[h]";
+    //                          t << "\\begin{TabularC}{" << c << "}\n\\hline\n"; 
+    //                        }
+    //void endTable(bool hasCaption)         
+    //                        { 
+    //                          if (hasCaption)
+    //                          {
+    //                            t << "\\end{table}\n";
+    //                          }
+    //                          else
+    //                          {
+    //                            t << "\\\\\\hline\n\\end{TabularC}\n"; 
+    //                          }
+    //                        }
+    //void startCaption()     { t << "\\\\\\hline\n\\end{TabularC}\n"
+    //                               "\\centering\n\\caption{"; 
+    //                        }
+    //void endCaption()       { t << "}\n"; }
+    //void nextTableRow()     {}
+    //void endTableRow()      { t << "\\\\\\hline\n"; }
+    //void nextTableColumn()  { t << "&"; }
+    //void endTableColumn()   {}
+    //void writeCopyright()    { t << "\\copyright"; }
+    //void writeQuote()        { t << "''"; }
+    //void writeUmlaut(char c) { if (c=='i') t << "\\\"{\\i}"; else 
+    //                                       t << "\\\"{" << c << "}"; 
+    //                         }
+    //void writeAcute(char c)  { if (c=='i') t << "\\'{\\i}"; else
+    //                                       t << "\\'{"  << c << "}"; 
+    //                         }
+    //void writeGrave(char c)  { if (c=='i') t << "\\`{\\i}"; else
+    //                                       t << "\\`{"  << c << "}"; 
+    //                         }
+    //void writeCirc(char c)   { if (c=='i') t << "\\^{\\i}"; else
+    //                                       t << "\\^{"  << c << "}"; 
+    //                         }
+    //void writeTilde(char c)  { t << "\\~{"  << c << "}"; }
+    //void writeRing(char c)   { t << "\\" << c << c; }
+    //void writeSharpS()       { t << "\"s"; }
+    //void writeCCedil(char c) { t << "\\c{" << c << "}"; }
     void startMemberDescription() { t << "\\begin{CompactList}\\small\\item\\em "; }
     void endMemberDescription() { t << "\\item\\end{CompactList}"; }
     void startDescList(SectionTypes)     { t << "\\begin{Desc}\n\\item["; }
     void endDescList()       { t << "\\end{Desc}" << endl; }
-    void startParamList(ParamListTypes) { t << "\\begin{Desc}\n\\item["; }   
-    void endParamList()      { t << "\\end{Desc}" << endl; }
-    void endDescTitle()      { t << "]"; }
+    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
+    void endSimpleSect();
+    void startParamList(ParamListTypes,const char *title);
+    void endParamList();
+    //void endDescTitle()      { t << "]"; }
     void writeDescItem()     { t << "\\par" << endl; }
-    void startSection(const char *,const char *,bool);
-    void endSection(const char *,bool);
-    void writeSectionRef(const char *,const char *,const char *,const char *);
-    void writeSectionRefItem(const char *,const char *,const char *);
+    void startSection(const char *,const char *,SectionInfo::SectionType);
+    void endSection(const char *,SectionInfo::SectionType);
+    //void writeSectionRef(const char *,const char *,const char *,const char *);
+    //void writeSectionRefItem(const char *,const char *,const char *);
     //void writeSectionRefAnchor(const char *,const char *,const char *);
     void addIndexItem(const char *,const char *);
     void startIndent()       {}
@@ -226,14 +233,15 @@ class LatexGenerator : public OutputGenerator
     //void endColorFont()   {}
     void startPageRef();
     void endPageRef(const char *,const char *);
-    void startQuickIndexItem(const char *,const char *) {}
-    void endQuickIndexItem() {}
-    void writeFormula(const char *,const char *);
+    //void startQuickIndexItem(const char *,const char *) {}
+    //void endQuickIndexItem() {}
+    void writeQuickLinks(bool,HighlightedItem) {}
+    //void writeFormula(const char *,const char *);
     void writeNonBreakableSpace(int);
-    void startImage(const char *,const char *,bool);
-    void endImage(bool);
-    void startDotFile(const char *,bool);
-    void endDotFile(bool);
+    //void startImage(const char *,const char *,bool);
+    //void endImage(bool);
+    //void startDotFile(const char *,bool);
+    //void endDotFile(bool);
     
     void startDescTable()
     { t << "\\begin{description}" << endl; }
@@ -253,6 +261,8 @@ class LatexGenerator : public OutputGenerator
     void endDotGraph(DotClassGraph &);
     void startInclDepGraph();
     void endInclDepGraph(DotInclDepGraph &);
+    void startCallGraph();
+    void endCallGraph(DotCallGraph &);
     void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
 
     void startTextBlock(bool) {}
@@ -260,31 +270,36 @@ class LatexGenerator : public OutputGenerator
 
     void startMemberDocPrefixItem() {}
     void endMemberDocPrefixItem() {}
-    void startMemberDocName() {}
+    void startMemberDocName(bool) {}
     void endMemberDocName() {}
-    void startParameterType(bool) {}
+    void startParameterType(bool,const char *) {}
     void endParameterType() {}
     void startParameterName(bool) {}
-    void endParameterName(bool,bool) {}
-    void startParameterList() {}
+    void endParameterName(bool,bool,bool) {}
+    void startParameterList(bool) {}
     void endParameterList() {}
 
     void startFontClass(const char *) {}
     void endFontClass() {}
 
-    void startHtmlOnly()  {}
-    void endHtmlOnly()    {}
-    void startLatexOnly() {}
-    void endLatexOnly()   {}
+    //void startHtmlOnly()  {}
+    //void endHtmlOnly()    {}
+    //void startLatexOnly() {}
+    //void endLatexOnly()   {}
     
+    //void startSectionRefList();
+    //void endSectionRefList();
+
+    void writeCodeAnchor(const char *) {}
+
   private:
-    void latin2ToLatex(unsigned char);
     LatexGenerator(const LatexGenerator &);
     LatexGenerator &operator=(const LatexGenerator &);
     int col;
     bool insideTabbing;
     bool firstDescItem;
     bool insidePre;
+    QCString relPath;
 };
 
 #endif
