@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: htmlgen.cpp,v 1.17 1998/11/28 11:33:19 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,15 +28,16 @@
 #include "util.h"
 #include "doxygen.h"
 #include <string.h>
+#include "docparser.h"
+#include "mandocvisitor.h"
 
 static QCString getExtension()
 {
   QCString ext = Config_getString("MAN_EXTENSION");
   if( ext.length() >= 2 &&
-      ext.data()[0] == '.' &&
-      isdigit( ext.data()[1] ) )
+      ext.data()[0] == '.')
   {
-    ext = ext.mid(1, 1);
+    ext = ext.mid(1, ext.length()-1);
   }
   else
   {
@@ -60,21 +61,21 @@ ManGenerator::~ManGenerator()
 {
 }
 
-void ManGenerator::append(const OutputGenerator *g)
-{
-  QCString r=g->getContents();
-  if (upperCase)
-    t << r.upper();
-  else
-    t << r;
-  if (!r.isEmpty())
-    firstCol = r.at(r.length()-1)=='\n';
-  else
-    firstCol = ((ManGenerator *)g)->firstCol;
-  col+=((ManGenerator *)g)->col;
-  inHeader=((ManGenerator *)g)->inHeader;
-  paragraph=FALSE;
-}
+//void ManGenerator::append(const OutputGenerator *g)
+//{
+//  QCString r=g->getContents();
+//  if (upperCase)
+//    t << r.upper();
+//  else
+//    t << r;
+//  if (!r.isEmpty())
+//    firstCol = r.at(r.length()-1)=='\n';
+//  else
+//    firstCol = ((ManGenerator *)g)->firstCol;
+//  col+=((ManGenerator *)g)->col;
+//  inHeader=((ManGenerator *)g)->inHeader;
+//  paragraph=FALSE;
+//}
 
 void ManGenerator::init()
 {
@@ -93,6 +94,7 @@ void ManGenerator::init()
     err("Could not create output directory %s/man%s\n",manOutput.data(),ext.data());
     exit(1);
   }
+  createSubDirs(d);
 }
 
 static QCString buildFileName(const char *name)
@@ -118,6 +120,7 @@ static QCString buildFileName(const char *name)
       case '~':
       case '%':
       case '+':
+      case '/':
         fileName+="_";
         break;
       default:
@@ -134,8 +137,7 @@ static QCString buildFileName(const char *name)
   return fileName;
 }
 
-void ManGenerator::startFile(const char *,const char *manName,
-                             const char *,bool)
+void ManGenerator::startFile(const char *,const char *manName,const char *)
 {
   startPlainFile( buildFileName( manName ) );
   firstCol=TRUE;
@@ -151,6 +153,8 @@ void ManGenerator::endTitleHead(const char *,const char *name)
 {
   t << ".TH \"" << name << "\" " << getExtension() << " \"" 
     << dateToString(FALSE) << "\" \"";
+  if (!Config_getString("PROJECT_NUMBER").isEmpty())
+    t << "Version " << Config_getString("PROJECT_NUMBER") << "\" \"";
   if (Config_getString("PROJECT_NAME").isEmpty()) 
     t << "Doxygen";
   else
@@ -210,10 +214,10 @@ void ManGenerator::endHtmlLink()
 {
 }
 
-void ManGenerator::writeMailLink(const char *url)
-{
-  docify(url);
-}
+//void ManGenerator::writeMailLink(const char *url)
+//{
+//  docify(url);
+//}
 
 void ManGenerator::startGroupHeader()
 {
@@ -320,16 +324,6 @@ void ManGenerator::startDescList(SectionTypes)
   startBold();
 }
 
-void ManGenerator::startParamList(ParamListTypes)      
-{
-  if (!firstCol) 
-  { t << endl << ".PP" << endl; 
-    firstCol=TRUE; paragraph=TRUE; 
-    col=0;
-  }
-  paragraph=FALSE;
-}
-
 void ManGenerator::startTitle() 
 { 
   if (!firstCol) t << endl; 
@@ -411,7 +405,7 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
     linkfile.close();
 }
 
-void ManGenerator::endMemberDoc()
+void ManGenerator::endMemberDoc(bool)
 {
     t << "\"";
 }
@@ -458,11 +452,11 @@ void ManGenerator::startDescItem()
   firstCol=FALSE;
 }
 
-void ManGenerator::endDescTitle()
-{
-  endBold();
-  paragraph=TRUE;
-}
+//void ManGenerator::endDescTitle()
+//{
+//  endBold();
+//  paragraph=TRUE;
+//}
 
 void ManGenerator::writeDescItem()
 {
@@ -480,21 +474,32 @@ void ManGenerator::endDescItem()
   firstCol=TRUE;
 }
 
-void ManGenerator::startMemberItem(int annType) 
+void ManGenerator::startAnonTypeScope(int indentLevel)
+{
+  if (indentLevel==0)
+  {
+    insideTabbing=TRUE;
+  }
+}
+
+void ManGenerator::endAnonTypeScope(int indentLevel)
+{
+  if (indentLevel==0)
+  {
+    insideTabbing=FALSE;
+  }
+}
+
+
+void ManGenerator::startMemberItem(int) 
 { 
   if (firstCol && !insideTabbing) t << ".in +1c\n";
   t << "\n.ti -1c\n.RI \""; 
   firstCol=FALSE;
-  if (annType!=0) insideTabbing=TRUE;
 }
 
-void ManGenerator::endMemberItem(bool endItem) 
+void ManGenerator::endMemberItem() 
 { 
-  if (endItem)
-  {
-    insideTabbing=FALSE;
-    t << "\"\n.br\n.RI \"";
-  }
   t << "\"\n.br"; 
 }
 
@@ -545,32 +550,34 @@ void ManGenerator::endMemberGroup(bool)
   firstCol=FALSE;
 }
 
-void ManGenerator::startSection(const char *,const char *,bool sub)
+void ManGenerator::startSection(const char *,const char *,SectionInfo::SectionType type)
 {
   if( !inHeader ) 
   {
-    if( sub )
+    switch(type)
     {
-      startMemberHeader();
-    }
-    else
-    {
-      startGroupHeader();
+      case SectionInfo::Page:          startGroupHeader(); break;
+      case SectionInfo::Section:       startGroupHeader(); break;
+      case SectionInfo::Subsection:    startMemberHeader(); break;
+      case SectionInfo::Subsubsection: startMemberHeader(); break;
+      case SectionInfo::Paragraph:     startMemberHeader(); break;
+      default: ASSERT(0); break;
     }
   }
 }
 
-void ManGenerator::endSection(const char *,bool sub)
+void ManGenerator::endSection(const char *,SectionInfo::SectionType type)
 {
   if( !inHeader )
   {
-    if( sub )
+    switch(type)
     {
-      endMemberHeader();
-    }
-    else
-    {
-      endGroupHeader();
+      case SectionInfo::Page:          endGroupHeader(); break;
+      case SectionInfo::Section:       endGroupHeader(); break;
+      case SectionInfo::Subsection:    endMemberHeader(); break;
+      case SectionInfo::Subsubsection: endMemberHeader(); break;
+      case SectionInfo::Paragraph:     endMemberHeader(); break;
+      default: ASSERT(0); break;
     }
   }
   else
@@ -582,4 +589,48 @@ void ManGenerator::endSection(const char *,bool sub)
   }
 }
 
+void ManGenerator::startSimpleSect(SectionTypes,const char *,
+                                   const char *,const char *title)
+{
+  if (!firstCol) 
+  { t << endl << ".PP" << endl; 
+    firstCol=TRUE; paragraph=TRUE; 
+    col=0;
+  }
+  paragraph=FALSE;
+  startBold();
+  docify(title);
+  endBold();
+  paragraph=TRUE;
+}
+
+void ManGenerator::endSimpleSect()
+{
+}
+
+void ManGenerator::startParamList(ParamListTypes,const char *title)
+{
+  if (!firstCol) 
+  { t << endl << ".PP" << endl; 
+    firstCol=TRUE; paragraph=TRUE; 
+    col=0;
+  }
+  paragraph=FALSE;
+  startBold();
+  docify(title);
+  endBold();
+  paragraph=TRUE;
+}
+
+void ManGenerator::endParamList()
+{
+}
+
+void ManGenerator::printDoc(DocNode *n)
+{
+  ManDocVisitor *visitor = new ManDocVisitor(t,*this);
+  n->accept(visitor);
+  delete visitor; 
+  firstCol=FALSE;
+}
 
