@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: htmlgen.h,v 1.15 1998/11/28 11:33:19 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -28,9 +28,9 @@ class ManGenerator : public OutputGenerator
     ManGenerator();
    ~ManGenerator();
     
-    OutputGenerator *copy()  { return new ManGenerator; } 
+    //OutputGenerator *copy()  { return new ManGenerator; } 
     //OutputGenerator *clone() { return new ManGenerator(*this); }
-    void append(const OutputGenerator *o);
+    //void append(const OutputGenerator *o);
     void enable() { active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==Man) active=TRUE;  }
@@ -39,10 +39,11 @@ class ManGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==Man && active); } 
     OutputGenerator *get(OutputType o) { return (o==Man) ? this : 0; }
 
+    void printDoc(DocNode *);
+
     static void init();
-    void startFile(const char *name,const char *manName,
-                   const char *title,bool external);
-    void writeFooter(int,bool) {}
+    void startFile(const char *name,const char *manName,const char *title);
+    void writeFooter() {}
     void endFile();
     void clearBuffer();
 
@@ -67,11 +68,11 @@ class ManGenerator : public OutputGenerator
     void endIndexValue(const char *,bool)   {} 
     void startItemList()  {}
     void endItemList()    { newParagraph(); }
-    void startEnumList()  {}
-    void endEnumList()    { newParagraph(); }
-    void startAlphabeticalIndexList() {}
-    void endAlphabeticalIndexList() {} 
-    void writeIndexHeading(const char *) {}
+    //void startEnumList()  {}
+    //void endEnumList()    { newParagraph(); }
+    //void startAlphabeticalIndexList() {}
+    //void endAlphabeticalIndexList() {} 
+    //void writeIndexHeading(const char *) {}
     void writeIndexItem(const char *ref,const char *file,const char *name);
     void docify(const char *text);
     void codify(const char *text);
@@ -83,7 +84,7 @@ class ManGenerator : public OutputGenerator
     void endTextLink() {}
     void startHtmlLink(const char *url);
     void endHtmlLink();
-    void writeMailLink(const char *url);
+    //void writeMailLink(const char *url);
     void startTypewriter() { t << "\\fC"; firstCol=FALSE; }
     void endTypewriter()   { t << "\\fP"; firstCol=FALSE; }
     void startGroupHeader();
@@ -98,8 +99,10 @@ class ManGenerator : public OutputGenerator
     void writeListItem();
     void startMemberList();
     void endMemberList();
+    void startAnonTypeScope(int);
+    void endAnonTypeScope(int);
     void startMemberItem(int);
-    void endMemberItem(bool);
+    void endMemberItem();
 
     void startMemberGroupHeader(bool);
     void endMemberGroupHeader();
@@ -112,8 +115,10 @@ class ManGenerator : public OutputGenerator
     void writeAnchor(const char *,const char *) {}
     void startCodeFragment();
     void endCodeFragment();
-    void startPreFragment() { startCodeFragment(); }
-    void endPreFragment() { endCodeFragment(); }
+    //void startPreFragment()      { startCodeFragment(); }
+    //void endPreFragment()        { endCodeFragment(); }
+    //void startVerbatimFragment() { startCodeFragment(); }
+    //void endVerbatimFragment()   { endCodeFragment(); }
     void writeLineNumber(const char *,const char *,const char *,int l) { t << l << " "; }
     void startCodeLine() {}
     void endCodeLine() { codify("\n"); col=0; }
@@ -130,7 +135,7 @@ class ManGenerator : public OutputGenerator
     void lineBreak() { t << "\n.br" << endl; }
     void writeChar(char c);
     void startMemberDoc(const char *,const char *,const char *,const char *);
-    void endMemberDoc();
+    void endMemberDoc(bool);
     void startDoxyAnchor(const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *) {}
     void startCodeAnchor(const char *) {}
@@ -148,45 +153,47 @@ class ManGenerator : public OutputGenerator
     void endCenter()          {}
     void startSmall()         {}
     void endSmall()           {}
-    void startSubscript()     { t << "\\*<"; firstCol=FALSE; }
-    void endSubscript()       { t << "\\*>"; firstCol=FALSE; }
-    void startSuperscript()   { t << "\\*{"; firstCol=FALSE; }
-    void endSuperscript()     { t << "\\*}"; firstCol=FALSE; }
-    void startTable(bool,int) {}
-    void endTable(bool)       {}
-    void startCaption()       {}
-    void endCaption()         {}
-    void nextTableRow()       {}
-    void endTableRow()        {}
-    void nextTableColumn()    {}
-    void endTableColumn()     {}
-    void writeCopyright()     { t << "(c)"; firstCol=FALSE; }
-    void writeQuote()         { t << "`"; firstCol=FALSE; }
-    void writeUmlaut(char c)  { t << c << "\\*'"; firstCol=FALSE; }
-    void writeAcute(char c)   { t << c << "\\*`"; firstCol=FALSE; }
-    void writeGrave(char c)   { t << c << "\\*:"; firstCol=FALSE; }
-    void writeCirc(char c)    { t << c << "\\*^"; firstCol=FALSE; }
-    void writeTilde(char c)   { t << c << "\\*~"; firstCol=FALSE; } 
-    void writeRing(char c)    { t << c << "\\*0"; /* just a wild guess, 
-                                                     need to check! */ 
-                                                  firstCol=FALSE; }
-    void writeSharpS()        { t << "s\\*:";     /* just a wild guess,
-                                                     need to check! */
-                                                  firstCol=FALSE; }
-    void writeCCedil(char c)  { t << c;           /* TODO: fix this */
-                                firstCol=FALSE; }
+    //void startSubscript()     { t << "\\*<"; firstCol=FALSE; }
+    //void endSubscript()       { t << "\\*>"; firstCol=FALSE; }
+    //void startSuperscript()   { t << "\\*{"; firstCol=FALSE; }
+    //void endSuperscript()     { t << "\\*}"; firstCol=FALSE; }
+    //void startTable(bool,int) {}
+    //void endTable(bool)       {}
+    //void startCaption()       {}
+    //void endCaption()         {}
+    //void nextTableRow()       {}
+    //void endTableRow()        {}
+    //void nextTableColumn()    {}
+    //void endTableColumn()     {}
+    //void writeCopyright()     { t << "(c)"; firstCol=FALSE; }
+    //void writeQuote()         { t << "`"; firstCol=FALSE; }
+    //void writeUmlaut(char c)  { t << c << "\\*'"; firstCol=FALSE; }
+    //void writeAcute(char c)   { t << c << "\\*`"; firstCol=FALSE; }
+    //void writeGrave(char c)   { t << c << "\\*:"; firstCol=FALSE; }
+    //void writeCirc(char c)    { t << c << "\\*^"; firstCol=FALSE; }
+    //void writeTilde(char c)   { t << c << "\\*~"; firstCol=FALSE; } 
+    //void writeRing(char c)    { t << c << "\\*0"; /* just a wild guess, 
+    //                                                 need to check! */ 
+    //                                              firstCol=FALSE; }
+    //void writeSharpS()        { t << "s\\*:";     /* just a wild guess,
+    //                                                 need to check! */
+    //                                              firstCol=FALSE; }
+    //void writeCCedil(char c)  { t << c;           /* TODO: fix this */
+    //                            firstCol=FALSE; }
     void startMemberDescription() { t << "\n.RI \"\\fI"; firstCol=FALSE; }
     void endMemberDescription()   { t << "\\fP\""; firstCol=FALSE; }
     void startDescList(SectionTypes);
     void endDescList()        {}
-    void startParamList(ParamListTypes);
-    void endParamList()       {}
-    void endDescTitle();
+    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
+    void endSimpleSect();
+    void startParamList(ParamListTypes,const char *title);
+    void endParamList();
+    //void endDescTitle();
     void writeDescItem();
-    void startSection(const char *,const char *,bool);
-    void endSection(const char *,bool);
-    void writeSectionRef(const char *,const char *,const char *,const char *) {}
-    void writeSectionRefItem(const char *,const char *,const char *) {}
+    void startSection(const char *,const char *,SectionInfo::SectionType);
+    void endSection(const char *,SectionInfo::SectionType);
+    //void writeSectionRef(const char *,const char *,const char *,const char *) {}
+    //void writeSectionRefItem(const char *,const char *,const char *) {}
     //void writeSectionRefAnchor(const char *,const char *,const char *) {}
     void addIndexItem(const char *,const char *) {}
     void startIndent()        {}
@@ -199,14 +206,15 @@ class ManGenerator : public OutputGenerator
     //void endColorFont()   {}
     void startPageRef() {}
     void endPageRef(const char *,const char *) {}
-    void startQuickIndexItem(const char *,const char *) {}
-    void endQuickIndexItem() {}
-    void writeFormula(const char *,const char *) {}
+    //void startQuickIndexItem(const char *,const char *) {}
+    //void endQuickIndexItem() {}
+    void writeQuickLinks(bool,HighlightedItem) {}
+    //void writeFormula(const char *,const char *) {}
     void writeNonBreakableSpace(int n) { int i; for (i=0;i<n;i++) t << " "; }
-    void startImage(const char *,const char *,bool) {}
-    void endImage(bool) {}
-    void startDotFile(const char *,bool) {}
-    void endDotFile(bool) {}
+    //void startImage(const char *,const char *,bool) {}
+    //void endImage(bool) {}
+    //void startDotFile(const char *,bool) {}
+    //void endDotFile(bool) {}
     
     void startDescTable() {}
     void endDescTable() {}
@@ -219,6 +227,8 @@ class ManGenerator : public OutputGenerator
     void endDotGraph(DotClassGraph &) {}
     void startInclDepGraph() {}
     void endInclDepGraph(DotInclDepGraph &) {}
+    void startCallGraph() {}
+    void endCallGraph(DotCallGraph &) {}
     void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
 
     void startTextBlock(bool) {}
@@ -227,22 +237,27 @@ class ManGenerator : public OutputGenerator
 
     void startMemberDocPrefixItem() {}
     void endMemberDocPrefixItem() {}
-    void startMemberDocName() {}
+    void startMemberDocName(bool) {}
     void endMemberDocName() {}
-    void startParameterType(bool) {}
+    void startParameterType(bool,const char *) {}
     void endParameterType() {}
     void startParameterName(bool) {}
-    void endParameterName(bool,bool) {}
-    void startParameterList() {}
+    void endParameterName(bool,bool,bool) {}
+    void startParameterList(bool) {}
     void endParameterList() {}
 
     void startFontClass(const char *) {}
     void endFontClass() {}
 
-    void startHtmlOnly()  {}
-    void endHtmlOnly()    {}
-    void startLatexOnly() {}
-    void endLatexOnly()   {}
+    //void startHtmlOnly()  {}
+    //void endHtmlOnly()    {}
+    //void startLatexOnly() {}
+    //void endLatexOnly()   {}
+
+    //void startSectionRefList() {}
+    //void endSectionRefList() {}
+
+    void writeCodeAnchor(const char *) {}
 
   private:
     bool firstCol;
