@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: membergroup.cpp,v 1.19 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -27,6 +27,7 @@
 #include "scanner.h"
 #include "groupdef.h"
 #include "doxygen.h"
+#include "docparser.h"
 
 //static QCString idToName(int id)
 //{
@@ -35,16 +36,20 @@
 //  return result;
 //}
 
-MemberGroup::MemberGroup(int id,const char *hdr,const char *d) /* : Definition(idToName(id)) */
+MemberGroup::MemberGroup(Definition *parent,
+      int id,const char *hdr,const char *d,const char *docFile) 
 {
-  memberList     = new MemberList;
-  grpId          = id;
-  grpHeader      = hdr;
-  doc            = d;
-  scope          = 0;
-  inSameSection  = TRUE;
-  inDeclSection  = 0;
+  //printf("New member group id=%d header=%s desc=%s\n",id,hdr,d);
+  memberList      = new MemberList;
+  grpId           = id;
+  grpHeader       = hdr;
+  doc             = d;
+  scope           = 0;
+  inSameSection   = TRUE;
+  inDeclSection   = 0;
   m_numDecMembers = -1;
+  m_parent        = parent;
+  m_docFile       = docFile;
   //printf("Member group docs=`%s'\n",doc.data());
 }
 
@@ -53,7 +58,7 @@ MemberGroup::~MemberGroup()
   delete memberList;
 }
 
-void MemberGroup::insertMember(Definition *d,MemberDef *md)
+void MemberGroup::insertMember(MemberDef *md)
 {
   //printf("MemberGroup::insertMember memberList=%p count=%d"
   //       " member section list: %p\n",
@@ -62,13 +67,13 @@ void MemberGroup::insertMember(Definition *d,MemberDef *md)
   //       md->getSectionList());
   MemberDef *firstMd = memberList->first();
   if (inSameSection && memberList->count()>0 && 
-      firstMd->getSectionList(d)!=md->getSectionList(d))
+      firstMd->getSectionList(m_parent)!=md->getSectionList(m_parent))
   {
     inSameSection=FALSE;
   }
   else if (inDeclSection==0)
   {
-    inDeclSection = md->getSectionList(d);
+    inDeclSection = md->getSectionList(m_parent);
   }
   memberList->append(md);
 
@@ -133,17 +138,22 @@ void MemberGroup::distributeMemberGroupDocumentation()
 {
   //printf("MemberGroup::distributeMemberGroupDocumentation() %s\n",grpHeader.data());
   MemberDef *md=memberList->first();
-  if (md && !(md->documentation().isEmpty() && md->briefDescription().isEmpty()))
+  if (md && !(md->documentation().isEmpty() && 
+              md->briefDescription().isEmpty() && 
+              md->inbodyDocumentation().isEmpty()
+             )
+     )
   {
     //printf("First member %s has documentation!\n",md->name().data());
     MemberDef *omd=memberList->next();
     while (omd)
     {
-      if (omd->documentation().isEmpty() && omd->briefDescription().isEmpty())
+      if (omd->documentation().isEmpty() && omd->briefDescription().isEmpty() && omd->inbodyDocumentation().isEmpty())
       {
         //printf("Copying documentation to member %s\n",omd->name().data());
         omd->setBriefDescription(md->briefDescription(),md->briefFile(),md->briefLine());
         omd->setDocumentation(md->documentation(),md->docFile(),md->docLine());
+        omd->setInbodyDocumentation(md->inbodyDocumentation(),md->inbodyFile(),md->inbodyLine());
       }
       omd=memberList->next();
     }
@@ -209,3 +219,10 @@ void MemberGroup::addListReferences(Definition *def)
 {
   memberList->addListReferences(def);
 }
+
+void MemberGroup::findSectionsInDocumentation()
+{
+  docFindSections(doc,0,this,m_docFile);
+  memberList->findSectionsInDocumentation();
+}
+
