@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: membergroup.h,v 1.13 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -22,7 +22,7 @@
 #include <qlist.h>
 #include "sortdict.h"
 
-#define NOGROUP -1
+#define DOX_NOGROUP -1
 
 class MemberDef;
 class ClassDef;
@@ -36,11 +36,12 @@ class Definition;
 class MemberGroup 
 {
   public:
-    MemberGroup(int id,const char *header,const char *docs);
+    MemberGroup(Definition *parent,int id,const char *header,
+                const char *docs,const char *docFile);
    ~MemberGroup();
     QCString header() const { return grpHeader; }
     int groupId() const { return grpId; }
-    void insertMember(Definition *d,MemberDef *md);
+    void insertMember(MemberDef *md);
     void setAnchors(ClassDef *);
     void writePlainDeclarations(OutputList &ol,
                ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
@@ -54,6 +55,7 @@ class MemberGroup
     void addToDeclarationSection();
     int countDecMembers(/*bool sectionPerType*/);
     void distributeMemberGroupDocumentation();
+    void findSectionsInDocumentation();
     int varCount() const;
     int funcCount() const;
     int enumCount() const;
@@ -67,6 +69,7 @@ class MemberGroup
     void setInGroup(bool b);
     void addListReferences(Definition *d);
     MemberList *members() const { return memberList; }
+    Definition *parent() const { return m_parent; }
 
   private: 
     MemberList *memberList;      // list of all members in the group
@@ -78,6 +81,8 @@ class MemberGroup
     MemberList *inDeclSection;
     bool inSameSection;
     int  m_numDecMembers;
+    Definition *m_parent;
+    QCString m_docFile;
 };
 
 class MemberGroupList : public QList<MemberGroup>
@@ -98,6 +103,13 @@ class MemberGroupSDict : public SIntDict<MemberGroup>
    ~MemberGroupSDict() {}
 };
 
+struct MemberGroupInfo
+{
+  QCString header;
+  QCString doc;
+  QCString docFile;
+};
+
 //class MemberGroupDict : public QIntDict<MemberGroup>
 //{
 //  public:
