@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: memberlist.cpp,v 1.35 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -24,7 +24,6 @@
 #include "language.h"
 #include "doxygen.h"
 #include "outputlist.h"
-#include "doc.h"
 #include "groupdef.h"
 
 MemberList::MemberList() : QList<MemberDef>()
@@ -67,6 +66,7 @@ void MemberList::countDecMembers()
       switch(md->memberType())
       {
         case MemberDef::Variable:    // fall through
+        case MemberDef::Event:       // fall through
         case MemberDef::Property:    m_varCnt++,m_numDecMembers++;  
                                      break;
         case MemberDef::Function:    // fall through
@@ -179,206 +179,101 @@ void MemberList::writePlainDeclarations(OutputList &ol,
   MemberListIterator mli(*this);
   for ( ; (md=mli.current()); ++mli )
   {
-    //printf(">>> Member `%s' type=%d\n",md->name().data(),md->memberType());
-    switch(md->memberType())
+    if (md->isBriefSectionVisible())
     {
-      case MemberDef::Define:    // fall through
-      case MemberDef::Prototype: // fall through
-      case MemberDef::Typedef:   // fall through
-      case MemberDef::Variable:  // fall through
-      case MemberDef::Function:  // fall through
-      case MemberDef::Signal:    // fall through
-      case MemberDef::Slot:      // fall through
-      case MemberDef::DCOP:      // fall through
-      case MemberDef::Property:  
+      //printf(">>> Member `%s' type=%d\n",md->name().data(),md->memberType());
+      switch(md->memberType())
       {
-        if (first) ol.startMemberList(),first=FALSE;
-        md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
-        break;
-      }
-      case MemberDef::Enumeration: 
-      {
-        if (first) ol.startMemberList(),first=FALSE;
-        OutputList typeDecl(&ol);
-        QCString name(md->name());
-        int i=name.findRev("::");
-        if (i!=-1) name=name.right(name.length()-i-2); // strip scope (TODO: is this needed?)
-        if (name[0]!='@') // not an anonymous enum
-        {
-          if (md->isLinkableInProject() || md->hasDocumentedEnumValues())
+        case MemberDef::Define:    // fall through
+        case MemberDef::Prototype: // fall through
+        case MemberDef::Typedef:   // fall through
+        case MemberDef::Variable:  // fall through
+        case MemberDef::Function:  // fall through
+        case MemberDef::Signal:    // fall through
+        case MemberDef::Slot:      // fall through
+        case MemberDef::DCOP:      // fall through
+        case MemberDef::Property:  // fall through
+        case MemberDef::Event:  
           {
-            if (!Config_getString("GENERATE_TAGFILE").isEmpty())
+            if (first) ol.startMemberList(),first=FALSE;
+            md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
+            break;
+          }
+        case MemberDef::Enumeration: 
+          {
+            if (first) ol.startMemberList(),first=FALSE;
+            int enumVars=0;
+            MemberListIterator vmli(*this);
+            MemberDef *vmd;
+            QCString name(md->name());
+            int i=name.findRev("::");
+            if (i!=-1) name=name.right(name.length()-i-2); // strip scope (TODO: is this needed?)
+            if (name[0]=='@') // anonymous enum => append variables
             {
-              Doxygen::tagFile << "    <member kind=\"enumeration\">" << endl;
-              Doxygen::tagFile << "      <name>" << convertToXML(md->name()) << "</name>" << endl; 
-              Doxygen::tagFile << "      <anchor>" << convertToXML(md->anchor()) << "</anchor>" << endl; 
-              Doxygen::tagFile << "      <arglist>" << convertToXML(md->argsString()) << "</arglist>" << endl; 
-              Doxygen::tagFile << "    </member>" << endl;
-            }
-            md->writeLink(typeDecl,cd,nd,fd,gd);
-          }
-          else
-          {
-            typeDecl.startBold();
-            typeDecl.docify(name);
-            typeDecl.endBold();
-          }
-          typeDecl.writeChar(' ');
-        }
-
-        int enumMemCount=0;
-
-        QList<MemberDef> *fmdl=md->enumFieldList();
-        uint numVisibleEnumValues=0;
-        if (fmdl)
-        {
-          MemberDef *fmd=fmdl->first();
-          while (fmd)
-          {
-            if (fmd->isBriefSectionVisible()) numVisibleEnumValues++;
-            fmd=fmdl->next();
-          }
-        }
-        if (numVisibleEnumValues==0 && !md->isBriefSectionVisible()) break;
-        if (numVisibleEnumValues>0)
-        {
-          uint enumValuesPerLine = (uint)Config_getInt("ENUM_VALUES_PER_LINE");
-          typeDecl.docify("{ ");
-          if (fmdl)
-          {
-            MemberDef *fmd=fmdl->first();
-            bool fmdVisible = fmd->isBriefSectionVisible();
-            while (fmd)
-            {
-              if (fmdVisible)
+              for ( ; (vmd=vmli.current()) ; ++vmli)
               {
-                /* in html we start a new line after a number of items */
-                if (numVisibleEnumValues>enumValuesPerLine
-                    && (enumMemCount%enumValuesPerLine)==0
-                   )
+                QCString vtype=vmd->typeString();
+                if ((vtype.find(name))!=-1) 
                 {
-                  typeDecl.pushGeneratorState();
-                  typeDecl.disableAllBut(OutputGenerator::Html);
-                  typeDecl.lineBreak(); 
-                  typeDecl.writeString("&nbsp;&nbsp;");
-                  typeDecl.popGeneratorState();
-                }
-
-                if (fmd->hasDocumentation()) // enum value has docs
-                {
-                  if (!Config_getString("GENERATE_TAGFILE").isEmpty())
-                  {
-                    Doxygen::tagFile << "    <member kind=\"enumvalue\">" << endl;
-                    Doxygen::tagFile << "      <name>" << convertToXML(fmd->name()) << "</name>" << endl; 
-                    Doxygen::tagFile << "      <anchor>" << convertToXML(fmd->anchor()) << "</anchor>" << endl; 
-                    Doxygen::tagFile << "      <arglist>" << convertToXML(fmd->argsString()) << "</arglist>" << endl; 
-                    Doxygen::tagFile << "    </member>" << endl;
-                  }
-                  fmd->writeLink(typeDecl,cd,nd,fd,gd);
-                }
-                else // no docs for this enum value
-                {
-                  typeDecl.startBold();
-                  typeDecl.docify(fmd->name());
-                  typeDecl.endBold();
-                }
-                if (fmd->hasOneLineInitializer()) // enum value has initializer
-                {
-                  typeDecl.writeString(" = ");
-                  parseText(typeDecl,fmd->initializer());
+                  enumVars++;
+                  vmd->setAnonymousEnumType(md);
                 }
               }
-
-              bool prevVisible = fmdVisible;
-              fmd=fmdl->next();
-              if (fmd && (fmdVisible=fmd->isBriefSectionVisible())) 
-              {
-                typeDecl.writeString(", ");
-              }
-              if (prevVisible)
-              {
-                typeDecl.disable(OutputGenerator::Man);
-                typeDecl.writeString("\n"); // to prevent too long lines in LaTeX
-                typeDecl.enable(OutputGenerator::Man);
-                enumMemCount++;
-              }
             }
-            if (numVisibleEnumValues>enumValuesPerLine)
+            // if this is an anoymous enum and there are variables of this
+            // enum type (i.e. enumVars>0), then we do not show the enum here.
+            if (enumVars==0) // show enum here
             {
-              typeDecl.pushGeneratorState();
-              typeDecl.disableAllBut(OutputGenerator::Html);
-              typeDecl.lineBreak(); 
-              typeDecl.popGeneratorState();
-            }
-          }
-          typeDecl.docify(" }");
-          md->setEnumDecl(typeDecl);
-        }
-        int enumVars=0;
-        MemberListIterator vmli(*this);
-        MemberDef *vmd;
-        if (name[0]=='@') // anonymous enum => append variables
-        {
-          for ( ; (vmd=vmli.current()) ; ++vmli)
-          {
-            QCString vtype=vmd->typeString();
-            if ((vtype.find(name))!=-1) 
-            {
-              enumVars++;
-              vmd->setAnonymousEnumType(md);
-            }
-          }
-        }
-        // if this is an anoymous enum and there are variable of this
-        // enum type (i.e. enumVars>0), then we do not show the enum here.
-        if (enumVars==0) // show enum here
-        {
-          ol.startMemberItem(0);
-          ol.writeString("enum ");
-          ol.insertMemberAlign();
-          ol+=typeDecl; // append the enum values.
-          ol.endMemberItem(FALSE);
-          if (!md->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
-          {
-            ol.startMemberDescription();
-            parseDoc(ol,
-                md->getDefFileName(),md->getDefLine(),
-                cd?cd->name().data():0,md,
-                md->briefDescription()
+              ol.startMemberItem(0);
+              ol.writeString("enum ");
+              ol.insertMemberAlign();
+              //ol+=typeDecl; // append the enum values.
+              md->writeEnumDeclaration(ol,cd,nd,fd,gd);
+              ol.endMemberItem();
+              if (!md->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
+              {
+                ol.startMemberDescription();
+                ol.parseDoc(
+                    md->briefFile(),md->briefLine(),
+                    cd,md,
+                    md->briefDescription(),
+                    TRUE,
+                    FALSE
                     );
-            if (md->isDetailedSectionLinkable())
-            {
-              ol.disableAllBut(OutputGenerator::Html);
-              ol.endEmphasis();
-              ol.docify(" ");
-              if (md->getGroupDef()!=0 && gd==0) // forward link to group
-              {
-                ol.startTextLink(md->getGroupDef()->getOutputFileBase(),
-                                 md->anchor());
+                if (md->isDetailedSectionLinkable())
+                {
+                  ol.disableAllBut(OutputGenerator::Html);
+                  //ol.endEmphasis();
+                  ol.docify(" ");
+                  if (md->getGroupDef()!=0 && gd==0) // forward link to group
+                  {
+                    ol.startTextLink(md->getOutputFileBase(),
+                                     md->anchor());
+                  }
+                  else
+                  {
+                    ol.startTextLink(0,md->anchor());
+                  }
+                  ol.parseText(theTranslator->trMore());
+                  ol.endTextLink();
+                  //ol.startEmphasis();
+                  ol.enableAll();
+                }
+                ol.endMemberDescription();
               }
-              else
-              {
-                ol.startTextLink(0,md->anchor());
-              }
-              parseText(ol,theTranslator->trMore());
-              ol.endTextLink();
-              ol.startEmphasis();
-              ol.enableAll();
             }
-            ol.endMemberDescription();
+            md->warnIfUndocumented();
+            break;
           }
-        }
-        md->warnIfUndocumented();
-        break;
+        case MemberDef::Friend:
+          {
+            if (first) ol.startMemberList(),first=FALSE;
+            md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
+            break;
+          }
+        case MemberDef::EnumValue: 
+          break;
       }
-      case MemberDef::Friend:
-      {
-        if (first) ol.startMemberList(),first=FALSE;
-        md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
-        break;
-      }
-      case MemberDef::EnumValue: 
-        break;
     }
   }
 
@@ -392,10 +287,13 @@ void MemberList::writePlainDeclarations(OutputList &ol,
       if (md->fromAnonymousScope() && !md->anonymousDeclShown())
       {
         //printf("anonymous compound members\n");
-        if (first) ol.startMemberList(),first=FALSE;
-        md->setFromAnonymousScope(FALSE);
-        md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
-        md->setFromAnonymousScope(TRUE);
+        if (md->isBriefSectionVisible())
+        {
+          if (first) ol.startMemberList(),first=FALSE;
+          md->setFromAnonymousScope(FALSE);
+          md->writeDeclaration(ol,cd,nd,fd,gd,m_inGroup);
+          md->setFromAnonymousScope(TRUE);
+        }
       }
     }
   }
@@ -418,14 +316,14 @@ void MemberList::writeDeclarations(OutputList &ol,
   if (title) 
   {
     ol.startMemberHeader();
-    parseText(ol,title);
+    ol.parseText(title);
     ol.endMemberHeader();
   }
   if (subtitle && subtitle[0]!=0) 
   {
     //printf("subtitle=`%s'\n",subtitle);
     ol.startMemberSubtitle();
-    parseDoc(ol,"<generated>",1,0,0,subtitle);
+    ol.parseDoc("<generated>",-1,0,0,subtitle,FALSE,FALSE);
     ol.endMemberSubtitle();
   }
 
@@ -442,14 +340,14 @@ void MemberList::writeDeclarations(OutputList &ol,
       ol.startMemberGroupHeader(hasHeader);
       if (hasHeader)
       {
-        parseText(ol,mg->header());
+        ol.parseText(mg->header());
       }
       ol.endMemberGroupHeader();
       if (!mg->documentation().isEmpty())
       {
         //printf("Member group has docs!\n");
         ol.startMemberGroupDocs();
-        parseDoc(ol,"<generated>",1,0,0,mg->documentation()+"\n");
+        ol.parseDoc("<generated>",-1,0,0,mg->documentation()+"\n",FALSE,FALSE);
         ol.endMemberGroupDocs();
       }
       ol.startMemberGroup();
@@ -476,7 +374,7 @@ void MemberList::writeDocumentation(OutputList &ol,
   {
     ol.writeRuler();
     ol.startGroupHeader();
-    parseText(ol,title);
+    ol.parseText(title);
     ol.endGroupHeader();
   }
   
@@ -516,6 +414,15 @@ void MemberList::addListReferences(Definition *def)
     if (md->getGroupDef()==0 || def->definitionType()==Definition::TypeGroup)
     {
       md->addListReference(def);
+      if (md->memberType()==MemberDef::Enumeration && md->enumFieldList())
+      {
+        MemberListIterator vmli(*md->enumFieldList());
+        MemberDef *vmd;
+        for ( ; (vmd=vmli.current()) ; ++vmli)
+        {
+          vmd->addListReference(def);
+        }
+      }
     }
   }
   if (memberGroupList)
@@ -529,6 +436,25 @@ void MemberList::addListReferences(Definition *def)
   }
 }
 
+void MemberList::findSectionsInDocumentation()
+{
+  MemberListIterator mli(*this);
+  MemberDef *md;
+  for ( ; (md=mli.current()) ; ++mli)
+  {
+    md->findSectionsInDocumentation();
+  }
+  if (memberGroupList)
+  {
+    MemberGroupListIterator mgli(*memberGroupList);
+    MemberGroup *mg;
+    for (;(mg=mgli.current());++mgli)
+    {
+      mg->findSectionsInDocumentation();
+    }
+  }
+}
+
 //--------------------------------------------------------------------------
 
 int MemberSDict::compareItems(GCI item1, GCI item2)
