@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: memberlist.h,v 1.19 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -57,6 +57,7 @@ class MemberList : public QList<MemberDef>
     void addMemberGroup(MemberGroup *mg);
     void setInGroup(bool group) { m_inGroup=group; }
     void addListReferences(Definition *def);
+    void findSectionsInDocumentation();
 
   private:
     int m_varCnt,m_funcCnt,m_enumCnt,m_enumValCnt,m_typeCnt;
