@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: namespacedef.h,v 1.18 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -43,6 +43,8 @@ class NamespaceDef : public Definition
     DefType definitionType() { return TypeNamespace; }
     QCString getOutputFileBase() const;
     void insertUsedFile(const char *fname);
+    
+    void writeDetailedDocumentation(OutputList &ol);
     void writeDocumentation(OutputList &ol);
 
     void insertClass(ClassDef *cd);
@@ -52,9 +54,10 @@ class NamespaceDef : public Definition
     void computeAnchors();
     int countMembers();
     void addUsingDirective(NamespaceDef *nd);
-    NamespaceList *getUsedNamespaces() const { return usingDirList; }
-    void addUsingDeclaration(ClassDef *cd);
-    ClassList *getUsedClasses() const { return usingDeclList; }
+    NamespaceSDict *getUsedNamespaces() const { return usingDirList; }
+    void addUsingDeclaration(Definition *def);
+    SDict<Definition> *getUsedClasses() const { return usingDeclList; }
+    void combineUsingRelations();
     QCString displayName() const;
     
     bool isLinkableInProject() const
@@ -70,6 +73,7 @@ class NamespaceDef : public Definition
     }
     void addMembersToMemberGroup();
     void distributeMemberGroupDocumentation();
+    void findSectionsInDocumentation();
 
     virtual Definition *findInnerCompound(const char *name);
     void addInnerCompound(Definition *d);
@@ -105,14 +109,16 @@ class NamespaceDef : public Definition
     /*! Namespaces inside this namespace */
     NamespaceSDict *namespaceSDict;
 
+    bool visited;
+
   private:
     //QCString reference;
     QCString fileName;
     QStrList files;
 
 
-    NamespaceList *usingDirList;
-    ClassList *usingDeclList;
+    NamespaceSDict *usingDirList;
+    SDict<Definition> *usingDeclList;
     SDict<Definition> *m_innerCompounds;
 
     MemberList allMemberList;
