@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: outputgen.h,v 1.48 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -24,42 +24,84 @@
 #include <qfile.h>
 #include <qstack.h>
 #include "index.h"
+#include "section.h"
 
 class ClassDiagram;
 class DotClassGraph;
 class DotInclDepGraph;
+class DotCallGraph;
 class DotGfxHierarchyTable;
+class DocNode;
+class MemberDef;
 
-/*! \brief Base Interface used for generating documentation.
- *
- *  This abstract class is used by the documentation block
- *  parser to generate the output for a specific format,
- *  or a list of formats (see OutputList). This interface
- *  contains functions that generate output.
+/*! \brief Output interface for code parser. 
  */
-class BaseOutputDocInterface
+class BaseCodeDocInterface
+{
+  public:
+    /*! Writes an ASCII string to the output. This function should keep 
+     *  spaces visible, should break lines at a newline and should convert 
+     *  tabs to the right number of spaces.
+     */
+    virtual void codify(const char *s) = 0;
+
+    /*! Writes a link to an object in a code fragment.
+     *  \param ref    If this is non-zero, the object is to be found in
+     *                an external documentation file.
+     *  \param file   The file in which the object is located.
+     *  \param anchor The anchor uniquely identifying the object within 
+     *                the file. 
+     *  \param name   The text to display as a placeholder for the link.
+     */
+    virtual void writeCodeLink(const char *ref,const char *file,
+                               const char *anchor,const char *name) = 0;
+
+    virtual void writeLineNumber(const char *ref,const char *file,
+                                 const char *anchor,int lineNumber) = 0;
+    virtual void startCodeLine() = 0;
+    virtual void endCodeLine() = 0;
+    virtual void startCodeAnchor(const char *label) = 0;
+    virtual void endCodeAnchor() = 0;
+    virtual void startFontClass(const char *) = 0;
+    virtual void endFontClass() = 0;
+    virtual void writeCodeAnchor(const char *name) = 0;
+};
+
+/*! \brief Base Interface used for generating output outside of the
+ *  comment blocks.
+ *
+ *  This abstract class is used by output generation functions
+ *  to generate the output for a specific format,
+ *  or a list of formats (see OutputList). This interface
+ *  contains functions that generate fragments of the output.
+ */
+class BaseOutputDocInterface : public BaseCodeDocInterface
 {
   public:
     enum ParamListTypes { Param, RetVal, Exception };
-    enum SectionTypes { See, Return, Author, Version, 
+    enum SectionTypes { /*See, Return, Author, Version, 
                         Since, Date, Bug, Note,
                         Warning, Par, Deprecated, Pre, 
                         Post, Invar, Remark, Attention, 
-                        Todo, Test, RCS, EnumValues, 
+                        Todo, Test, RCS, */ EnumValues, 
                         Examples 
                       };
 
-    /*! Start of a bullet list: e.g. \c <ul> in html. writeListItem() is
+    virtual void parseDoc(const char *,int, const char *,MemberDef *,
+                          const QCString &,bool)  {} 
+    virtual void parseText(const QCString &)  {}
+    
+    /*! Start of a bullet list: e.g. \c \<ul\> in html. writeListItem() is
      *  Used for the bullet items.
      */
     virtual void startItemList()  = 0;
 
     /*! Writes a list item for a bullet or enumerated 
-     *  list: e.g. \c <li> in html 
+     *  list: e.g. \c \<li\> in html 
      */
     virtual void writeListItem()  = 0;
 
-    /*! Ends a bullet list: e.g. \c </ul> in html */
+    /*! Ends a bullet list: e.g. \c \</ul\> in html */
     virtual void endItemList()    = 0;
 
     /*! Writes an ASCII string to the output. Converts characters that have
@@ -80,15 +122,6 @@ class BaseOutputDocInterface
     /*! Starts a new paragraph */
     virtual void newParagraph()   = 0;
 
-    /*! Starts a enumeration list: e.g. \c <ol> in html. 
-     *  writeListItem() is used for the items. 
-     */
-    virtual void startEnumList()  = 0;
-
-    /*! Ends an enumeration list: e.g. \c </ol> in html. 
-     */
-    virtual void endEnumList()    = 0;
-
     /*! Writes a link to an object in the documentation.
      *  \param ref    If this is non-zero, the object is to be found in
      *                an external documentation file.
@@ -100,16 +133,6 @@ class BaseOutputDocInterface
     virtual void writeObjectLink(const char *ref,const char *file,
                                  const char *anchor, const char *name) = 0;
 
-    /*! Writes a link to an object in a code fragment.
-     *  \param ref    If this is non-zero, the object is to be found in
-     *                an external documentation file.
-     *  \param file   The file in which the object is located.
-     *  \param anchor The anchor uniquely identifying the object within 
-     *                the file. 
-     *  \param name   The text to display as a placeholder for the link.
-     */
-    virtual void writeCodeLink(const char *ref,const char *file,
-                               const char *anchor,const char *name) = 0;
 
     /*! Starts a (link to an) URL found in the documentation.
      *  \param url    The URL to link to.
@@ -120,10 +143,6 @@ class BaseOutputDocInterface
      */
     virtual void endHtmlLink() = 0;
 
-    /*! Writes a (link to an) email address found in the documentation.
-     *  \param url    The email address, this is also used for the link text.
-     */
-    virtual void writeMailLink(const char *url) = 0;
     
     /*! Changes the text font to bold face. The bold section ends with
      *  endBold()
@@ -160,84 +179,51 @@ class BaseOutputDocInterface
      */
     virtual void endCodeFragment() = 0;
 
-    /*! Starts a fragment of preformatted text. This means that
-     *  spacing, tabs and newlines should be kept in the output
-     */
+    
 
-    virtual void startPreFragment() = 0;
-    /*! Ens a preformatted text fragment. */
-
-    virtual void endPreFragment() = 0;
+    
     /*! Writes a horizontal ruler to the output */
     virtual void writeRuler() = 0;
     
-    /*! Starts a description list: e.g. \c <dl> in HTML 
+    /*! Starts a description list: e.g. \c \<dl\> in HTML 
      *  Items are surrounded by startDescItem() and endDescItem()
      */
     virtual void startDescription() = 0;
 
-    /*! Ends a description list: e.g. \c </dl> in HTML */
+    /*! Ends a description list: e.g. \c \</dl\> in HTML */
     virtual void endDescription() = 0;
 
-    /*! Starts an item of a description list: e.g. \c <dt> in HTML. */
+    /*! Starts an item of a description list: e.g. \c \<dt\> in HTML. */
     virtual void startDescItem() = 0;
 
     /*! Ends an item of a description list and starts the 
-     *  description itself: e.g. \c <dd> in HTML. 
+     *  description itself: e.g. \c \<dd\> in HTML. 
      */
     virtual void endDescItem() = 0;
-    virtual void startSubsection() = 0;
-    virtual void endSubsection() = 0;
-    virtual void startSubsubsection() = 0;
-    virtual void endSubsubsection() = 0;
+
     virtual void startCenter() = 0;
     virtual void endCenter() = 0;
     virtual void startSmall() = 0;
     virtual void endSmall() = 0;
-    virtual void startSubscript() = 0;
-    virtual void endSubscript() = 0;
-    virtual void startSuperscript() = 0;
-    virtual void endSuperscript() = 0;
-    virtual void startTable(bool hasCaption,int cols) = 0;
-    virtual void endTable(bool hasCaption) = 0;
-    virtual void startCaption() = 0;
-    virtual void endCaption() = 0;
-    virtual void nextTableRow() = 0;
-    virtual void endTableRow() = 0;
-    virtual void nextTableColumn() = 0;
-    virtual void endTableColumn() = 0;
-    virtual void writeQuote() = 0;
-    virtual void writeCopyright() = 0;
-    virtual void writeUmlaut(char) = 0;
-    virtual void writeAcute(char) = 0;
-    virtual void writeGrave(char) = 0;
-    virtual void writeCirc(char) = 0;
-    virtual void writeTilde(char) = 0;
-    virtual void writeRing(char) = 0;
-    virtual void writeSharpS() = 0;
-    virtual void writeCCedil(char) = 0;
-    virtual void startDescList(SectionTypes t) = 0;
-    virtual void endDescList() = 0;
-    virtual void startParamList(ParamListTypes t) = 0;
+
+    virtual void startSimpleSect(SectionTypes t,const char *file,
+                                 const char *anchor,const char *title) = 0;
+    virtual void endSimpleSect() = 0;
+    virtual void startParamList(ParamListTypes t,const char *title) = 0;
     virtual void endParamList() = 0;
-    virtual void endDescTitle() = 0;
+
     virtual void writeDescItem() = 0;
     virtual void startTitle() = 0;
     virtual void endTitle()   = 0;
+
     virtual void writeAnchor(const char *fileName,const char *name) = 0;
-    virtual void startSection(const char *,const char *,bool) = 0;
-    virtual void endSection(const char *,bool) = 0;
-    virtual void writeSectionRef(const char *,const char *,
-                                 const char *,const char *) = 0;
-    virtual void writeSectionRefItem(const char *,const char *,const char *) = 0;
+    virtual void startSection(const char *,const char *,SectionInfo::SectionType) = 0;
+    virtual void endSection(const char *,SectionInfo::SectionType) = 0;
+
     virtual void lineBreak() = 0;
     virtual void addIndexItem(const char *s1,const char *s2) = 0;
-    virtual void writeFormula(const char *,const char *) = 0;
+
     virtual void writeNonBreakableSpace(int) = 0;
-    virtual void startImage(const char *,const char *,bool) = 0;
-    virtual void endImage(bool) = 0;
-    virtual void startDotFile(const char *,bool) = 0;
-    virtual void endDotFile(bool) = 0;
     virtual void startDescTable() = 0;
     virtual void endDescTable() = 0;
     virtual void startDescTableTitle() = 0;
@@ -248,28 +234,10 @@ class BaseOutputDocInterface
     virtual void endTextLink() = 0;
     virtual void startPageRef() = 0;
     virtual void endPageRef(const char *,const char *) = 0;
-
-
-    virtual void writeLineNumber(const char *ref,const char *file,
-                                 const char *anchor,int lineNumber) = 0;
-    virtual void startCodeLine() = 0;
-    virtual void endCodeLine() = 0;
-    virtual void startCodeAnchor(const char *label) = 0;
-    virtual void endCodeAnchor() = 0;
-    virtual void startFontClass(const char *) = 0;
-    virtual void endFontClass() = 0;
-
-    virtual void startHtmlOnly() = 0;
-    virtual void endHtmlOnly() = 0;
-    virtual void startLatexOnly() = 0;
-    virtual void endLatexOnly() = 0;
-
-    /*! Writes an ASCII string to the output. This function should keep 
-     *  spaces visible, should break lines at a newline and should convert 
-     *  tabs to the right number of spaces.
-     */
-    virtual void codify(const char *s) = 0;
-    
+    virtual void startSubsection() = 0;
+    virtual void endSubsection() = 0;
+    virtual void startSubsubsection() = 0;
+    virtual void endSubsubsection() = 0;
 };
 
 /*! \brief Abstract output generator.
@@ -279,7 +247,7 @@ class BaseOutputDocInterface
 class OutputGenerator : public BaseOutputDocInterface
 {
   public:
-    enum OutputType { Html, Latex, Man, RTF, XML, DEF };
+    enum OutputType { Html, Latex, Man, RTF, XML, DEF, Perl };
 
     OutputGenerator();
     virtual ~OutputGenerator();
@@ -287,8 +255,6 @@ class OutputGenerator : public BaseOutputDocInterface
     ///////////////////////////////////////////////////////////////
     // generic generator methods
     ///////////////////////////////////////////////////////////////
-    virtual OutputGenerator *copy() = 0;
-    virtual void append(const OutputGenerator *) = 0;
     virtual void enable() = 0;
     virtual void disable() = 0;
     virtual void enableIf(OutputType o) = 0;
@@ -296,8 +262,6 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void disableIfNot(OutputType o) = 0;
     virtual bool isEnabled(OutputType o) = 0;
     virtual OutputGenerator *get(OutputType o) = 0;
-    void clear() { b.close(); a.resize(0); b.setBuffer(a); 
-                   b.open(IO_WriteOnly); t.setDevice(&b); }
     void startPlainFile(const char *name);
     void endPlainFile();
     QCString getContents() const;
@@ -305,12 +269,14 @@ class OutputGenerator : public BaseOutputDocInterface
     void pushGeneratorState();
     void popGeneratorState();
 
+    virtual void printDoc(DocNode *) = 0;
+
     ///////////////////////////////////////////////////////////////
     // structural output interface
     ///////////////////////////////////////////////////////////////
     virtual void startFile(const char *name,const char *manName,
-                           const char *title,bool ext) = 0;
-    virtual void writeFooter(int,bool) = 0;
+                           const char *title) = 0;
+    virtual void writeFooter() = 0;
     virtual void endFile() = 0;
     virtual void startIndexSection(IndexSections) = 0;
     virtual void endIndexSection(IndexSections) = 0;
@@ -325,9 +291,6 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endIndexKey()   = 0;
     virtual void startIndexValue(bool) = 0;
     virtual void endIndexValue(const char *,bool) = 0;
-    virtual void startAlphabeticalIndexList() = 0;
-    virtual void endAlphabeticalIndexList() = 0;
-    virtual void writeIndexHeading(const char *s) = 0;
     virtual void writeIndexItem(const char *ref,const char *file,
                                 const char *text) = 0;
     virtual void startGroupHeader() = 0;
@@ -340,8 +303,10 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endMemberSubtitle() = 0;
     virtual void startMemberList()  = 0;
     virtual void endMemberList()    = 0;
+    virtual void startAnonTypeScope(int) = 0;
+    virtual void endAnonTypeScope(int) = 0;
     virtual void startMemberItem(int) = 0;
-    virtual void endMemberItem(bool) = 0;
+    virtual void endMemberItem() = 0;
     virtual void startMemberGroupHeader(bool) = 0;
     virtual void endMemberGroupHeader() = 0;
     virtual void startMemberGroupDocs() = 0;
@@ -351,7 +316,7 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void insertMemberAlign() = 0;
     virtual void startMemberDoc(const char *,const char *,
                                 const char *,const char *) = 0;
-    virtual void endMemberDoc() = 0;
+    virtual void endMemberDoc(bool) = 0;
     virtual void startDoxyAnchor(const char *fName,const char *manName,
                                  const char *anchor,const char *name) = 0;
     virtual void endDoxyAnchor(const char *fileName,const char *anchor) = 0;
@@ -370,21 +335,24 @@ class OutputGenerator : public BaseOutputDocInterface
     virtual void endDotGraph(DotClassGraph &g) = 0;
     virtual void startInclDepGraph() = 0;
     virtual void endInclDepGraph(DotInclDepGraph &g) = 0;
+    virtual void startCallGraph() = 0;
+    virtual void endCallGraph(DotCallGraph &g) = 0;
     virtual void writeGraphicalHierarchy(DotGfxHierarchyTable &g) = 0;
-    virtual void startQuickIndexItem(const char *s,const char *l) = 0;
-    virtual void endQuickIndexItem() = 0;
+    //virtual void startQuickIndexItem(const char *s,const char *l) = 0;
+    //virtual void endQuickIndexItem() = 0;
+    virtual void writeQuickLinks(bool compact,HighlightedItem hli) = 0;
     virtual void startTextBlock(bool) = 0;
     virtual void endTextBlock() = 0;
     virtual void lastIndexPage() = 0;
     virtual void startMemberDocPrefixItem() = 0;
     virtual void endMemberDocPrefixItem() = 0;
-    virtual void startMemberDocName() = 0;
+    virtual void startMemberDocName(bool) = 0;
     virtual void endMemberDocName() = 0;
-    virtual void startParameterType(bool) = 0;
+    virtual void startParameterType(bool,const char *) = 0;
     virtual void endParameterType() = 0;
     virtual void startParameterName(bool) = 0;
-    virtual void endParameterName(bool,bool) = 0;
-    virtual void startParameterList() = 0;
+    virtual void endParameterName(bool,bool,bool) = 0;
+    virtual void startParameterList(bool) = 0;
     virtual void endParameterList() = 0;
 
   protected:
@@ -404,8 +372,8 @@ class OutputGenerator : public BaseOutputDocInterface
 
 /*! \brief Interface used for generating documentation.
  *
- *  This abstract class is used by the documentation block
- *  parser to generate the output for a specific format. 
+ *  This abstract class is used by several functions
+ *  to generate the output for a specific format. 
  *  This interface contains some state saving and changing
  *  functions for dealing with format specific output.
  */
@@ -417,7 +385,7 @@ class OutputDocInterface : public BaseOutputDocInterface
     /*! Create a new output generator. This can later by appended
      *  to the current one using append().
      */
-    virtual OutputDocInterface *clone() = 0;
+    //virtual OutputDocInterface *clone() = 0;
 
     /*! Disables all output formats except format \a o 
      *  (useful for OutputList only) 
@@ -446,7 +414,7 @@ class OutputDocInterface : public BaseOutputDocInterface
     /*! Appends the output generated by generator \a g to this
      *  generator.
      */ 
-    virtual void append(const OutputDocInterface *g) = 0;
+    //virtual void append(const OutputDocInterface *g) = 0;
 
     /*! Pushes the state of the current generator (or list of 
      *  generators) on a stack.
