@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: reflist.cpp,v 1.2 2001/03/19 19:27:41 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,19 +18,22 @@
 
 #include "reflist.h"
 
-/*! The one and only todo list */
-RefList todoList;
-/*! The test criteria list */
-RefList testList;
-/*! The bug list */
-RefList bugList;
-
-/*! Create a todo list */
-RefList::RefList() 
+/*! Create a list of items that are cross referenced with documentation blocks
+ *  @param listName String representing the name of the list.
+ *  @param pageTitle String representing the title of the list page.
+ *  @param secTitle String representing the title of the section.
+ */
+RefList::RefList(const char *listName,
+                 const char *pageTitle,
+                 const char *secTitle
+                )
 { 
   m_dict = 0;
   m_dictIterator = 0;
   m_id = 0;
+  m_listName = listName;
+  m_pageTitle = pageTitle;
+  m_secTitle = secTitle;
 }
 
 /*! Destroy the todo list. Currently not called! */
@@ -84,3 +87,19 @@ RefItem *RefList::getNextRefItem()
   return m_dictIterator ? m_dictIterator->operator++() : 0;
 }
 
+/*! Returns the name of the list as set in the constructor. */
+QCString RefList::listName() const
+{
+  return m_listName;
+}
+
+QCString RefList::pageTitle() const
+{
+  return m_pageTitle;
+}
+
+QCString RefList::sectionTitle() const
+{
+  return m_secTitle;
+}
+
