@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: reflist.h,v 1.2 2001/03/19 19:27:41 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,6 +21,7 @@
 
 #include "qtbc.h"
 #include <qintdict.h>
+//#include "outputgen.h"
 
 /*! This struct represents an item in the list of references. */
 struct RefItem
@@ -31,7 +32,16 @@ struct RefItem
   bool written;
 };
 
-/*! Singleton for the one and only RefList */
+/*! @brief List of cross-referenced items 
+ * 
+ *  This class represents a list of items that are put
+ *  at a certain point in the documentation by some special command 
+ *  and are collected in a list. The items cross-reference the 
+ *  documentation and the list.
+ *
+ *  Examples are the todo list, the test list and the bug list,
+ *  introduced by the \\todo, \\test, and \\bug commands respectively.
+ */
 class RefList
 {
   public:
@@ -39,18 +49,22 @@ class RefList
     RefItem *getRefItem(int todoItemId);
     RefItem *getFirstRefItem();
     RefItem *getNextRefItem();
+    QCString listName() const;
+    QCString pageTitle() const;
+    QCString sectionTitle() const;
 
-    RefList();
+    RefList(const char *listName,
+            const char *pageTitle,const char *secTitle
+           );
    ~RefList();
 
   private:
     int m_id;
+    QCString m_listName;
+    QCString m_pageTitle;
+    QCString m_secTitle;
     QIntDict<RefItem> *m_dict;
     QIntDictIterator<RefItem> *m_dictIterator;
 };
 
-extern RefList todoList;
-extern RefList testList;
-extern RefList bugList;
-
 #endif
