@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: rtfgen.h,v 1.13 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Parker Waechter & Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Parker Waechter & Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -31,9 +31,9 @@ class RTFGenerator : public OutputGenerator
     static void writeStyleSheetFile(QFile &f);
     static void writeExtensionsFile(QFile &file);
 
-    OutputGenerator *copy();
+    //OutputGenerator *copy();
     //OutputGenerator *clone() { return new RTFGenerator(*this); }
-    void append(const OutputGenerator *o);
+    //void append(const OutputGenerator *o);
     void enable() { active=TRUE; }
     void disable() { active=FALSE; }
     void enableIf(OutputType o)  { if (o==RTF) active=TRUE;  }
@@ -42,9 +42,10 @@ class RTFGenerator : public OutputGenerator
     bool isEnabled(OutputType o) { return (o==RTF && active); } 
     OutputGenerator *get(OutputType o) { return (o==RTF) ? this : 0; }
 
-    void startFile(const char *name,const char *manName,
-                   const char *title, bool external);
-    void writeFooter(int,bool) {}
+    void printDoc(DocNode *);
+
+    void startFile(const char *name,const char *manName,const char *title);
+    void writeFooter() {}
     void endFile();
     void clearBuffer();
     
@@ -70,11 +71,11 @@ class RTFGenerator : public OutputGenerator
     void endIndexValue(const char *,bool);
     void startItemList();
     void endItemList();
-    void startEnumList();
-    void endEnumList();
-    void startAlphabeticalIndexList() {}
-    void endAlphabeticalIndexList() {} 
-    void writeIndexHeading(const char *) {}
+    //void startEnumList();
+    //void endEnumList();
+    //void startAlphabeticalIndexList() {}
+    //void endAlphabeticalIndexList() {} 
+    //void writeIndexHeading(const char *) {}
     void writeIndexItem(const char *ref,const char *file,const char *name);
     void docify(const char *text);
     void codify(const char *text);
@@ -86,7 +87,7 @@ class RTFGenerator : public OutputGenerator
     void endTextLink();
     void startHtmlLink(const char *url);
     void endHtmlLink();
-    void writeMailLink(const char *url);
+    //void writeMailLink(const char *url);
     void startTypewriter() { t << "{\\f2 "; }
     void endTypewriter()   { t << "}";      }
     void startGroupHeader();
@@ -101,19 +102,23 @@ class RTFGenerator : public OutputGenerator
     void endMemberSubtitle(); 
     void startMemberList();
     void endMemberList();
+    void startAnonTypeScope(int) {}
+    void endAnonTypeScope(int) {}
     void startMemberItem(int);
-    void endMemberItem(bool);
+    void endMemberItem();
     //void memberGroupSpacing(bool) {}
     //void memberGroupSeparator() {}
     void insertMemberAlign() {}
 
-    void writeRuler(){RtfwriteRuler_thin();};
+    void writeRuler() { rtfwriteRuler_thin(); }
 	
     void writeAnchor(const char *fileName,const char *name);
     void startCodeFragment();
     void endCodeFragment();
-    void startPreFragment() { startCodeFragment(); }
-    void endPreFragment() { endCodeFragment(); }
+    //void startPreFragment()      { startCodeFragment(); }
+    //void endPreFragment()        { endCodeFragment(); }
+    //void startVerbatimFragment() { startCodeFragment(); }
+    //void endVerbatimFragment()   { endCodeFragment(); }
     void writeLineNumber(const char *,const char *,const char *,int l) { t << l << " "; }
     void startCodeLine() { col=0; }
     void endCodeLine() { lineBreak(); }
@@ -129,7 +134,7 @@ class RTFGenerator : public OutputGenerator
     void endDescItem();
     void lineBreak();
     void startMemberDoc(const char *,const char *,const char *,const char *);
-    void endMemberDoc();
+    void endMemberDoc(bool);
     void startDoxyAnchor(const char *,const char *,const char *,const char *);
     void endDoxyAnchor(const char *,const char *);
     void startCodeAnchor(const char *) {};
@@ -148,41 +153,43 @@ class RTFGenerator : public OutputGenerator
     void endCenter()        { t << "}"; }
     void startSmall()       { t << "{\\sub "; }
     void endSmall()         { t << "}"; }
-    void startSubscript()   { t << "{\\sub " << endl;}
-    void endSubscript()     { t << "}"; }
-    void startSuperscript() { t << "{\\super " << endl;}
-    void endSuperscript()   { t << "}"; }
-    void startTable(bool,int)   { /* not implemented */ }
-    void endTable(bool)         { /* not implemented */ }
-    void startCaption()     { /* not implemented */ }
-    void endCaption()       { /* not implemented */ }
-    void nextTableRow()     { /* not implemented */ }
-    void endTableRow()      { /* not implemented */ }
-    void nextTableColumn()  { /* not implemented */ }
-    void endTableColumn()   { /* not implemented */ }
-    void writeCopyright()    { t << "\251"; }
-    void writeQuote()        { t << "\""; }
-    void writeUmlaut(char c);
-    void writeAcute(char c);
-    void writeGrave(char c);
-    void writeCirc(char c);
-    void writeTilde(char c);
-    void writeRing(char c);
-    void writeSharpS()       { t << "\337"; }
-    void writeCCedil(char c);
+    //void startSubscript()   { t << "{\\sub " << endl;}
+    //void endSubscript()     { t << "}"; }
+    //void startSuperscript() { t << "{\\super " << endl;}
+    //void endSuperscript()   { t << "}"; }
+    //void startTable(bool,int);
+    //void endTable(bool);
+    //void startCaption();
+    //void endCaption();
+    //void nextTableRow();
+    //void endTableRow();
+    //void nextTableColumn();
+    //void endTableColumn();
+    //void writeCopyright()    { t << "\251"; }
+    //void writeQuote()        { t << "\""; }
+    //void writeUmlaut(char c);
+    //void writeAcute(char c);
+    //void writeGrave(char c);
+    //void writeCirc(char c);
+    //void writeTilde(char c);
+    //void writeRing(char c);
+    //void writeSharpS()       { t << "\337"; }
+    //void writeCCedil(char c);
 
     void startMemberDescription();
     void endMemberDescription();
     void startDescList(SectionTypes);
-    void endDescList();
-    void startParamList(ParamListTypes);
+    //void endDescList();
+    void startSimpleSect(SectionTypes,const char *,const char *,const char *);
+    void endSimpleSect();
+    void startParamList(ParamListTypes,const char *);
     void endParamList();
-    void endDescTitle();
+    //void endDescTitle();
     void writeDescItem();
-    void startSection(const char *,const char *,bool);
-    void endSection(const char *,bool);
-    void writeSectionRef(const char *,const char *,const char *,const char *);
-    void writeSectionRefItem(const char *,const char *,const char *);
+    void startSection(const char *,const char *,SectionInfo::SectionType);
+    void endSection(const char *,SectionInfo::SectionType);
+    //void writeSectionRef(const char *,const char *,const char *,const char *);
+    //void writeSectionRefItem(const char *,const char *,const char *);
     //void writeSectionRefAnchor(const char *,const char *,const char *);
     void addIndexItem(const char *,const char *);
     void startIndent();
@@ -195,14 +202,15 @@ class RTFGenerator : public OutputGenerator
     //void endColorFont()   {}
     void startPageRef();
     void endPageRef(const char *,const char *);
-    void startQuickIndexItem(const char *,const char *) {}
-    void endQuickIndexItem() {}
-    void writeFormula(const char *,const char *);
+    //void startQuickIndexItem(const char *,const char *) {}
+    //void endQuickIndexItem() {}
+    void writeQuickLinks(bool,HighlightedItem) {}
+    //void writeFormula(const char *,const char *);
     void writeNonBreakableSpace(int);
-    void startImage(const char *,const char *,bool);
-    void endImage(bool);
-    void startDotFile(const char *,bool);
-    void endDotFile(bool);
+    //void startImage(const char *,const char *,bool);
+    //void endImage(bool);
+    //void startDotFile(const char *,bool);
+    //void endDotFile(bool);
 	
     void startDescTable();
     void endDescTable();
@@ -215,6 +223,8 @@ class RTFGenerator : public OutputGenerator
     void endDotGraph(DotClassGraph &);
     void startInclDepGraph();
     void endInclDepGraph(DotInclDepGraph &);
+    void startCallGraph();
+    void endCallGraph(DotCallGraph &);
     void writeGraphicalHierarchy(DotGfxHierarchyTable &) {}
 
     void startMemberGroupHeader(bool);
@@ -230,22 +240,27 @@ class RTFGenerator : public OutputGenerator
 
     void startMemberDocPrefixItem() {}
     void endMemberDocPrefixItem() {}
-    void startMemberDocName() {}
+    void startMemberDocName(bool) {}
     void endMemberDocName() {}
-    void startParameterType(bool) {}
+    void startParameterType(bool,const char *) {}
     void endParameterType() {}
     void startParameterName(bool) {}
-    void endParameterName(bool,bool) {}
-    void startParameterList() {}
+    void endParameterName(bool,bool,bool) {}
+    void startParameterList(bool) {}
     void endParameterList() {}
 
     void startFontClass(const char *) {}
     void endFontClass() {}
 
-    void startHtmlOnly()  {}
-    void endHtmlOnly()    {}
-    void startLatexOnly() {}
-    void endLatexOnly()   {}
+    //void startHtmlOnly()  {}
+    //void endHtmlOnly()    {}
+    //void startLatexOnly() {}
+    //void endLatexOnly()   {}
+
+    //void startSectionRefList() {}
+    //void endSectionRefList() {}
+
+    void writeCodeAnchor(const char *) {}
 
     static bool preProcessFileInplace(const char *path,const char *name);
     
@@ -253,12 +268,12 @@ class RTFGenerator : public OutputGenerator
     RTFGenerator(const RTFGenerator &);
     RTFGenerator &operator=(const RTFGenerator &);
 
-    const char *Rtf_BList_DepthStyle();
-    const char *Rtf_CList_DepthStyle();
-    const char *Rtf_EList_DepthStyle();
-    const char *Rtf_LCList_DepthStyle();
-    const char *Rtf_DList_DepthStyle();
-    const char *Rtf_Code_DepthStyle();
+    const char *rtf_BList_DepthStyle();
+    const char *rtf_CList_DepthStyle();
+    const char *rtf_EList_DepthStyle();
+    const char *rtf_LCList_DepthStyle();
+    const char *rtf_DList_DepthStyle();
+    const char *rtf_Code_DepthStyle();
     void incrementIndentLevel();
     void decrementIndentLevel();
     int  col;
@@ -266,15 +281,18 @@ class RTFGenerator : public OutputGenerator
     bool m_bstartedBody;  // has startbody been called yet?
     int  m_listLevel; // // RTF does not really have a addative indent...manually set list level.
     bool m_omitParagraph; // should a the next paragraph command be ignored?
+    int  m_numCols; // number of columns in a table
+    QCString relPath;
 
     void beginRTFDocument();
     void beginRTFChapter();
     void beginRTFSection();
-    void RtfwriteRuler_doubleline() { t << "{\\pard\\widctlpar\\brdrb\\brdrdb\\brdrw15\\brsp20 \\adjustright \\par}" << endl; };
-    void RtfwriteRuler_emboss() { t << "{\\pard\\widctlpar\\brdrb\\brdremboss\\brdrw15\\brsp20 \\adjustright \\par}" << endl; };
-    void RtfwriteRuler_thick() { t << "{\\pard\\widctlpar\\brdrb\\brdrs\\brdrw75\\brsp20 \\adjustright \\par}" << endl; };
-    void RtfwriteRuler_thin() { t << "{\\pard\\widctlpar\\brdrb\\brdrs\\brdrw5\\brsp20 \\adjustright \\par}" << endl; };
-    void WriteRTFReference(const char *label);
+    void rtfwriteRuler_doubleline() { t << "{\\pard\\widctlpar\\brdrb\\brdrdb\\brdrw15\\brsp20 \\adjustright \\par}" << endl; };
+    void rtfwriteRuler_emboss() { t << "{\\pard\\widctlpar\\brdrb\\brdremboss\\brdrw15\\brsp20 \\adjustright \\par}" << endl; };
+    void rtfwriteRuler_thick() { t << "{\\pard\\widctlpar\\brdrb\\brdrs\\brdrw75\\brsp20 \\adjustright \\par}" << endl; };
+    void rtfwriteRuler_thin() { t << "{\\pard\\widctlpar\\brdrb\\brdrs\\brdrw5\\brsp20 \\adjustright \\par}" << endl; };
+    void writeRTFReference(const char *label);
+    char *getMultiByte(int c);
 };
 
 #endif
