@@ -1,9 +1,9 @@
 /******************************************************************************
  *
- * $Id: tagreader.cpp,v 1.2 2001/03/19 19:27:41 root Exp $
+ * 
  *
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -55,8 +55,8 @@ class TagMemberInfo
 class TagClassInfo
 {
   public:
-    enum Kind { Class, Struct, Union, Interface, Exception };
-    TagClassInfo() { bases=0, templateArguments=0; members.setAutoDelete(TRUE); }
+    enum Kind { Class, Struct, Union, Interface, Exception, Protocol, Category };
+    TagClassInfo() { bases=0, templateArguments=0; members.setAutoDelete(TRUE); isObjC=FALSE; }
    ~TagClassInfo() { delete bases; delete templateArguments; }
     QString name;
     QString filename;
@@ -65,6 +65,7 @@ class TagClassInfo
     QList<TagMemberInfo> members;
     QList<QString> *templateArguments;
     Kind kind;
+    bool isObjC;
 };
 
 /*! Container for namespace specific info that can be read from a tagfile */
@@ -91,11 +92,20 @@ class TagPackageInfo
     QStringList classList;
 };
 
+class TagIncludeInfo
+{
+  public:
+    QString id;
+    QString name;
+    QString text;
+    bool isLocal;
+};
+
 /*! Container for file specific info that can be read from a tagfile */
 class TagFileInfo
 {
   public:
-    TagFileInfo() { members.setAutoDelete(TRUE); }
+    TagFileInfo() { members.setAutoDelete(TRUE); includes.setAutoDelete(TRUE); }
     QString name;
     QString path;
     QString filename;
@@ -103,6 +113,7 @@ class TagFileInfo
     QList<TagMemberInfo> members;
     QStringList classList;
     QStringList namespaceList;
+    QList<TagIncludeInfo> includes;
 };
 
 /*! Container for group specific info that can be read from a tagfile */
@@ -176,12 +187,15 @@ class TagFileParser : public QXmlDefaultHandler
                                          m_endElementHandlers(17),
                                          m_tagName(tagName)
     {
+      m_startElementHandlers.setAutoDelete(TRUE);
+      m_endElementHandlers.setAutoDelete(TRUE);
     }
 
     void startCompound( const QXmlAttributes& attrib )
     {
       m_curString = "";
       QString kind = attrib.value("kind");
+      QString isObjC = attrib.value("objc");
       if (kind=="class")
       {
         m_curClass = new TagClassInfo;
@@ -212,6 +226,18 @@ class TagFileParser : public QXmlDefaultHandler
         m_curClass->kind = TagClassInfo::Exception;
         m_state = InClass;
       }
+      else if (kind=="protocol")
+      {
+        m_curClass = new TagClassInfo;
+        m_curClass->kind = TagClassInfo::Protocol;
+        m_state = InClass;
+      }
+      else if (kind=="category")
+      {
+        m_curClass = new TagClassInfo;
+        m_curClass->kind = TagClassInfo::Category;
+        m_state = InClass;
+      }
       else if (kind=="file")
       {
         m_curFile = new TagFileInfo;
@@ -241,6 +267,10 @@ class TagFileParser : public QXmlDefaultHandler
       {
         err("Error: Unknown compound attribute `%s' found!\n",kind.data());
       }
+      if (isObjC=="yes" && m_curClass)
+      {
+        m_curClass->isObjC = TRUE; 
+      }
     }
     void endCompound()
     {
@@ -295,7 +325,8 @@ class TagFileParser : public QXmlDefaultHandler
 
     void endMember()
     {
-      m_state = *m_stateStack.pop();
+      m_state = *m_stateStack.top();
+      m_stateStack.remove();
       switch(m_state)
       {
         case InClass:     m_curClass->members.append(m_curMember); break;
@@ -431,6 +462,26 @@ class TagFileParser : public QXmlDefaultHandler
         err("Error: Unexpected tag `base' found\n");
       }
     }
+    void startIncludes(const QXmlAttributes& attrib )
+    {
+      if (m_state==InFile && m_curFile)
+      {
+        m_curIncludes = new TagIncludeInfo;
+        m_curIncludes->id = attrib.value("id");
+        m_curIncludes->name = attrib.value("name");
+        m_curIncludes->isLocal = attrib.value("local")=="yes" ? TRUE : FALSE;
+        m_curFile->includes.append(m_curIncludes);
+      }
+      else
+      {
+        err("Error: Unexpected tag `includes' found\n");
+      }
+      m_curString="";
+    }
+    void endIncludes()
+    {
+      m_curIncludes->text = m_curString;
+    }
     void endTemplateArg()
     {
       if (m_state==InClass && m_curClass)
@@ -541,6 +592,7 @@ class TagFileParser : public QXmlDefaultHandler
       m_startElementHandlers.insert("name",        new StartElementHandler(this,&TagFileParser::startStringValue));
       m_startElementHandlers.insert("base",        new StartElementHandler(this,&TagFileParser::startBase));
       m_startElementHandlers.insert("filename",    new StartElementHandler(this,&TagFileParser::startStringValue));
+      m_startElementHandlers.insert("includes",    new StartElementHandler(this,&TagFileParser::startIncludes));
       m_startElementHandlers.insert("path",        new StartElementHandler(this,&TagFileParser::startStringValue));
       m_startElementHandlers.insert("anchor",      new StartElementHandler(this,&TagFileParser::startStringValue));
       m_startElementHandlers.insert("arglist",     new StartElementHandler(this,&TagFileParser::startStringValue));
@@ -560,6 +612,7 @@ class TagFileParser : public QXmlDefaultHandler
       m_endElementHandlers.insert("name",        new EndElementHandler(this,&TagFileParser::endName));
       m_endElementHandlers.insert("base",        new EndElementHandler(this,&TagFileParser::endBase));
       m_endElementHandlers.insert("filename",    new EndElementHandler(this,&TagFileParser::endFilename));
+      m_endElementHandlers.insert("includes",    new EndElementHandler(this,&TagFileParser::endIncludes));
       m_endElementHandlers.insert("path",        new EndElementHandler(this,&TagFileParser::endPath));
       m_endElementHandlers.insert("anchor",      new EndElementHandler(this,&TagFileParser::endAnchor));
       m_endElementHandlers.insert("arglist",     new EndElementHandler(this,&TagFileParser::endArglist));
@@ -612,6 +665,7 @@ class TagFileParser : public QXmlDefaultHandler
     }
     void dump();
     void buildLists(Entry *root);
+    void addIncludes();
     
   private:
     void buildMemberList(Entry *ce,QList<TagMemberInfo> &members);
@@ -631,6 +685,7 @@ class TagFileParser : public QXmlDefaultHandler
     TagGroupInfo              *m_curGroup;
     TagPageInfo               *m_curPage;
     TagMemberInfo             *m_curMember;
+    TagIncludeInfo            *m_curIncludes;
     QCString                   m_curString;
     QString                    m_tagName;
     State                      m_state;
@@ -752,6 +807,13 @@ void TagFileParser::dump()
       msg("    anchor: `%s'\n",md->anchor.data());
       msg("    arglist: `%s'\n",md->arglist.data());
     }
+
+    QListIterator<TagIncludeInfo> mii(fd->includes);
+    TagIncludeInfo *ii;
+    for (;(ii=mii.current());++mii)
+    {
+      msg("  includes id: %s name: %s\n",ii->id.data(),ii->name.data());
+    }
   }
 
   //============== GROUPS
@@ -816,36 +878,17 @@ void TagFileParser::addDocAnchors(Entry *e,QStrList &l)
   while (s)
   {
     QCString *anchorName = new QCString(s);
-    //if (anchorName->left(5)=="_todo")
-    //{
-    //  int todoItemId  = todoList.addRefItem();
-    //  char anchorLabel[12];
-    //  sprintf(anchorLabel,"_todo%06d",todoItemId);
-    //  RefItem *item = todoList.getRefItem(todoItemId);
-    //  item->listAnchor = anchorLabel;
-    //}
-    //else if (anchorName->left(5)=="_test")
-    //{
-    //  int testItemId  = testList.addRefItem();
-    //  char anchorLabel[12];
-    //  sprintf(anchorLabel,"_test%06d",testItemId);
-    //  RefItem *item = testList.getRefItem(testItemId);
-    //  item->listAnchor = anchorLabel;
-    //}
-    //else
-    //{
-      if (Doxygen::sectionDict.find(*anchorName)==0)
-      {
-        SectionInfo *si=new SectionInfo(*anchorName,*anchorName,
-            SectionInfo::Anchor,m_tagName);
-        Doxygen::sectionDict.insert(*anchorName,si);
-        e->anchors->append(anchorName);
-      }
-      else
-      {
-        err("Duplicate anchor %s found\n",anchorName->data());
-      }
-    //}
+    if (Doxygen::sectionDict.find(*anchorName)==0)
+    {
+      SectionInfo *si=new SectionInfo(e->fileName,*anchorName,*anchorName,
+          SectionInfo::Anchor,m_tagName);
+      Doxygen::sectionDict.insert(*anchorName,si);
+      e->anchors->append(si);
+    }
+    else
+    {
+      err("Duplicate anchor %s found\n",anchorName->data());
+    }
     s=l.next();
   }
 }
@@ -862,6 +905,7 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
     me->args       = tmi->arglist;
     if (!me->args.isEmpty())
     {
+      delete me->argList;
       me->argList = new ArgumentList;
       stringToArgumentList(me->args,me->argList);
     }
@@ -869,6 +913,10 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
     me->virt       = tmi->virt;
     me->stat       = tmi->isStatic;
     me->fileName   = ce->fileName;
+    if (ce->section == Entry::GROUPDOC_SEC)
+    {
+      me->groups->append(new Grouping(ce->name,Grouping::GROUPING_INGROUP));
+    }
     addDocAnchors(me,tmi->docAnchors);
     TagInfo *ti    = new TagInfo;
     ti->tagName    = m_tagName;
@@ -886,9 +934,14 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
     }
     else if (tmi->kind=="property")
     {
-      me->section = Entry::FUNCTION_SEC;
+      me->section = Entry::VARIABLE_SEC;
       me->mtype = Property;
     }
+    else if (tmi->kind=="event")
+    {
+      me->section = Entry::VARIABLE_SEC;
+      me->mtype = Event;
+    }
     else if (tmi->kind=="variable")
     {
       me->section = Entry::VARIABLE_SEC;
@@ -897,6 +950,7 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
     else if (tmi->kind=="typedef")
     {
       me->section = Entry::VARIABLE_SEC; //Entry::TYPEDEF_SEC;
+      me->type.prepend("typedef ");
       me->mtype = Method;
     }
     else if (tmi->kind=="enumeration")
@@ -922,6 +976,7 @@ void TagFileParser::buildMemberList(Entry *ce,QList<TagMemberInfo> &members)
     else if (tmi->kind=="friend")
     {
       me->section = Entry::FUNCTION_SEC;
+      me->type.prepend("friend ");
       me->mtype = Method;
     }
     else if (tmi->kind=="dcop")
@@ -969,6 +1024,8 @@ void TagFileParser::buildLists(Entry *root)
       case TagClassInfo::Union:     ce->section = Entry::UNION_SEC;     break;
       case TagClassInfo::Interface: ce->section = Entry::INTERFACE_SEC; break;
       case TagClassInfo::Exception: ce->section = Entry::EXCEPTION_SEC; break;
+      case TagClassInfo::Protocol:  ce->section = Entry::PROTOCOL_SEC;  break;
+      case TagClassInfo::Category:  ce->section = Entry::CATEGORY_SEC;  break;
     }
     ce->name     = tci->name;
     addDocAnchors(ce,tci->docAnchors);
@@ -976,14 +1033,20 @@ void TagFileParser::buildLists(Entry *root)
     ti->tagName  = m_tagName;
     ti->fileName = tci->filename;
     ce->tagInfo = ti;
+    ce->objc = tci->isObjC;
     // transfer base class list
     if (tci->bases)
     {
+      delete ce->extends;
       ce->extends = tci->bases; tci->bases = 0;
     }
     if (tci->templateArguments)
     {
-      if (ce->tArgLists==0) ce->tArgLists = new QList<ArgumentList>;
+      if (ce->tArgLists==0) 
+      {
+        ce->tArgLists = new QList<ArgumentList>;
+        ce->tArgLists->setAutoDelete(TRUE);
+      }
       ArgumentList *al = new ArgumentList;
       ce->tArgLists->append(al);
       
@@ -1018,7 +1081,11 @@ void TagFileParser::buildLists(Entry *root)
     
     QString fullName = m_tagName+":"+tfi->path+stripPath(tfi->name);
     fe->fileName = fullName;
-    FileDef *fd = new FileDef(m_tagName+":"+tfi->path,tfi->name,m_tagName);
+    //printf("new FileDef() filename=%s\n",tfi->filename.data());
+    FileDef *fd = new FileDef(m_tagName+":"+tfi->path,
+                              tfi->name,m_tagName,
+                              tfi->filename
+                             );
     FileName *mn;
     if ((mn=Doxygen::inputNameDict->find(tfi->name)))
     {
@@ -1073,8 +1140,8 @@ void TagFileParser::buildLists(Entry *root)
   }
 
   // build group list, but only if config file says to include it
-  if (Config_getBool("EXTERNAL_GROUPS")) 
-  {
+  //if (Config_getBool("EXTERNAL_GROUPS")) 
+  //{
     TagGroupInfo *tgi = m_tagFileGroups.first();
     while (tgi)
     {
@@ -1092,7 +1159,7 @@ void TagFileParser::buildLists(Entry *root)
       root->addSubEntry(ge);
       tgi = m_tagFileGroups.next();
     }
-  }
+  //}
 
   // build page list
   TagPageInfo *tpi = m_tagFilePages.first();
@@ -1113,6 +1180,53 @@ void TagFileParser::buildLists(Entry *root)
   }
 }
 
+void TagFileParser::addIncludes()
+{
+  TagFileInfo *tfi = m_tagFileFiles.first();
+  while (tfi)
+  {
+    //printf("tag file tagName=%s path=%s name=%s\n",m_tagName.data(),tfi->path.data(),tfi->name.data());
+    FileName *fn = Doxygen::inputNameDict->find(tfi->name);
+    if (fn)
+    {
+      //printf("found\n");
+      FileNameIterator fni(*fn);
+      FileDef *fd;
+      for (;(fd=fni.current());++fni)
+      {
+        //printf("input file path=%s name=%s\n",fd->getPath().data(),fd->name().data());
+        if (fd->getPath()==QCString(m_tagName+":"+tfi->path))
+        {
+          //printf("found\n");
+          QListIterator<TagIncludeInfo> mii(tfi->includes);
+          TagIncludeInfo *ii;
+          for (;(ii=mii.current());++mii)
+          {
+            //printf("ii->name=`%s'\n",ii->name.data());
+            FileName *ifn = Doxygen::inputNameDict->find(ii->name);
+            ASSERT(ifn!=0);
+            if (ifn)
+            {
+              FileNameIterator ifni(*ifn);
+              FileDef *ifd;
+              for (;(ifd=ifni.current());++ifni)
+              {
+                //printf("ifd->getOutputFileBase()=%s ii->id=%s\n",
+                //        ifd->getOutputFileBase().data(),ii->id.data());
+                if (ifd->getOutputFileBase()==QCString(ii->id))
+                {
+                  fd->addIncludeDependency(ifd,ii->text,ii->isLocal);
+                }
+              }
+            }
+          }
+        }
+      } 
+    }
+    tfi = m_tagFileFiles.next();
+  }
+}
+
 void parseTagFile(Entry *root,const char *fullName,const char *tagName)
 {
   QFileInfo fi(fullName);
@@ -1126,5 +1240,7 @@ void parseTagFile(Entry *root,const char *fullName,const char *tagName)
   reader.setErrorHandler( &errorHandler );
   reader.parse( source );
   handler.buildLists(root);
+  handler.addIncludes();
 }
 
+
