@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: translator_cn.h,v 1.3 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -495,6 +495,8 @@ class TranslatorChinese : public TranslatorAdapter_1_2_13
         case ClassDef::Struct: result+="ṹ"; break;
         case ClassDef::Union:  result+=""; break;
         case ClassDef::Interface:  result+="ӿ"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="쳣"; break;
       }
       result+="ο";
@@ -659,6 +661,8 @@ class TranslatorChinese : public TranslatorAdapter_1_2_13
         case ClassDef::Struct:     result+="ṹ"; break;
         case ClassDef::Union:      result+=""; break;
         case ClassDef::Interface:  result+="ӿ"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="쳣"; break;
       }
       result+="ĵļɣ";
