@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -20,18 +20,9 @@
 
 
 // The first translation from English to Czech was started by
-// Vlastimil Havran. He wrote:
-// 
-//   "In the cases where are more translations possible I hope
-//    that I have selected the most reasonable text. If not, please,
-//    mail the comments and text proposals to
-//
-//    havran@fel.cvut.cz, 1999/04/11"
-//    
-// The prototype version of Czech strings with diacritics was
-// implemented by Petr Prikryl (prikrylp@skil.cz), 2000/06/20. As
-// Vlastimil was quite busy at the time, he agreed that Petr be the
-// maintainer for a while (from 2000/06/20).
+// Vlastimil Havran (1999--2000). The prototype version of Czech strings 
+// with diacritics was implemented by Petr Prikryl (prikrylp@skil.cz), 
+// 2000/06/20. Vlastimil agreed that Petr be the new maintainer.
 //
 // Updates:
 // --------
@@ -114,7 +105,7 @@
 //    use the approach.
 //  - Removed the obsolete method trFiles().
 //         
-// 2001/05/18
+// 2001/05/18 
 //  - The trRelatedPagesDescription() content updated to fit 
 //    the context better.
 //  - Implemented new method trAuthor(params).
@@ -125,19 +116,17 @@
 //  - The trReimplementedInList() updated to fit the context better.
 //  - The trReimplementedFromList() updated to fit the context better.
 //
-// 2001/07/16
-//  - trClassDocumentation() updated as in the English translator.
-//    
-// 2001/11/06
-//  - trReferences() implemented.
-//    
-// 2002/01/23
-//  - trImplementedFromList() and trImplementedInList() implemented
-//    (new since 1.2.13)
-//  
-// 2002/03/05
-//  - ... forgot to replace TranslatorAdapter... base class by Translator.
-//
+// 2001/07/16 - trClassDocumentation() updated as in the English translator.
+// 2001/11/06 - trReferences() implemented.
+// 2002/01/23 - Two new methods "since 1.2.13" implemented.
+// 2002/03/05 - ... forgot to replace TranslatorAdapter... by Translator.
+// 2002/07/08 (my birthday! ;) - The new trRTFTableOfContents() implemented.
+// 2002/07/29 - The new trDeprecatedList() implemented.
+// 2002/10/15 - The new trEvents() and trEventDocumentation() implemented.
+// 2003/04/28 - Five new methods "since 1.3" implemented.
+// 2003/06/10 - Two new methods "since 1.3.1" implemented.
+// 2003/08/13 - Four new "since 1.3.3" implemented. 
+
 // Todo
 // ----
 //  - The trReimplementedFromList() should pass the kind of the 
@@ -156,7 +145,6 @@
 // Windows version. The version which does not call the function is
 // probably slightly faster.
 
-
 class TranslatorCzech : public Translator
 {
   private:
@@ -165,11 +153,14 @@ class TranslatorCzech : public Translator
      */
     inline QCString decode(const QCString & sInput)
     { 
-#ifdef _WIN32
-      return sInput;
-#else
-      return Win1250ToISO88592(sInput);
-#endif
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
+        return sInput;
+      }
+      else
+      {
+        return Win1250ToISO88592(sInput);
+      }
     }
     
   public:
@@ -184,11 +175,14 @@ class TranslatorCzech : public Translator
     /*! return the language charset. This will be used for the HTML output */
     virtual QCString idLanguageCharset()
     {
-#ifdef _WIN32
+      if (Config_getBool("USE_WINDOWS_ENCODING"))
+      {
         return "windows-1250";
-#else
+      }
+      else
+      {
         return "iso-8859-2";
-#endif
+      }
     }
 
     // --- Language translation methods -------------------
@@ -769,6 +763,8 @@ class TranslatorCzech : public Translator
         case ClassDef::Struct:     result+="struktury "; break;
         case ClassDef::Union:      result+="unie "; break;
         case ClassDef::Interface:  result+="rozhran "; break;
+        case ClassDef::Protocol:   result+="protocol "; break; // translate me!
+        case ClassDef::Category:   result+="category "; break; // translate me!
         case ClassDef::Exception:  result+="vjimky "; break;
       }
       result+=clName;
@@ -939,6 +935,8 @@ class TranslatorCzech : public Translator
         case ClassDef::Struct:     result+="tuto strukturu (struct)"; break;
         case ClassDef::Union:      result+="tuto unii (union)"; break;
         case ClassDef::Interface:  result+="toto rozhran"; break;
+        case ClassDef::Protocol:   result+="protocol "; break; // translate me!
+        case ClassDef::Category:   result+="category "; break; // translate me!
         case ClassDef::Exception:  result+="tuto vjimku"; break;
       }
       result+=" byla generovna z ";
@@ -1502,9 +1500,156 @@ class TranslatorCzech : public Translator
      */
     virtual QCString trImplementedInList(int numEntries)
     {
-      return "Implementovno v "+trWriteList(numEntries)+".";
+      return decode("Implementovno v "+trWriteList(numEntries)+".");
     }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.16
+//////////////////////////////////////////////////////////////////////////
+
+    /*! used in RTF documentation as a heading for the Table
+     *  of Contents.
+     */
+    virtual QCString trRTFTableOfContents()
+    {
+      return decode("Obsah");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.17
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as the header of the list of item that have been 
+     *  flagged deprecated 
+     */
+    virtual QCString trDeprecatedList()
+    {
+      return decode("Seznam zastaralch prvk");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for declaration section of the events found in 
+     * a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return decode("Udlosti");
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return decode("Dokumentace udlost");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a heading for a list of Java class types with package scope.
+     */
+    virtual QCString trPackageTypes()
+    { 
+      return decode("Typy v balku");
+    }
+    /*! Used as a heading for a list of Java class functions with package 
+     * scope. 
+     */
+    virtual QCString trPackageMembers()
+    { 
+      return decode("Funkce v balku");
+    }
+    /*! Used as a heading for a list of static Java class functions with 
+     *  package scope.
+     */
+    virtual QCString trStaticPackageMembers()
+    { 
+      return decode("Statick funkce v balku");
+    }
+    /*! Used as a heading for a list of Java class variables with package 
+     * scope.
+     */
+    virtual QCString trPackageAttribs()
+    { 
+      return decode("Atributy balku");
+    }
+    /*! Used as a heading for a list of static Java class variables with 
+     * package scope.
+     */
+    virtual QCString trStaticPackageAttribs()
+    { 
+      return decode("Statick atributy balku");
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used in the quick index of a class/file/namespace member list page 
+     *  to link to the unfiltered list of all members.
+     */
+    virtual QCString trAll()
+    {
+      return decode("Ve");
+    }
+    /*! Put in front of the call graph for a function. */
+    virtual QCString trCallGraph()
+    {
+      return decode("Graf voln tto funkce:");
+    }
+   
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! When the search engine is enabled this text is put in the index 
+     *  of each page before the search field. 
+     */
+    virtual QCString trSearchForIndex()
+    {
+      return decode("Vyhledat");
+    }
+    /*! This string is used as the title for the page listing the search
+     *  results.
+     */
+    virtual QCString trSearchResultsTitle()
+    {
+      return decode("Vsledky vyhledvn");
+    }
+    /*! This string is put just before listing the search results. The
+     *  text can be different depending on the number of documents found.
+     *  Inside the text you can put the special marker $num to insert
+     *  the number representing the actual number of search results.
+     *  The @a numDocuments parameter can be either 0, 1 or 2, where the 
+     *  value 2 represents 2 or more matches. HTML markup is allowed inside
+     *  the returned string.
+     */
+    virtual QCString trSearchResults(int numDocuments)
+    {
+      if (numDocuments==0)
+      {
+        return decode("Lituji. Vaemu dotazu neodpovd dn dokument.");
+      }
+      else if (numDocuments==1)
+      {
+        return decode("Nalezen jedin dokument, kter vyhovuje vaemu dotazu.");
+      }
+      else 
+      {
+        return decode("Nalezeno <b>$num</b> dokument, kter vyhovuj vaemu "
+                      "dotazu. Nejlpe odpovdajc dokumenty jsou zobrazeny "
+                      "jako prvn.");
+      }
+    }
+    /*! This string is put before the list of matched words, for each search 
+     *  result. What follows is the list of words that matched the query.
+     */
+    virtual QCString trSearchMatches()
+    {
+      return decode("Nalezen slova:");
+    }
 };
 
 #endif // TRANSLATOR_CZ_H
