@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: translator_nl.h,v 1.22 2001/03/19 19:27:42 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -88,13 +88,13 @@ class TranslatorDutch : public Translator
     QCString trClassHierarchy()
     { return "Klasse Hi&euml;rarchie"; }
     QCString trCompoundList()
-    { return "Compound Lijst"; }
+    { return "Klasse Lijst"; }
     QCString trFileList()
     { return "File Lijst"; }
     QCString trHeaderFiles()
     { return "Header Lijst"; }
     QCString trCompoundMembers()
-    { return "Compound Members"; }
+    { return "Klasse Members"; }
     QCString trFileMembers()
     { return "File members"; }
     QCString trRelatedPages()
@@ -123,7 +123,7 @@ class TranslatorDutch : public Translator
       QCString result="Hieronder volgt de lijst met alle ";
       if (!extractAll) result+="gedocumenteerde ";
       result+="klasse members met links naar ";
-      if (extractAll) result+="de klasse documentatie voor elke member:";
+      if (!extractAll) result+="de klasse documentatie voor elke member:";
       else result+="de klassen waartoe ze behoren:";
       return result;
     }
@@ -154,7 +154,7 @@ class TranslatorDutch : public Translator
     QCString trHierarchicalIndex()
     { return "Hi&euml;rarchische Index"; }
     QCString trCompoundIndex()
-    { return "Compound Index"; }
+    { return "Klasse Index"; }
     QCString trFileIndex() 
     { return "File Index"; }
     QCString trModuleDocumentation()
@@ -199,7 +199,7 @@ class TranslatorDutch : public Translator
     QCString trVariableDocumentation()
     { return "Documentatie van variabelen"; }
     QCString trCompounds()
-    { return "Compounds"; }
+    { return "Klassen"; }
     QCString trGeneratedAt(const char *date,const char *projName)
     { 
       QCString result=(QCString)"Gegenereerd op "+date;
@@ -272,7 +272,7 @@ class TranslatorDutch : public Translator
                                  bool isTemplate)
       // used as the title of the HTML page of a class/struct/union
     {
-      QCString result=(QCString)clName+" ";
+      QCString result=(QCString)clName;
       if (isTemplate) result+=" Template";
       switch(compType)
       {
@@ -280,6 +280,8 @@ class TranslatorDutch : public Translator
         case ClassDef::Struct: result+=" Struct"; break;
         case ClassDef::Union:  result+=" Union"; break;
         case ClassDef::Interface:  result+=" Interface"; break;
+        case ClassDef::Protocol:  result+=" Protocol"; break;
+        case ClassDef::Category:  result+=" Category"; break;
         case ClassDef::Exception:  result+=" Exception"; break;
       }
       result+=" Referentie";
@@ -426,6 +428,8 @@ class TranslatorDutch : public Translator
         case ClassDef::Struct:     result+="struct"; break;
         case ClassDef::Union:      result+="union"; break;
         case ClassDef::Interface:  result+="interface"; break;
+        case ClassDef::Protocol:   result+="protocol"; break;
+        case ClassDef::Category:   result+="category"; break;
         case ClassDef::Exception:  result+="exception"; break;
       }
       result+=" is gegenereerd op grond van de volgende file";
@@ -703,9 +707,12 @@ class TranslatorDutch : public Translator
         "relatie tussen twee klassen.\n"
         "<li>Een donkergroene pijl wordt gebruikt voor protected inheritance.\n"
         "<li>Een donkerrode pijl wordt gebruikt voor private inheritance.\n"
-        "<li>Een paars gestreepte pijl wordt gebruikt indien een klasse bevat is of gebruikt wordt "
+        "<li>Een paars gestippelde pijl wordt gebruikt indien een klasse bevat is of gebruikt wordt "
         "door een andere klasse. De pijl is gelabeled met de variable(n) "
         "die toegang geven tot de aangewezen klasse of structure. \n"
+        "<li>Een geel gestippelde pijl wordt gebruikt om een relatie tussen een \n"
+        "template instantie en een template klasse aan te geven. De pijl is gelabeld met \n"
+        "template parameters van de instantie.\n"
         "</ul>\n";
     }
     /*! text for the link to the legend page */
@@ -975,6 +982,10 @@ class TranslatorDutch : public Translator
       return "Gebruikt";
     }
 
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.13
+//////////////////////////////////////////////////////////////////////////
+
     /*! used in member documentation blocks to produce a list of 
      *  members that are implemented by this one.
      */
@@ -990,6 +1001,149 @@ class TranslatorDutch : public Translator
     {
       return "Wordt ge&iuml;mplementeerd door "+trWriteList(numEntries)+".";
     }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.16
+//////////////////////////////////////////////////////////////////////////
+
+    virtual QCString trRTFTableOfContents()
+    { return "Inhoudsopgave"; }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.17
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as the header of the list of item that have been 
+     *  flagged deprecated 
+     */
+    virtual QCString trDeprecatedList()
+    {
+      return "Deprecated Lijst";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.18
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a header for a section of events found in a C# program
+     */
+    virtual QCString trEvents()
+    {
+      return "Events";
+    }
+    /*! Header used for the documentation section of a class' events. */
+    virtual QCString trEventDocumentation()
+    {
+      return "Event Documentatie";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used as a heading for a list of Java class types with package scope.
+     */
+    virtual QCString trPackageTypes()
+    { 
+      return "Package Types";
+    }
+    /*! Used as a heading for a list of Java class functions with package 
+     * scope. 
+     */
+    virtual QCString trPackageMembers()
+    { 
+      return "Package Functies";
+    }
+    /*! Used as a heading for a list of static Java class functions with 
+     *  package scope.
+     */
+    virtual QCString trStaticPackageMembers()
+    { 
+      return "Statische Package Functies";
+    }
+    /*! Used as a heading for a list of Java class variables with package 
+     * scope.
+     */
+    virtual QCString trPackageAttribs()
+    { 
+      return "Package Attributen";
+    }
+    /*! Used as a heading for a list of static Java class variables with 
+     * package scope.
+     */
+    virtual QCString trStaticPackageAttribs()
+    { 
+      return "Statische Package Attributen";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.1
+//////////////////////////////////////////////////////////////////////////
+
+    /*! Used in the quick index of a class/file/namespace member list page 
+     *  to link to the unfiltered list of all members.
+     */
+    virtual QCString trAll()
+    {
+      return "Alle";
+    }
+    /*! Put in front of the call graph for a function. */
+    virtual QCString trCallGraph()
+    {
+      return "Hier is de call graaf voor deze functie:";
+    }
+
+//////////////////////////////////////////////////////////////////////////
+// new since 1.3.3
+//////////////////////////////////////////////////////////////////////////
+
+    /*! When the search engine is enabled this text is put in the index 
+     *  of each page before the search field. 
+     */
+    virtual QCString trSearchForIndex()
+    {
+      return "Zoek naar";
+    }
+    /*! This string is used as the title for the page listing the search
+     *  results.
+     */
+    virtual QCString trSearchResultsTitle()
+    {
+      return "Zoek Resultaten";
+    }
+    /*! This string is put just before listing the search results. The
+     *  text can be different depending on the number of documents found.
+     *  Inside the text you can put the special marker $num to insert
+     *  the number representing the actual number of search results.
+     *  The @a numDocuments parameter can be either 0, 1 or 2, where the 
+     *  value 2 represents 2 or more matches. HTML markup is allowed inside
+     *  the returned string.
+     */
+    virtual QCString trSearchResults(int numDocuments)
+    {
+      if (numDocuments==0)
+      {
+        return "Helaas, er zijn geen documenten gevonden die aan de zoekopdracht voldoen.";
+      }
+      else if (numDocuments==1)
+      {
+        return "Er is <b>1</b> document gevonden dat aan de zoekopdracht voldoet.";
+      }
+      else 
+      {
+        return "Er zijn <b>$num</b> documenten gevonden die aan de zoekopdracht voldoen. "
+               "De beste resultaten worden eerst getoond.";
+      }
+    }
+    /*! This string is put before the list of matched words, for each search 
+     *  result. What follows is the list of words that matched the query.
+     */
+    virtual QCString trSearchMatches()
+    {
+      return "Gevonden:";
+    }
+
+    
 };
 
 #endif
