@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: translator_ro.h,v 1.3 2001/03/19 19:27:42 root Exp $
+ * 
  *
- * Copyright (C) 1997-2002 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -42,7 +42,7 @@
 #define TRANSLATOR_RO_H
 
 
-class TranslatorRomanian: public Translator
+class TranslatorRomanian: public TranslatorAdapter_1_2_16
 {
   public:
 
@@ -167,7 +167,7 @@ class TranslatorRomanian: public Translator
     // quick reference sections
 
     /*! This is put above each page as a link to the list of all groups of 
-     *  compounds or files (see the \group command).
+     *  compounds or files (see the \\group command).
      */
     virtual QCString trModules()
     { return "Module"; }
@@ -287,7 +287,7 @@ class TranslatorRomanian: public Translator
         result+="din toate clasele ";
       }
       result+=", cu legturi ctre ";
-      if (extractAll) 
+      if (!extractAll) 
       {
         if (Config_getBool("OPTIMIZE_OUTPUT_FOR_C"))
         {
@@ -659,6 +659,8 @@ class TranslatorRomanian: public Translator
         case ClassDef::Struct:     result+="structura"; break;
         case ClassDef::Union:      result+="uniunea"; break;
         case ClassDef::Interface:  result+="interfaa"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="excepia"; break;
       }
 	  if (isTemplate) result+=" (Template) ";
@@ -819,6 +821,8 @@ class TranslatorRomanian: public Translator
         case ClassDef::Struct:     result+="structur"; break;
         case ClassDef::Union:      result+="uniune"; break;
         case ClassDef::Interface:  result+="interfa"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="excepie"; break;
       }
       result+=" a fost generat din fiier";
