@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: translator_si.h,v 1.1 2000/07/22 17:04:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2000 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,7 +21,7 @@
 #define TRANSLATOR_SI_H
 
 
-class TranslatorSlovene : public TranslatorAdapter_1_2_13
+class TranslatorSlovene : public TranslatorAdapter_1_2_16
 {
   public:
     QCString idLanguage()
@@ -274,6 +274,8 @@ class TranslatorSlovene : public TranslatorAdapter_1_2_13
         case ClassDef::Struct: result+=" Struktura "; break;
         case ClassDef::Union:  result+=" Mnoica "; break;
         case ClassDef::Interface:  result+=" IDL vmesnik "; break;
+        case ClassDef::Protocol:   result+=" protocol "; break; // translate me!
+        case ClassDef::Category:   result+=" category "; break; // translate me!
         case ClassDef::Exception:  result+=" IDL prekinitev "; break;
       }
 	  result += (QCString)clName;
@@ -420,6 +422,8 @@ class TranslatorSlovene : public TranslatorAdapter_1_2_13
         case ClassDef::Struct:     result+="strukture"; break;
         case ClassDef::Union:      result+="unije"; break;
         case ClassDef::Interface:  result+="vmesnika (interface)"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; // translate me!
+        case ClassDef::Category:   result+="category"; break; // translate me!
         case ClassDef::Exception:  result+="prekinitve (exception)"; break;
       }
       result+=" je zgrajen na podlagi naslednj";
@@ -991,6 +995,26 @@ class TranslatorSlovene : public TranslatorAdapter_1_2_13
     {
       return "Reference";
     }
+//////////////////////////////////////////////////////////////////////////
+// new since 1.2.13
+//////////////////////////////////////////////////////////////////////////
+
+    /*! used in member documentation blocks to produce a list of 
+     *  members that are implemented by this one.
+     */
+    virtual QCString trImplementedFromList(int numEntries)
+    {
+      return "Implementira "+trWriteList(numEntries)+".";
+    }
+
+    /*! used in member documentation blocks to produce a list of
+     *  all members that implement this abstract member.
+     */
+    virtual QCString trImplementedInList(int numEntries)
+    {
+      return "Implementirano v "+trWriteList(numEntries)+".";
+    }
+
 };
 
 #endif
